/*==============================================================================
|                               SAS SCENARIO FILE
|
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  UNIX/WINDOWS
================================================================================
| NAME:            | SAS10220
|                  |
| CATEGORY:        | Wire Activity
|                  |
| SEVERITY:        | Scenario
|                  |
| FREQUENCY:       | Daily
|                  |
| DURATION:        | 1 day
|                  |
| SUBJECT:         | Customer
|                  |
| PREP FILE:       | PARTY_TRANSACTIONS
|                  |
| HEADER FILE:     | party_header
|                  |
| TRIGGER:         | Yes
|                  |
| DAYS CONSIDERED: | Business
================================================================================

SHORT DESCRIPTION
-----------------
Large Incoming Wires by Risk Classification

DESCRIPTION
-----------
A customer receives large amounts of wired funds. This may involve one or more
transactions that occur over one or more days.

BUSINESS NOTES
--------------
This scenario calculates daily wire-in totals for a customer and records the
maximum wire-in transaction amount. If a customer exceeds the thresholds, a
match is found. This scenario can be used for different objectives depending on
how its parameters are combined and set. The thresholds of interest include the
following: (a) minimum aggregate amount, (b) number of business days in an
interval, (c) minimum amount for current day, (d) minimum aggregate amount of
any day during the interval, and (e) minimum amount of the largest single
transaction during the interval. This scenario considers a customer's risk
classification, whereas SAS10020 does not. This scenario considers business
days.

TECHNICAL NOTES
---------------
Not Applicable

MESSAGE
-------
r10220_threshold_message = %str("Total Wire Deposits = #$ #1; Wire Count = #2");

SAMPLE MESSAGE
--------------
Total Wire Deposits = USD 139,123; Wire Count = 3

================================================================================
PARAMETERS

Name                              Type                Default Value
---------------------------------|-------------------|--------------------------
 p10220_account_type_desc        | Character List    | 'P'
 p10220_min_amount               | Numeric Constant  | 0
 p10220_min_cr_total             | Numeric Constant  | 0
 p10220_min_total                | Numeric Constant  | 100000
 p10220_min_total_cr_today       | Numeric Constant  | 0
 p10220_primary_medium_desc      | Character List    | 'WIRE'
 p10220_risk_classification      | Numeric List      | 3
 p10220_num_days                 | Numeric Constant  | 1
 p10220_status_desc              | Character List    | 'SUCCESS'
 p10220_transaction_cdi_code     | Character List    | 'C'

Descriptions
------------
- p10220_account_type_desc
List denoting valid types of accounts

- p10220_min_amount
Minimum amount of the single largest transaction during the interval

- p10220_min_cr_total
Minimum aggregate amount of any day in a multi-day interval

- p10220_min_total
Minimum aggregate credit amount

- p10220_min_total_cr_today
Minimum aggregate amount for current day. This parameter should be set to
zero for frequency/interval combinations without overlap, eg. weekly
frequency

- p10220_primary_medium_desc
List denoting valid primary mediums

- p10220_risk_classification
List denoting risk classifications for stratification

- p10220_num_days
Number of business days in interval

- p10220_status_desc
List denoting completed transactions

- p10220_transaction_cdi_code
List denoting credit activities

================================================================================

================================================================================
Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
- All rights reserved
==============================================================================*/
%let this_macro_name = &sysmacroname;  /* save this macro name */
/*----------------------------------------------------------------------------*/

retain p10220_begin_date;

/* This init section should be placed before any goto statements.  This section of
   code will only be run when processing the first BY group of the prep data set.
   Any goto statements placed before it may prevent the init code from being run. */
if run_scenario_init eq 1 then do;
  if &p10220_num_days = 0 then p10220_begin_date = rundate - day(rundate) + 1;
  else if &p10220_num_days > 0 then p10220_begin_date = input(put(rundate_number-&p10220_num_days+1,num_to_date.),8.);
end;

/* Skip parties that do not have transactions on the current day, comment out if monthly frequency */
if date_key{n} < rundate then goto &sysmacroname.;

p10220_agg_amt = 0; p10220_largest_amt = 0;
p10220_largest_day_amt = 0; p10220_tran_count = 0;
p10220_daily_agg = 0; p10220_today_agg = 0;
k = 0;

do i = n to 1 by -1 while (date_key{i} >= p10220_begin_date);
    if (p10220_begin_date <= date_key{i} <= rundate) and
       upcase(account_type_desc{i}) in (&p10220_account_type_desc) and
       upcase(primary_medium_desc{i}) in (&p10220_primary_medium_desc) and
       upcase(transaction_cdi_code{i}) in (&p10220_transaction_cdi_code) and
       upcase(status_desc{i}) in (&p10220_status_desc) and
       risk_classification in (&p10220_risk_classification)
    then do;
        p10220_date_key_num = input(put(date_key{i},date_to_num.),8.);
        temp_date_key = date_key{i};
        do while (p10220_date_key_num = -1);
            temp_date_key = temp_date_key + 1;
            p10220_date_key_num = input(put(temp_date_key,date_to_num.),8.);
        end;
        p10220_agg_amt = p10220_agg_amt + currency_amount{i};
        %fcf_save_transaction_key (calling_macro=&this_macro_name);
        p10220_tran_count = p10220_tran_count + 1;
        j = rundate_number-p10220_date_key_num + 1;
        if j > k then do;
            p10220_daily_agg = currency_amount{i};
            k = j;
        end;
        else if j = k then do;
            p10220_daily_agg = p10220_daily_agg + currency_amount{i};
        end;
        if j = 1 then p10220_today_agg = p10220_daily_agg;
        p10220_largest_amt = max(p10220_largest_amt, currency_amount{i});
        p10220_largest_day_amt = max(p10220_largest_day_amt, p10220_daily_agg);
    end;
end;

/* Compare party values to parameters for a match */
if p10220_today_agg >= &p10220_min_total_cr_today and
   p10220_agg_amt >= &p10220_min_total and
   p10220_largest_amt >= &p10220_min_amount and
   p10220_largest_day_amt >= &p10220_min_cr_total
then do;
    actual_values_text  =  tranwrd(&r10220_threshold_message,'#1',kstrip(put(p10220_agg_amt,nlnum32.)) );
    actual_values_text  =  tranwrd(actual_values_text,'#2',kstrip(put(p10220_tran_count,nlnum32.)) );
    actual_values_text  =  tranwrd(actual_values_text,'#$', "&currency_code" );
    output &alert_fname;
end;

/* EXIT */
&sysmacroname.:;


/* ========================================================================== */
/* END SCENARIO - SAS10220                                                    */
/* ========================================================================== */
