/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* validated against FinCEN rules while loading sample data for regulatory     */
/* reports.  The validation rules should be modified to meet your              */
/* organizational needs.  These are provided as a sample.                      */
/* *****************************************************************************/

/*-------------------------------------------------------------------
 * Copyright (c) 2020 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%MACRO VALIDATE_FINCEN_FILER;

/*
  Validate Part III Filer Information Section 
*/

/* A00: Schema Validation Failure. */
error_code="A00";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code ne "" and index("BCDEF",strip(institution_prim_fed_reg_code))=0  then 
do;
   error_record_type="batch";
   output;
   error_record_type="&record_type.";
end;


/* B01: The primary regulator of the bank is not provided.  */
error_code="B01";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code=""  then output;


/* B02: The legal name of the bank is not provided. */
error_code="B02";
target_field_name="institution_legal_name";
if institution_legal_name = ""  then output;

/* B03: The address of the bank is not provided. */
error_code="B03";
target_field_name="institution_address";
if institution_address=""  
   and institution_city_name = "" 
   and institution_state_code = ""
   and institution_zip_code = ""
   then output;

/* B04: The street address of the bank is not provided. */
error_code="B04";
target_field_name="institution_address";
if institution_address=""  then output;

/* B05: The city of the bank is not provided.*/
error_code="B05";
target_field_name="institution_city_name";
if institution_city_name=""  then output;

/* B06: The state/territory of the bank is not provided. */
error_code="B06";
target_field_name="institution_state_code";
if institution_state_code=""  then output;

/* B07: The state/territory code of the bank is invalid. */
/* TODO need to check US states and terrioities               */ 
error_code="B07";
target_field_name="institution_state_code";
if put('US_'||trim(institution_state_code),$RT_STATE_PROVINCE.) ne 'VALID' and
    institution_state_code  not in ("PW" "MP" "VI" "US" "AS" "FM" "PR" "GU" "MH") 
   then output;
 

/* B08: The ZIP Code of the bank is not provided. */
error_code="B08";
target_field_name="institution_zip_code";
if institution_zip_code=""  then output;

/* B09: The ZIP Code of the bank is invalid. */
error_code="B09";
target_field_name="institution_zip_code";
 
if  institution_zip_code ne ' ' 
    and ( (length(institution_zip_code) ne 5 and length(institution_zip_code) ne 9)
          or notdigit(strip(institution_zip_code)) > 0
        ) 
    then output;


/* B10*: The EIN of the bank is not provided..  */
error_code="B10";
target_field_name="institution_tax_id";
if institution_tax_id="" then output;

/* B11: The EIN of the bank is invalid.  */
error_code="B11";
target_field_name="institution_tax_id";
if (length(institution_tax_id) ne 9 
    or notdigit(strip(institution_tax_id))
    or institution_tax_id in ('999999999' 
                              '888888888' 
                              '777777777' 
                              '666666666' 
                              '555555555' 
                              '444444444' 
                              '333333333' 
                              '222222222' 
                              '111111111'  
                              '000000000' 
                              '123456789'
                           ) 
                          
    )  
    then output;


/*
  Validate Part IV Signature section 
*/    

/* D06: The phone number recorded for the Authorized Official is invalid. */
error_code="D06";
target_field_name="contact_phone_number";
if contact_phone_number ne "" 
   and (length(contact_phone_number) ne 10 or 
        notdigit(strip(contact_phone_number)) > 0) then 
do;
   error_record_type="report";
   output;
   error_record_type="&record_type.";
end;


 
 %mend; /* VALIDATE_FINCEN_FILER */

%macro VALIDATE_FINCEN_TRANSMITTER;
 
/* T01: The legal name is not recorded. */
error_code="T01";
target_field_name="transmitter_name";
if transmitter_name=""  then output;

/* T02: The element is not recorded. Address */
error_code="T02";
target_field_name="transmitter_address";
if transmitter_address=""  
   and transmitter_city_name = ""    
   and transmitter_zip_code = ""
   and transmitter_country_code = ""
then output;


/* T03: The element is not recorded or does not contain a value. RawCityText */
error_code="T03";
target_field_name="transmitter_city_name";
if transmitter_city_name=""  then output;

/* T04: The element is not recorded or does not contain a value. RawCountryCodeText */
error_code="T04";
target_field_name="transmitter_country_code";
if transmitter_country_code ='' then output;

/* T05: The element is not recorded or does not contain a value when the country is US, CA, MX, or a U.S. Territory. */
error_code="T05";
target_field_name="transmitter_state_code";
if transmitter_state_code="" and transmitter_country_code in ("US" "MX" "CA" "PW" "MP" "VI" "US" "AS" "FM" "PR" "GU" "MH") 
   then output;

/* T06: The element is recorded but does not contain a value. RawStreetAddress1Text */
error_code="T06";
target_field_name="transmitter_address";
if transmitter_address=""  then output;
  

/* T07: The element is recorded but does not contain a value.  RawZIPCode */
error_code="T07";
target_field_name="transmitter_zip_code";
if transmitter_zip_code=""  then output;

/* T08: The element is recorded but does not contain a value.  PhoneNumberText */
error_code="T08";
target_field_name="transmitter_telephone";
if transmitter_telephone=""  then output;

/* T09: The element is not recorded or does not contain a value. PartyIdentification (EIN) */
error_code="T09";
target_field_name="transmitter_tax_id";
if transmitter_tax_id="" then output;


/* T10: The element is not recorded for this party type information. PartyIdentification (TCC) */
error_code="T10";
target_field_name="transmitter_control_code";
if transmitter_control_code=""  then output;

/* T11: The element contains an invalid value.  RawCountryCodeText RawStateCodeText*/
/* SKIP. Handled by T12 */

/* T12: The state/territory/country code mismatch. */
error_code="T12";
target_field_name="transmitter_state_code";
if transmitter_state_code ne "" and transmitter_country_code ne ""
   and (put((trim(transmitter_country_code)||'_'||trim(transmitter_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  
   then output;

/* T13: The element contains a value for the Transmitter and the country is not US, CA, MX, or a U.S. Territory. */
error_code="T13";
target_field_name="transmitter_state_code";
if transmitter_state_code ne "" and transmitter_country_code ne ""
   and transmitter_country_code not in ("US" "MX" "CA" "PW" "MP" "VI" "US" "AS" "FM" "PR" "GU" "MH") 
   then output;

%mend; /* VALIDATE_FINCEN_TRANSMITTER */


