/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* validated against FinCEN rules while loading sample data for regulatory     */
/* reports.  The validation rules should be modified to meet your              */
/* organizational needs.  These are provided as a sample.                      */
/* *****************************************************************************/

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%MACRO VALIDATE_FINCEN_FILER;

/* F3: The element is not recorded.  */
error_code="F3";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code=""  then output;

/* F5: The element contains a value that is not a valid code. */
error_code="F5";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code ne "" and index("ABCDEFGZ",strip(institution_prim_fed_reg_code))=0  then output;

/* F7: The element is not recorded or does not contain a value. */
error_code="F7";
target_field_name="institution_legal_name";
if institution_legal_name=""  then output;

/* F9: The element is not recorded or does not contain a value.  */
error_code="F9";
target_field_name="institution_tax_id";
if institution_tax_id="" then output;

/* F8: The element contains an invalid number string for a US TIN.  */
error_code="F8";
target_field_name="institution_tax_id";
if ((length(institution_tax_id) ne 9 or institution_tax_id in ('999999999' '000000000' '123456789') or notdigit(strip(institution_tax_id))))  
    then output;
    

/* G3: The element is not recorded or does not contain a value.*/
error_code="G3";
target_field_name="contact_office";
if contact_office=""  then output;

/* G5: The element is not recorded or does not contain a value (e.g. the value is null). */
error_code="G5";
target_field_name="contact_phone_number";
if contact_phone_number=""  then output;

/* G4: The value recorded for the element contains non-numeric characters. */
error_code="G4";
target_field_name="contact_phone_number";
if (contact_phone_number ne "" and (notdigit(strip(contact_phone_number)) > 0))   then output;
 

/* F12: The element is not recorded or does not contain a value. */
error_code="F12";
target_field_name="institution_address";
if institution_address=""  then output;

/* F13: The element is not recorded or does not contain a value. */
error_code="F13";
target_field_name="institution_city_name";
if institution_city_name=""  then output;

/* F15: The element contains a value that is not a valid code. */
error_code="F15";
target_field_name="institution_state_code";
if (put((trim(institution_country_code)||'_'||trim(institution_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* F14: The element is not recorded or does not contain a value. */
error_code="F14";
target_field_name="institution_state_code";
if institution_state_code=""  then output;

/* F20: The element contains a value that is not a valid code. */
error_code="F20";
target_field_name="institution_country_code";
if ((institution_country_code ne ' ' and institution_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* F17: The element is not recorded or does not contain a value. */
error_code="F17";
target_field_name="institution_zip_code";
if institution_zip_code=""  then output;

/* F18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="F18";
target_field_name="institution_zip_code";
if (length(institution_zip_code) =9 and substr(institution_zip_code,6,4) in ('0000' '9999'))  then output;

/* F18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="F18";
target_field_name="institution_zip_code";
if (((length(institution_zip_code) ne 5 and length(institution_zip_code) ne 9)  or notdigit(strip(institution_zip_code)) > 0)) and (institution_zip_code ne ' ') then output;

/* F23: The element is recorded but without a value (i.e. the value is null). */
error_code="F23";
target_field_name="institution_type_code";
if institution_type_code=""  then output;

/* F24: The element contains a value that is not a valid code. */
error_code="F24";
target_field_name="institution_type_code";
if institution_type_code ne "" and index("ABCDZ",strip(institution_type_code))=0 
   and form_config_key=312 then output;

/* F25: The element does not contain a value and Other is indicated. */
error_code="F25";
target_field_name="institution_type_other";
if institution_type_other="" and (institution_type_code  ='Z') then output;

/* F28: The element is recorded but without a value (i.e. the value is null). */
error_code="F28";
target_field_name="institution_gaming_type_code";
if institution_gaming_type_code="" and (institution_type_code = 'A') then output;

/* F29: The element contains a value that is not a valid code. */
error_code="F29";
target_field_name="institution_gaming_type_code";
if institution_gaming_type_code ne "" and index("ABCZ",strip(institution_gaming_type_code))=0 and (institution_type_code = 'A') then output;

/* F31: The element does not contain a value and Other is indicated. */
error_code="F31";
target_field_name="institution_gaming_type_other";
if institution_gaming_type_other="" and (institution_gaming_type_code = 'Z') then output;

/* F34: The element contains a value that is not a valid code. */
error_code="F34";
target_field_name="institution_id_type_code";
if institution_id_type_code ne "" and index("ABCDEFG",strip(institution_id_type_code))=0  then output;

/* F35: The element is not recorded or does not contain a value. */
error_code="F35";
target_field_name="institution_id";
if institution_id="" and (institution_id_type_code ne '') then output;

/*  future additions           */
/*  institution_id_type_code   */
/*  institution_sf_type_code   */ 

%mend; /* VALIDATE_FINCEN_FILER */

%macro VALIDATE_FINCEN_TRANSMITTER;

/* H7: The element is not recorded or does not contain a value. */
error_code="H7";
target_field_name="transmitter_name";
if transmitter_name=""  then output;

/* H8: The element is not recorded or does not contain a value. */
error_code="H8";
target_field_name="transmitter_address";
if transmitter_address=""  then output;

/* H9: The element is not recorded or does not contain a value. */
error_code="H9";
target_field_name="transmitter_city_name";
if transmitter_city_name=""  then output;

/* H11: The element contains a value that is not a valid code. */
error_code="H11";
target_field_name="transmitter_state_code";
if (put((trim(transmitter_country_code)||'_'||trim(transmitter_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* H10: The element is not recorded or does not contain a value. */
error_code="H10";
target_field_name="transmitter_state_code";
if transmitter_state_code=""  then output;

/* H16: The element contains a value that is not a valid code. */
error_code="H16";
target_field_name="transmitter_country_code";
if ((transmitter_country_code ne ' ' and transmitter_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* H19: The element is not recorded or does not contain a value. */
error_code="H19";
target_field_name="transmitter_zip_code";
if transmitter_zip_code=""  then output;

/* H18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="H18";
target_field_name="transmitter_zip_code";
if (length(transmitter_zip_code) =9 and substr(transmitter_zip_code,6,4) in ('0000' '9999'))  then output;

/* H18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="H18";
target_field_name="transmitter_zip_code";
if (((length(transmitter_zip_code) ne 5 and length(transmitter_zip_code) ne 9)  or notdigit(strip(transmitter_zip_code)) > 0)) and (transmitter_zip_code ne ' ') then output;

/* H19: The element is not recorded or does not contain a value.  */
error_code="H19";
target_field_name="transmitter_tax_id";
if transmitter_tax_id="" then output;

/* H21: The element contains an invalid number string for a US TIN.  */
error_code="H21";
target_field_name="transmitter_tax_id";
if ((length(transmitter_tax_id) ne 9 or transmitter_tax_id in ('999999999' '000000000' '123456789') or notdigit(strip(transmitter_tax_id))))  
    then output;

/* H23: The element is not recorded or does not contain a value. */
error_code="H23";
target_field_name="transmitter_control_code";
if transmitter_control_code=""  then output;
    
/* I7: The element is not recorded or does not contain a value. */
error_code="I7";
target_field_name="transmitter_contact_name";
if transmitter_contact_name=""  then output;

/* H26: The element is not recorded or does not contain a value (e.g. the value is null). */
error_code="G5";
target_field_name="transmitter_telephone";
if transmitter_telephone=""  then output;

/* H25: The value recorded for the element contains non-numeric characters. */
error_code="G4";
target_field_name="transmitter_telephone";
if (transmitter_telephone ne "" and (notdigit(strip(transmitter_telephone)) > 0))   then output;

/* I3: The element is not recorded or does not contain a value. */
error_code="I3";
target_field_name="effective_from_date";
if effective_from_date = . then output;

/* I5: The element is not recorded or does not contain a value. */
error_code="I5";
target_field_name="effective_to_date";
if effective_to_date = . then output;

%mend; /* VALIDATE_FINCEN_TRANSMITTER */


