/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* validated against FinCEN rules while loading sample data for regulatory     */
/* reports.  The validation rules should be modified to meet your              */
/* organizational needs.  These are provided as a sample. These validation     */
/* rules will be specific to SARX error codes and rules.                       */
/* *****************************************************************************/

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%MACRO VALIDATE_FINCEN_FILER;

/* E2: The element is not recorded.  */
error_code="E2";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code=""  then output;

/* E4: The element contains a value that is not a valid code. */
error_code="E4";
target_field_name="institution_prim_fed_reg_code";
if institution_prim_fed_reg_code ne "" and index("ABCDEFGHZ",strip(institution_prim_fed_reg_code))=0  then output;

/* Cross validation for INSTITUTION_TYPE_CODE and INSTITUTION_PRIM_FED_REG_CODE */
error_code="E4";
target_field_name="institution_prim_fed_reg_code";
/* IRS specific Institution Type codes */
if (institution_type_code ='A' or institution_type_code='C' or institution_type_code='D') and 
   (institution_prim_fed_reg_code ^= 'D') then output;
/* Depository codes */
if (institution_type_code ='B') and 
   (index("BCDEF",strip(institution_prim_fed_reg_code))=0) then output;
/* Securities codes */
if (institution_type_code ='E') and 
   (index("AG",strip(institution_prim_fed_reg_code))=0) then output;

/* E5: The element is not recorded or does not contain a value. */
error_code="E5";
target_field_name="institution_legal_name";
if institution_legal_name=""  then output;

/* E7: The element is not recorded or does not contain a value.  */
error_code="E7";
target_field_name="institution_tax_id";
if institution_tax_id="" then output;

/* E9: The element contains an invalid number string for a US TIN.  */
error_code="E9";
target_field_name="institution_tax_id";
if ((length(institution_tax_id) ne 9 or institution_tax_id in ('999999999' '000000000' '123456789') or notdigit(strip(institution_tax_id))))  
    then output;
    

/* E41: The element is not recorded or does not contain a value.*/
error_code="E41";
target_field_name="contact_office";
if contact_office=""  then output;

/* E42: The element is not recorded or does not contain a value (e.g. the value is null). */
error_code="E42";
target_field_name="contact_phone_number";
if contact_phone_number=""  then output;

/* E42: The value recorded for the element contains non-numeric characters. */
error_code="E42";
target_field_name="contact_phone_number";
if (contact_phone_number ne "" and (notdigit(strip(contact_phone_number)) > 0))   then output;
 

/* E24: The element is not recorded or does not contain a value. */
error_code="E24";
target_field_name="institution_address";
if institution_address=""  then output;

/* E26: The element is not recorded or does not contain a value. */
error_code="E26";
target_field_name="institution_city_name";
if institution_city_name=""  then output;

/* E28: The element contains a value that is not a valid code. */
error_code="E28";
target_field_name="institution_state_code";
if (put((trim(institution_country_code)||'_'||trim(institution_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* E27: The element is not recorded or does not contain a value. */
error_code="E27";
target_field_name="institution_state_code";
if institution_state_code=""  then output;

/* E36: The element contains a value that is not a valid code. */
error_code="E36";
target_field_name="institution_country_code";
if ((institution_country_code ne ' ' and institution_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* E34: The element is not recorded or does not contain a value. */
error_code="E34";
target_field_name="institution_zip_code";
if institution_zip_code=""  then output;

/* E35: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="E35";
target_field_name="institution_zip_code";
if (length(institution_zip_code) =9 and substr(institution_zip_code,6,4) in ('0000' '9999'))  then output;

/* E35: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="E35";
target_field_name="institution_zip_code";
if (((length(institution_zip_code) ne 5 and length(institution_zip_code) ne 9)  or notdigit(strip(institution_zip_code)) > 0)) and (institution_zip_code ne ' ') then output;

/* E11: The element is recorded but without a value (i.e. the value is null). */
error_code="E11";
target_field_name="institution_type_code";
if institution_type_code=""  then output;

/* E12: The element contains a value that is not a valid code. */
error_code="E12";
target_field_name="institution_type_code";
if institution_type_code ne "" and index("ABCDEFGZ",strip(institution_type_code))=0 
   then output;

/* E13: The element does not contain a value and Other is indicated. */
error_code="E13";
target_field_name="institution_type_other";
if institution_type_other="" and (institution_type_code  ='Z') then output;

/* D10: The element is recorded but without a value (i.e. the value is null). */
error_code="D10";
target_field_name="institution_gaming_type_code";
if institution_gaming_type_code="" and (institution_type_code = 'A') then output;


/* D16: The element contains a value that is not a valid code. */
error_code="D16";
target_field_name="institution_gaming_type_code";
if institution_gaming_type_code ne "" and index("ABCZ",strip(institution_gaming_type_code))=0 and (institution_type_code = 'A') then output;
*/

/* D15: The element does not contain a value and Other is indicated. */
error_code="D15";
target_field_name="institution_gaming_type_other";
if institution_gaming_type_other="" and (institution_gaming_type_code = 'Z') then output;


/* Exx: The element contains a value that is not a valid code. */
/** Error condition does not exist with a FinCEN code for SARX. 
error_code="Exx";
target_field_name="institution_id_type_code";
if institution_id_type_code ne "" and index("ABCDEFG",strip(institution_id_type_code))=0  then output;
**/


/* E21: The element is not recorded or does not contain a value. */
error_code="E21";
target_field_name="institution_id";
if institution_id="" and (institution_id_type_code ne '') then output;

/*  future additions           */
/*  institution_sf_type_code   */ 

%mend; /* VALIDATE_FINCEN_FILER */

%macro VALIDATE_FINCEN_TRANSMITTER;

/* G2: The element is not recorded or does not contain a value. */
error_code="G2";
target_field_name="transmitter_name";
if transmitter_name=""  then output;

/* G3: The element is not recorded or does not contain a value. */
error_code="G3";
target_field_name="transmitter_address";
if transmitter_address=""  then output;

/* G4: The element is not recorded or does not contain a value. */
error_code="G4";
target_field_name="transmitter_city_name";
if transmitter_city_name=""  then output;

/* G6: The element contains a value that is not a valid code. */
error_code="G6";
target_field_name="transmitter_state_code";
if (put((trim(transmitter_country_code)||'_'||trim(transmitter_state_code)),$RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* G5: The element is not recorded or does not contain a value. */
error_code="G5";
target_field_name="transmitter_state_code";
if transmitter_state_code=""  then output;

/* G10: The element contains a value that is not a valid code. */
error_code="G10";
target_field_name="transmitter_country_code";
if ((transmitter_country_code ne ' ' and transmitter_country_code not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* G7: The element is not recorded or does not contain a value. */
error_code="G7";
target_field_name="transmitter_zip_code";
if transmitter_zip_code=""  then output;

/* G8: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="G8";
target_field_name="transmitter_zip_code";
if (length(transmitter_zip_code) =9 and substr(transmitter_zip_code,6,4) in ('0000' '9999'))  then output;

/* G8: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_code="G8";
target_field_name="transmitter_zip_code";
if (((length(transmitter_zip_code) ne 5 and length(transmitter_zip_code) ne 9)  or notdigit(strip(transmitter_zip_code)) > 0)) and (transmitter_zip_code ne ' ') then output;

/* G13: The element is not recorded or does not contain a value.  */
error_code="G13";
target_field_name="transmitter_tax_id";
if transmitter_tax_id="" then output;

/* G12: The element contains an invalid number string for a US TIN.  */
error_code="G12";
target_field_name="transmitter_tax_id";
if ((length(transmitter_tax_id) ne 9 or transmitter_tax_id in ('999999999' '000000000' '123456789') or notdigit(strip(transmitter_tax_id))))  
    then output;

/* G14: The element is not recorded or does not contain a value. */
error_code="G14";
target_field_name="transmitter_control_code";
if transmitter_control_code=""  then output;
    
/* H2: The element is not recorded or does not contain a value. */
error_code="H2";
target_field_name="transmitter_contact_name";
if transmitter_contact_name=""  then output;

/* G20: The element is not recorded or does not contain a value (e.g. the value is null). */
error_code="G20";
target_field_name="transmitter_telephone";
if transmitter_telephone=""  then output;

/* G19: The value recorded for the element contains non-numeric characters. */
error_code="G19";
target_field_name="transmitter_telephone";
if (transmitter_telephone ne "" and (notdigit(strip(transmitter_telephone)) > 0))   then output;


%mend; /* VALIDATE_FINCEN_TRANSMITTER */


