/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_asofdttm.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    update cdd as of datetime. 
| Note:    This macro will be called by cdd_main_process 
+------------------------------------------------------------------------------+
| INPUTS
|     action - action to be taken  (required)
|         NEW  - get the last as of datetime and create a new run as of datetime.
|         GET  - get the last as of datettime and run as of datettime. No new
|                run as of datetime will be created.
|         FLAG - set used_flg to 1 for the current run as of datetime
|         RESET- clear all entries in cdd_asofdttm and reset the global 
|                macro variables.
|     run_asofdttm - (optional) datetime to be used as cdd_run_asofdttm. Used  
|                    only when action=NEW 
|     last_asofdttm- (optional) datetime to be used as cdd_last_asofdttm.
|                    Used only when action=NEW
+------------------------------------------------------------------------------+
| OUTPUTS:
|     CDD_RUN_ASOFDTTM - global macro variable for run as of datettime
|     CDD_LAST_ASOFDTTM - global macro variable for last as of datetime 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_asofdttm(action=GET,run_asofdttm=,last_asofdttm=) ;

   %let trans_rc=0;

   /* Initialized cdd_run_asofdttm and cdd_last_asofdttm global macro variable */
   %if not %symexist(cdd_run_asofdttm) %then %global cdd_run_asofdttm;
   %if not %symexist(cdd_last_asofdttm) %then %global cdd_last_asofdttm;

    %let max_used_asofdttm=.;
    %let max_unused_asofdttm=.;

    /* create CJOBSTAT.CDD_ASOFDTTM if it does not exist */
    %if not %sysfunc(exist(cjobstat.cdd_asofdttm)) %then %do;
      
       data cjobstat.cdd_asofdttm;
            attrib asofdttm length=8 format=datetime. informat=datetime.
                            label='Run As of Date Time'
                   used_flg length=$1 
                            label='Used Flag'
            ;
            stop;
       run;
       %fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
        %fcf_put_error;
        %goto MACRO_END;
    %end;  
      
    %end;
    %else %do;

         /* Get the max  used as of datetime */
          proc sql noprint;    
               select distinct put(max(asofdttm),best.) into :max_used_asofdttm
               from cjobstat.cdd_asofdttm
               where used_flg='1';

               select distinct put(max(asofdttm),best.) into :max_unused_asofdttm
               from cjobstat.cdd_asofdttm
               where used_flg='0';
          quit; 
          %fcf_rcset(&syserr)
          %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
          %end;  
    %end;

   /* if the action is to NEW, either derive it or use the provided parameters */
   %if %upcase(&action)=NEW %then %do;
       %if %str(&run_asofdttm) ne %str() %then %let cdd_run_asofdttm=&run_asofdttm; 
       %else %do;
            %let cdd_run_asofdttm=%sysfunc(datetime());
            %let cdd_run_asofdttm=%sysfunc(int(&cdd_run_asofdttm));
       %end;
       %if %str(&last_asofdttm) ne %str() %then %let cdd_last_asofdttm=&last_asofdttm; 
       %else %let cdd_last_asofdttm=&max_used_asofdttm; 
       proc sql noprint;
            insert into cjobstat.cdd_asofdttm
                   values (&cdd_run_asofdttm,'0');
       quit;
        %fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
            %fcf_put_error;
            %goto MACRO_END;
       %end;  
   %end;

    %if %upcase(&action)=GET %then %do;
       %let cdd_run_asofdttm=&max_unused_asofdttm;
       %let cdd_last_asofdttm=&max_used_asofdttm; 
   %end;

   /* if the action is to flag the as of datetime as used, set USER_FLG=1 for the current run as of datetime */
   %if %upcase(&action)=FLAG %then %do;
           proc sql noprint;
            update cjobstat.cdd_asofdttm
            set used_flg='1' 
            where asofdttm=&cdd_run_asofdttm
           ;
       quit;
        %fcf_rcset(&syserr)
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  
   %end;
   
   /* If the action is reset, delete all records in cjobstat.cdd_asofdttm */
   /* and set CDD_RUN_ASOFDTTM and CDD_LAST_ASOFDTTM to missing.         */

   %if %upcase(&action)=RESET %then %do;
       proc sql;
            delete from cjobstat.cdd_asofdttm
        ; 
        %fcf_rcset(&syserr)
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
              %fcf_put_error;
              %goto MACRO_END;
        %end;  
        %let cdd_last_asofdttm=.;
        %let cdd_run_asofdttm=.;
   %end;


 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %cdd_error;
   %else %do;
      %cdd_success;
   %end;

%mend;
