/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_crr_entity_fact_updt.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows Unix
| DESC:    Update SEG_KC.FSK_CDD_ENTITY_FACT  
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|           in_dsn= table name of the input dataset
|                       e. g. _TMP_CDD_ENTITY_FACT  
|           primary_entity_level_code_value=content of the PRIMARY_ENTITY_LEVEL_CODE
|                                           to be used for filtering FSK_CDD_ENTITY_FACT 
| Columns that will be updated are
| LAST_CHANGE_DATE, LAST_SCORING_EVENT_KEY, CURR_REVIEW_SCORING_EVENT_KEY, and
| OVERRIDE_SCORE_CLASS_CODE. OVERRIDE_SCORE_CLASS_CODE potentially can be blanked by the CRR process when 
| the parties are being scored for periodic review. 
| Note:
| For better performance the macro is assuming that
| 1. &in_dsn.LAST_CHANGE_DATE is the same for all records 
| 2. FSK_CDD_ENTITY_FACT.OVERRIDE_SCORE_CLASS_CODE should only be blanked out by the CRR process.
| 3. FSK_CDD_ENTITY_FACT.CURR_REVIEW_SCORING_EVENT_KEY can be changed by the CRR process only when 
|    <&in_dsn>.LAST_SCORING_EVENT_KEY=<&in_dsn>.CURR_REVIEW_SCORING_EVENT_KEY  
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_crr_entity_fact_updt(  in_dsn=WORK._TMP_CDD_ENTITY_FACT
                                , primary_entity_level_code_value='PTY');

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;


    /*--------------------------------------------------
    * Update FSK_CDD_ENTITY_FACT.LAST_CHANGE_DATE
    * This step is handled in SAS/Access for consistent
    * way of handling dates.
    *-------------------------------------------------*/
    /*------------------------------------------
    * Map input data columns to FSK_TEMP_TABLE
    *------------------------------------------*/

    %let _primary_entity_number=temp_char_1_value;
    %let _investigation_ind=temp_char_2_value;
    %let _cdd_review_flg=temp_char_3_value;
    %let _last_change_date=temp_date_1_value;
    %let _last_scoring_event_key=temp_num_1_value;
    %let _curr_review_scoring_event_key=temp_num_2_value;
    %let temptable=FSK_TEMP_TABLE;

    /*---------------------------------
    * Clear temp table
    *---------------------------------*/

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute (delete from &segKCSchema..&temptable) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

    %fcf_rcset(&sqlrc.);
    %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_CLEAR_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_TEMP_TABLE
                                    )
                              );
        %fcf_put_error;
        %goto MACRO_END;
    %end;

     /*----------------------------------------------------------
      * Load transaction data to temp table before master update
      *---------------------------------------------------------*/


    proc append base=seg_kc.&temptable (dbforce=yes)
               data=&in_dsn.
                    (rename=(
                             primary_entity_number=&_primary_entity_number
                             investigation_ind=&_investigation_ind
                             cdd_review_flg=&_cdd_review_flg
                             last_change_date=&_last_change_date
                             last_scoring_event_key=&_last_scoring_event_key
                             curr_review_scoring_event_key=&_curr_review_scoring_event_key
                      )
                     )
                     force nowarn
                     ;
    where __crr_new_flg = '0';
    run;
    %fcf_rcset(&syserr);
    %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_UPDATE_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_TEMP_TABLE
                                    )
                              );

        %fcf_put_error;
        %goto MACRO_END;
    %end;


    proc sql noprint;

        SELECT "'"||strip(primary_entity_number)||"'" INTO :first_rec
            FROM &in_dsn (obs=1)
        ;
        /* set the last change date of all the parties that have a newer score */
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);
        execute
        (
        UPDATE &segKCSchema..fsk_cdd_entity_fact
            set last_change_date=(select &_last_change_date. from &segKCSchema..&temptable.
                            where &_primary_entity_number. = &first_rec)
                            /* just pull first record as dates are all the same */
            WHERE primary_entity_level_code=&primary_entity_level_code_value
                  and primary_entity_number  in
                   (select distinct &_primary_entity_number.
                    from &segKCSchema..&temptable.)
        )
        by &dbflavor. ;
    quit;

    %fcf_rcset(&sqlrc.);
    %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_UPDATE_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_CDD_ENTITY_FACT
                                    )
                              );
        %fcf_put_error;
        %goto MACRO_END;
    %end;


    /*--------------------------------------------------------------
    * Set FSK_CDD_ENTITY_FACT.OVERRIDE_SCORE_CLASS_CODE to blank
    * This step is handled in SAS/Access for consistent
    * way of blank.
    *--------------------------------------------------------------*/

    proc sql noprint;

         /* set the last change date of all the parties that have a newer score */
         connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);
         execute(
         update &segKCSchema..fsk_cdd_entity_fact
         set   override_score_class_code=''
             , next_review_date=null
             , final_disposition_code=''
         where primary_entity_level_code=&primary_entity_level_code_value
               and primary_entity_number  in
                   (select distinct &_primary_entity_number
                    from &segKCSchema..&temptable.
                          where (&_investigation_ind='Y' or &_cdd_review_flg='1')
                    )
         )
         by &dbflavor. ;
    quit;

    %fcf_rcset(&sqlrc.);
    %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_UPDATE_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_CDD_ENTITY_FACT
                                    )
                              );
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql feedback noprint;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);
        /*----------------------------------------------------------
        * Update FSK_CD_ENTITY_FACT.LAST_SCORING_EVENT_KEY
        * This step will take awhile to run.
        *---------------------------------------------------------*/
        execute(
            MERGE INTO &segKCSchema..fsk_cdd_entity_fact
            USING (
                SELECT
                    temp_char_1_value AS entity_number,
                    temp_num_1_value AS scoring_event_key
                FROM &segKCSchema..fsk_temp_table
            ) data
            ON (primary_entity_number=entity_number AND primary_entity_level_code='PTY')
            WHEN MATCHED THEN
               UPDATE
               SET last_scoring_event_key=scoring_event_key
            %if %upcase(&dbflavor) eq ODBC %then %do;
                ;
            %end;
        ) by &dbflavor. ;
        disconnect from &dbflavor;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_UPDATE_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_CDD_ENTITY_FACT-MERGE));
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql feedback noprint;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain."  &segKCDBConnOpts);
        /*----------------------------------------------------------
        * Update FSK_CD_ENTITY_FACT.CURR_REVIEW_SCORING_EVENT_KEY
        * when CURR_REVIEW_SCORING_EVENT_KEY=LAST_SCORING_EVENT_KEY
        *---------------------------------------------------------*/
           execute(
           update &segKCSchema..fsk_cdd_entity_fact
              set    curr_review_scoring_event_key=last_scoring_event_key
              where primary_entity_level_code=&primary_entity_level_code_value
                    and primary_entity_number in
                        (select &_primary_entity_number
                         from  &segKCSchema..&temptable
                         where &_last_scoring_event_key=&_curr_review_scoring_event_key
                          )
           ) by &dbflavor. ;
        disconnect from &dbflavor;
    quit;
  

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg
                                    ,CDD_TABLE_UPDATE_ERROR
                                    ,NOQUOTE
                                    ,&sysmacroname
                                    ,SEG_KC.FSK_CDD_ENTITY_FACT-UPDATE));
        %fcf_put_error;
        %goto MACRO_END;
    %end;


  
     /*---------------------------------
      * Clear temp table after using it.
      *---------------------------------*/

    proc sql;
        %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
        execute (delete from &segKCSchema..&temptable) by &dbflavor;
        %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
    quit;

      
%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %cdd_error;
    %else
    %do;
         %cdd_success;
    %end;

%mend;
