/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_proc_append.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows Unix
| DESC:    Standard CDD macro to append data table to a base table
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|           base_ds=    name of destination dataset/table
|           data_ds=    name of source dataset 
|           where_cl=   where clause for filtering the source data before append  
|           option =    PROC APPEND valid options  
|
| Even with NOWARN option, warnings will still be given when the column lengths in base_ds
|           and data_ds do not match.
| This macro uses bulk load and uses fcf_bulk_load macro to get bulk load option if bulk load is enabled.
|
| For DB2 make sure that base_ds is not pointing to a view, bulkload is not supported
| for DB2 views.
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
%macro cdd_proc_append (  base_ds=
                        , data_ds= 
						, where_cl=
                        , option=force nowarn
                        );

    /* use bulk load if enabled (set fcf_enable_bulkload=Y in the fcf_autoexec.sas file) */
    %let bulkload_options=%fcf_bulk_load(module=CDD_CRR);

    %if &bulkload_options ne %then %do;
        %put NOTE: DBMS used: &dbflavor;
        %put NOTE: DBMS bulkload options: &bulkload_options;
    %end;

    /* Get the column list of base_ds */
    proc contents data=&base_ds out=_t_base_columns (keep=name) noprint;
    run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmodule_=CDD_CRR;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
    %end;


   proc contents data=&data_ds out=_t_data_columns (keep=name) noprint;
   run;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmodule_=CDD_CRR_DRIVER;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

   %let _t_keep_list=;

   proc sql noprint;
        select distinct upcase(name) into :_t_keep_list separated by ' '
        from _t_data_columns
        where upcase(name) in (select distinct upcase(name) from _t_base_columns)
   ;
   quit;

   %if %str(&_t_keep_list) ne %str() %then 
   %do; 
      
        proc append base=&base_ds (&bulkload_options)
                    data=&data_ds  &option ;
		 %if %nrstr(&where_cl) ne %nrstr() %then %do;
             where &where_cl ; 
		 %end;
        run;

        %fcf_rcset(&syserr)
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmodule_=CDD_CRR;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
      %end;
   %end;

%MACRO_END:

      %if &trans_rc eq 3 or &trans_rc ge 5 %then
         %cdd_error;
      %else 
      %do;
         %cdd_success;
      %end;
%mend;

 
