/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_ent_load_alert_txn.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Creates the table fsk_alert_transaction in Knowledge Center.  This
|          table contains replicated transactions for all entities with active 
|          alerts in seg_kc.fsk_alert.  The transactions are selected from
|          alert tables fsc_cash_flow_alert, fsc_account_trade_alert, and 
|          fsc_account_event_alert and augmented with information from the
|          dim tables for display in the Entity Triage UI.
+------------------------------------------------------------------------------+
| USAGE: %fcf_ent_load_alert_txn
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             None
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_ent_load_alert_txn;

/*------------------------------------------
 * Set error code to 0 and message to blank
 *------------------------------------------*/
 %let trans_rc = 0;
 %let _fcfmsg_=;

 %put NOTE: FCF: Building transaction table from replicated transactions for alerted entities;

 /*-------------------------------------------------------------------
 * Build table of transactions in cash_flow_alert for alerted entities
 *-------------------------------------------------------------------*/
 
proc sql;
connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table alert_transaction_cfa as
select * from 
connection to &dbflavor
(
select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       cfa.alert_id,
       cfa.transaction_key,
       trnd.transaction_reference_number as transaction_id,
       cfa.currency_amount as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       cfa.account_key,
       acc.account_number,
       ext.full_name as beneficiary_name,
       assc.associate_first_name,
       assc.associate_middle_name,
       assc.associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       cfa.trigger_ind as triggering_indicator_cd,
       cfa.time_key,
       cfa.date_key,
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss 
 from  (select distinct alert_id,
               transaction_key,
               account_key,
               transaction_type_key,
               time_key,
               date_key,
               transaction_status_key,
               branch_key,
               associate_key,
               beneficiary_ext_party_key,
               currency_amount,
               trigger_ind
              from   &segKCSchema..fsk_CASH_FLOW_ALERT   
                   where account_key in 
                   (select distinct account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc
                    )
         ) cfa  
join &segKCSchema..fsk_alert aler
on cfa.alert_id = aler.alert_id
and aler.alerted_entity_level_code<>'EXT'
and aler.alert_status_code='ACT'
join &segKCSchema..fsc_account_dim acc
on cfa.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on cfa.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=cfa.transaction_type_key
join &segKCSchema..fsc_ext_party_account_dim ext
on cfa.beneficiary_ext_party_key=ext.ext_party_account_key
join &segKCSchema..fsc_associate_dim assc
on cfa.associate_key=assc.associate_key
join &segKCSchema..fsc_branch_dim brch
on cfa.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on cfa.transaction_status_key=trns.transaction_status_key 
join &segKCSchema..fsc_date_dim dt
on cfa.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on cfa.time_key=tm.time_key
);

disconnect from &dbflavor;
quit;


%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_cfa;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

/*-------------------------------------------------------------------------
 * Build table of transactions in account_trade_alert for alerted entities
 *------------------------------------------------------------------------*/
proc sql;

connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table alert_transaction_ata as
select * from connection to &dbflavor
(
select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       ata.alert_id as alert_id,
       ata.transaction_key,
       trnd.transaction_reference_number as transaction_id,
       ata.principal_amount_in_txn_ccy as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       ata.account_key,
       acc.account_number,
       ' ' as beneficiary_name,
       assc.associate_first_name,
       assc.associate_middle_name,
       assc.associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       ata.trigger_ind as triggering_indicator_cd,
       ata.execution_time_key as time_key,
       trnd.date_key,
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss  
 from  (select distinct alert_id,
               transaction_key,
               account_key,
               transaction_type_key,
               execution_time_key,
               transaction_status_key,
               branch_key,
               associate_key,
               principal_amount_in_txn_ccy,
               trigger_ind
              from   &segKCSchema..fsk_ACCOUNT_TRADE_ALERT   
                   where account_key in 
                   (select account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc)
         ) ata  
join &segKCSchema..fsk_alert aler
on ata.alert_id = aler.alert_id
and aler.alerted_entity_level_code<>'EXT'
and aler.alert_status_code='ACT'
join &segKCSchema..fsc_account_dim acc
on ata.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on ata.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=ata.transaction_type_key
join &segKCSchema..fsc_associate_dim assc
on ata.associate_key=assc.associate_key
join &segKCSchema..fsc_branch_dim brch
on ata.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on ata.transaction_status_key=trns.transaction_status_key
join &segKCSchema..fsc_date_dim dt
on trnd.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on ata.execution_time_key=tm.time_key
);

disconnect from &dbflavor;


quit;

%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_ata;
     %fcf_put_error;
     %goto MACRO_END;
 %end;

/*----------------------------------------------------------------------------
 * Build table of transactions in account_event_fact for all alerted entities
 *---------------------------------------------------------------------------*/
proc sql;

connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

create table alert_transaction_aea as
select * from connection to &dbflavor

   (select distinct aler.alerted_entity_level_code,
       aler.alerted_entity_number,
       aea.alert_id,
       aea.transaction_key,
       trnd.transaction_reference_number as transaction_id,
       0 as transaction_amt,
       trnt.transaction_cdi_desc,
       trnt.primary_medium_desc,
       trnt.secondary_medium_desc,
       trnt.mechanism_desc as transaction_mechanism,
       aea.account_key,
       acc.account_number,
       ' ' as beneficiary_name,
       '' as associate_first_name,
       '' as associate_middle_name,
       '' as associate_last_name,
       brch.branch_name,
       trnd.transaction_description as transaction_desc,
       trns.status_desc as transaction_status,
       trigger_ind as triggering_indicator_cd,
       aea.time_key,
       aea.date_key,
       dt.calendar_date_sas,
       tm.time_hh,
       tm.time_mm,
       tm.time_ss 
    from  (select distinct alert_id,
               transaction_key,
               account_key,
               transaction_type_key,
               date_key,
               time_key,
               transaction_status_key,
               branch_key,
               trigger_ind
              from   &segKCSchema..FSK_ACCOUNT_EVENT_ALERT   
                   where account_key in 
                   (select account_key 
                    from &segKCSchema..fsk_account_keys_acc
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_pty
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_hhd
                    union
                    select account_key 
                    from &segKCSchema..fsk_account_keys_asc
                    )
         ) aea  
join &segKCSchema..fsk_alert aler
on aea.alert_id = aler.alert_id
and aler.alerted_entity_level_code<>'EXT'
and aler.alert_status_code='ACT'
join &segKCSchema..fsc_account_dim acc
on aea.account_key=acc.account_key
join &segKCSchema..fsc_transaction_dim trnd
on aea.transaction_key=trnd.transaction_key
join &segKCSchema..fsc_transaction_type_dim trnt
on trnt.transaction_type_key=aea.transaction_type_key
join &segKCSchema..fsc_branch_dim brch
on aea.branch_key=brch.branch_key
join &segKCSchema..fsc_transaction_status_dim trns
on aea.transaction_status_key=trns.transaction_status_key 
join &segKCSchema..fsc_date_dim dt
on aea.date_key=dt.date_key
join &segKCSchema..fsc_time_dim tm
on aea.time_key=tm.time_key
);
 disconnect from &dbflavor;
quit;

%fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_aea;
     %fcf_put_error;
     %goto MACRO_END;
 %end;


 /*------------------------------------------
 * Create tables with EXT alert transactions
 *------------------------------------------*/
 
 %fcf_ent_load_alert_txn_ext;

/*--------------------------------
 * Combine tables and add keys
 *-------------------------------*/

data alert_transaction;
set alert_transaction_cfa alert_transaction_ata alert_transaction_aea alert_transaction_ext;
if 1 eq 0 then set seg_kc.fsk_alert_transaction;
   fsk_alert_transaction_pk=_n_;
   associate_name=catx(' ',associate_first_name,associate_middle_name,associate_last_name);
   transaction_date=dhms(calendar_date_sas,input(time_hh,2.),input(time_mm,2.),input(time_ss,2.));
   created_timestamp=&fcf_start_datetime;
   drop associate_first_name associate_last_name associate_middle_name 
        calendar_date_sas time_hh time_mm time_ss
        account_key transaction_key;
run;

/*--------------------------------
 * Load table into database
 *-------------------------------*/

proc append base=seg_kc.fsk_alert_transaction data=alert_transaction force nowarn;
run;

/*--------------------------------
 * Clean-up Work Directory
 *-------------------------------*/
proc datasets library=work nolist;
    delete alert_transaction;
    delete alert_transaction_cfa;
    delete alert_transaction_ata;
    delete alert_transaction_aea;
    delete alert_transaction_ext;
run;

 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_ent_load_alert_txn;
