/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_ent_load_entity_txn_ext.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Called by fcf_ent_load_entity_txn to create tables of transactions
|          from from the cash_flow_fact table for EXT alerted entities.
|          Rows in cash_flow_fact with keys matching the alerted entity number
|          on the external beneficiary, external remitter, or external executing
|          party are selected and augmented with information from the dim 
|          tables for display in the Entity Triage UI.
+------------------------------------------------------------------------------+
| USAGE: %fcf_ent_load_alert_txn_ext
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             None
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_ent_load_entity_txn_ext;

 %global ext_ent_trx_list;
 %let ext_ent_trx_list=entity_transaction_ext1 entity_transaction_ext2 entity_transaction_ext3;

/*------------------------------------------
 * Set error code to 0 and message to blank
 *------------------------------------------*/
 %let trans_rc = 0;
 %let _fcfmsg_=;

 %put NOTE: FCF: Building transaction table from replicated transactions for alerted entities;

 %let trans_rc=0;

 /* define columns to check for external party keys in cash_flow_alert */
 %let replicate_col1=EXECUTING_EXT_PARTY_KEY;
 %let replicate_col2=REMITTER_EXT_PARTY_KEY;
 %let replicate_col3=BENEFICIARY_EXT_PARTY_KEY;


/* Load alert_id and primary_entity_level_code into in-database temp table */

proc sql;
    %fcf_begin_pass_through(dbflavor=&dbflavor, authenticationDomain=&segkcauthdomain, connOpts=&segKCDBConnOpts);
    execute (delete from &segKCSchema..FSK_TEMP_TABLE) by &dbflavor;
    %fcf_end_pass_through(dbflavor=&dbflavor, commit=yes);
quit;

/* Create temp table of alerts for External Parties.                  */
/* Find all ext_party_keys associated with the external party numbers */
/* Stage temp table by loading work table into in-database temp table */
 
 proc sql;
    create table fsk_temp_table as 
    select ae.alerted_entity_level_code as TEMP_CHAR_1_VALUE,
           ae.alerted_entity_number as TEMP_CHAR_2_VALUE,
           ext.ext_party_account_key as TEMP_NUM_1_VALUE
    from &seg_kc_libname..fsk_alerted_entity ae,
         &seg_kc_libname..fsc_ext_party_account_dim ext
    where alerted_entity_level_code="EXT" 
          and ae.alerted_entity_number = ext.external_party_number;
 quit;


%fcf_rcset(&sqlrc);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: Unable to create staging dataset to add EXT alerts to FSK_ENTITY_TRANSACTION;
    %fcf_put_error;
    %goto MACRO_END;
 %end;

 /*---------------------------------------------------------
  * There may be no EXT alerts of the type being processed
  *---------------------------------------------------------*/
 proc sql noprint;
     select put(count(*), best32.) into :numEXTKeys
     from fsk_temp_table;
 quit;

 %fcf_rcset(&sqlrc);
 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to determine number of rows in FSK_TEMP_TABLE.;
        %fcf_put_error;
        %goto MACRO_END;
 %end;

 %if  &numEXTKeys lt 1 %then %do;
     %let _fcfmsg_=Note: FCF: FSK_TEMP_TABLE is empty.  &_FCFMODULE_ is being skipped;
     %let ext_ent_trx_list=;
     %fcf_put_error;
 %end;
 %else %do;
    /* Loading work table into temp table */

    proc append    base=&seg_kc_libname..fsk_temp_table data=work.fsk_temp_table force;
    quit;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to append to FSK_TEMP_TABLE to add EXT alerts to FSK_ENTITY_TRANSACTION;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /* Create macro helper variables to avoid using temp table column names in subsequent joins */

    %let alerted_entity_level_code = TEMP_CHAR_1_VALUE;
    %let alerted_entity_number = TEMP_CHAR_2_VALUE;
    %let ext_party_account_key = TEMP_NUM_1_VALUE;


     /*-------------------------------------------------------------------
      * Find start date of transactions to save based on fcf_start_time
      * and look back value in the FSK_REFERENCE property table
      *-------------------------------------------------------------------*/
     %fcf_get_property_value(property=fcf.svr.aml.entityTriageDMLookback,
                             output_property_value=DMLookBack,
                             output_status=property_status);
     %if &property_status ne 0 %then %do;
        %let _fcfmsg_=WARNING: FCF: Missing property value to define number of days in lookback.  Setting to 90 days;
        %fcf_put_error;
        %let DMLookBack=90;
     %end;

     proc sql noprint;
        select date_key into :start_date_key
        from db_core.fsc_date_dim
        where calendar_date_sas=datepart(&fcf_start_datetime) - &DMLookBack;
     quit;

    /*-------------------------------------------------------------------
     * Build table of transactions in cash_flow_alert for EXT alerts
     * Note:  Need to loop through cash_flow_fact three times to pick
     *        up records for external remitters, external beneficiaries
     *        and external executing parties
     *-------------------------------------------------------------------*/
 
    %do i = 1 %to 3;

        %put FCF:NOTE:  Replicating column &&replicate_col&i;

        proc sql;
        connect to &dbflavor as &dbflavor (authdomain="&segkcAuthDomain." &segKCDBConnOpts);

        create table entity_transaction_ext&i as
        select * from 
        connection to &dbflavor
        (
           select distinct ext_acct_key.&alerted_entity_level_code as alerted_entity_level_code,
                           ext_acct_key.&alerted_entity_number as alerted_entity_number,
                           trnd.transaction_reference_number as transaction_id,
                           cfa.transaction_key,
                           cfa.currency_amount as transaction_amt,
                           trnt.transaction_cdi_desc,
                           trnt.primary_medium_desc,
                           trnt.secondary_medium_desc,
                           trnt.mechanism_desc as transaction_mechanism,
                           cfa.account_key,
                           acc.account_number,
                           '' as household_number,
                           ext.full_name as beneficiary_name,
                           assc.associate_first_name,
                           assc.associate_middle_name,
                           assc.associate_last_name,
                           brch.branch_name,
                           trnd.transaction_description as transaction_desc,
                           trns.status_desc as transaction_status,
                           cfa.time_key,
                           cfa.date_key,
                           dt.calendar_date_sas,
                           tm.time_hh,
                           tm.time_mm,
                           tm.time_ss 
            from  (select distinct transaction_key,
                          account_key,
                          transaction_type_key,
                          time_key,
                          date_key,
                          transaction_status_key,
                          branch_key,
                          associate_key,
                          &&replicate_col&i,
                          currency_amount
                    from &segKCSchema..fsc_cash_flow_fact   
                    where date_key >= &start_date_key
                          and &&replicate_col&i in 
                         (select distinct &ext_party_account_key
                          from &segKCSchema..fsk_temp_table
                          )
                    ) cfa  
           join &segKCSchema..fsk_temp_table ext_acct_key
           on ext_acct_key.&ext_party_account_key=cfa.&&replicate_col&i
           join &segKCSchema..fsc_account_dim acc
           on cfa.account_key=acc.account_key
           join &segKCSchema..fsc_transaction_dim trnd
           on cfa.transaction_key=trnd.transaction_key
           join &segKCSchema..fsc_transaction_type_dim trnt
           on trnt.transaction_type_key=cfa.transaction_type_key
           join &segKCSchema..fsc_ext_party_account_dim ext
           on cfa.&&replicate_col&i=ext.ext_party_account_key
           join &segKCSchema..fsc_associate_dim assc
           on cfa.associate_key=assc.associate_key
           join &segKCSchema..fsc_branch_dim brch
           on cfa.branch_key=brch.branch_key
           join &segKCSchema..fsc_transaction_status_dim trns
           on cfa.transaction_status_key=trns.transaction_status_key 
           join &segKCSchema..fsc_date_dim dt
           on cfa.date_key=dt.date_key
           join &segKCSchema..fsc_time_dim tm
           on cfa.time_key=tm.time_key
       );

       disconnect from &dbflavor;
       quit;

 
       %fcf_rcset(&sqlrc);
       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %let _fcfmsg_=ERROR: FCF: Failed to add create transaction table entity_transaction_ext&i;
          %fcf_put_error;
          %goto MACRO_END;
       %end;

   %end; /*else*/
%end; /* end of loop through columns */

/*------------------------------------------------------------
 * Create a work table with transactions found for all three 
 * types of external entities:
 *             EXECUTING_EXT_PARTY_KEY;
 *             REMITTER_EXT_PARTY_KEY;
 *             BENEFICIARY_EXT_PARTY_KEY;
 * Since a row in the cash flow fact table could have the 
 * same key in multiple columns, there is a possiblity of
 * duplicate transactions.  The tables created for each
 * column need to be combined and sorted to remove duplicates.
 *------------------------------------------------------------*/


data entity_transaction_ext;
if 1 eq 0 then set seg_kc.fsk_entity_transaction;
set &ext_ent_trx_list;
run;

%fcf_rcset(&syserr);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: Unable to create table of External Entity Transactions ENTITY_TRANSACTION_EXT;
    %fcf_put_error;
    %goto MACRO_END;
%end;

proc sort data=entity_transaction_ext nodupkey;
by alerted_entity_level_code alerted_entity_number transaction_id;
run;

%fcf_rcset(&syserr);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: Error sorting ENTITY_TRANSACTION_EXT;
    %fcf_put_error;
    %goto MACRO_END;
%end;

/*--------------------------------
 * Clean-up Work Directory
 *-------------------------------*/

proc datasets library=work nolist;
    %if &ext_ent_trx_list ne %str() %then %do;
       %do i=1 %to %sysfunc(countw(&ext_ent_trx_list,%str(' ')));
           delete %kscan(&ext_ent_trx_list,&i);
       %end;
    %end;
run;

%MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_ent_load_entity_txn_ext;
