/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_route_alert_queue.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    This template macro can either call the default round robin routing
|          macro or it can be used to implement custom routing logic for queues.
|          This template contains logic to route employee entities and alerted
|          entities containing a watchlist alert to EMP and WAT queues
|          respectively
+------------------------------------------------------------------------------+
| USAGE: %fcf_route_entity_queue(Lib=, DS=);
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             Lib=      SAS library containing the inDS data set
|                         i.e., STG_ALER
|             DS=       Table name to update using queue routing
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_route_alert_queue(Lib=,      /* input library name          */
                             DS=,       /* input data set name         */
                            );

   /*------------
    * Set Module
    *------------*/
   %local _fcfmodule_;
   %let _fcfmodule_=FCF_ROUTE_ALERT_QUEUE;

   /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
   %let trans_rc = 0;
   %let _fcfmsg_=;

    /* execute alert queue routing code  */
    /*------------------------------------------
        Get list of configured queue codes
        for later validation.
    *------------------------------------------*/
    proc sql noprint;
        select quote(strip(queue_code)) into :valid_queue_codes separated by ','
          from &seg_kc_libname..fsk_queue(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_queue))
        where change_current_ind='Y' and logical_delete_ind='N' and active_ind='Y';
    quit;
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to get list of valid queues;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /************************************************************************
     update alert table with sample queues for employee and watchlist alerts
     This sample code may need modifying for installation specific queues
     ************************************************************************/

    proc sql noprint;
        update &Lib..&DS.(%fcf_get_ds_opts(lib_name=&Lib, ds_name=&DS))
            set queue_code=
            case
                when queue_code = '' and employee_ind='Y'
                    then 'EMP'
                when queue_code = '' and alert_category_cd = 'WAT'
                    then 'WAT'
                else queue_code
            end;
    quit;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to update queue_code in table &Lib..&DS.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    data _null_;
        set &Lib..&DS.(
                       %fcf_get_ds_opts(lib_name=&Lib, ds_name=&DS)
                       where=(queue_code not in (&valid_queue_codes.) and queue_code)
                      );
        put 'WAR' "NING: FCF: Invalid queues being set in &_fcfmodule_. for &Lib..&DS. "
        queue_code= primary_entity_number=;
    run;


%MACRO_END:

        %if &trans_rc eq 3 or &trans_rc ge 5 %then
            %fcf_execute_macro(fcf_error);
        %else %do;
            %fcf_execute_macro(fcf_success);
        %end;

%mend fcf_route_alert_queue;
