/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_route_entity_alert.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Auto Route Alerted Entities using a round robin method to assign
|          to persons having the input role.
+------------------------------------------------------------------------------+
| USAGE: %fcf_route_entity_alert(inLib=, inDS=, pk_key=,
|                          bank_users=, role_name=, sasdataset=);
+------------------------------------------------------------------------------+
| COMMENTS: INPUTS
|             inLib=  SAS library containing the inDS data set
|                     i.e., STG_ALER
|             inDS=   Table name to update using round-robin routing
|             bank_name= The bank ID for selection of members from metadata
|             role_name= The metadata role name used to extract persons
|                     to assign entities for investigation
|------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_route_entity_alert(inLib=,           /* input library name          */
                 inDS=,                         /* input data set name         */
                 bank_users=&bankUsersGroup,    /* bank name to select members */
                 role_name=                     /* role name to extract users  */
                ) ;

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=FCF_ROUTE_ENTITY;

  /*-------------------------------------
   * Call prefcf_route_entity stub macro
   *------------------------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

  /*-----------------------------------------------
   * Extract identity information from the metadata
   *----------------------------------------------*/
   %fcf_route_entity_investigators(
                     inDS=&inDS.,                    /* input data set name         */
                     bank_users=&bank_users.,        /* bank name to select members */
                     role_name=&role_name.,          /* role name to extract users  */
                     outDS=work.investigator_numbers /* dataset to return           */
                    )
   /*------------------------------------------------------------
   * Assign investigator # to entities that do not have an owner.
   * Save the last investigator assigned for the next time auto
   * routing is done on the input table.
   *-------------------------------------------------------------*/

   /*
   create table of entities with no entry in entity_queue
   */

    proc sql noprint;
        create table need_to_route as select
        t.*, d.routed_flag
        from &inLib..&inDS(%fcf_get_ds_opts(lib_name=&inLib, ds_name=&inDS)) t
        left join
        (select distinct alerted_entity_number,
                1 as routed_flag
           from &seg_kc_libname..fsk_entity_queue(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_entity_queue))) d
        on t.alerted_entity_number=d.alerted_entity_number;
    quit;

   data need_to_route route_last_assignment (keep=investigator_num);
   set need_to_route (keep= alerted_entity_level_code
                            alerted_entity_number
                            routed_flag
                            ) END=last;
      retain assign_investigator &last_investigator_number;
      investigator_num=.;
      if missing(routed_flag)
            then do;
            if assign_investigator eq &num_investigators then assign_investigator=0;
            assign_investigator + 1;
            investigator_num=assign_investigator;
            output need_to_route;
      end;
      if last then do;
        investigator_num = assign_investigator;
        output route_last_assignment;
      end;
   run;

   /*-----------------------------------------------------------
   * Create a table of investigator names for each key in the
   * input table
   *------------------------------------------------------------*/
   proc sql;
      create table routing_info as
     select
          '' as queue_code,
          a.alerted_entity_level_code,
          a.alerted_entity_number,
       b.investigator_name as owner_userid
    from  need_to_route  A 
    left join investigator_numbers B
      on a.investigator_num=b.investigator_num
      order by 
          a.alerted_entity_level_code,
          a.alerted_entity_number; 
   quit;
      
   %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to create a routing table with investigator name;
        %fcf_put_error;
        %goto MACRO_END;
     %end;

   /*-----------------------------------------------------------------------
   * For sas datasets, sort and merge data to replace missing owner_userids
   * For database tables, do proc sql update on table
   *-----------------------------------------------------------------------*/

    proc append base=&seg_kc_libname..fsk_entity_queue(%fcf_get_ds_opts(lib_name=&seg_kc_libname, ds_name=fsk_entity_queue, output_ds=Y))
                data=routing_info;
    run;

      %fcf_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %let _fcfmsg_=ERROR: FCF: Unable to append new owners onto &seg_kc_libname..fsk_entity_queue ;
          %fcf_put_error;
          %goto MACRO_END;
      %end;


  /*---------------------------------------------------------------------------------
  * Update the control table with the  table name and the last investigator assigned
  *--------------------------------------------------------------------------------*/

  proc sql;
     delete
       from stg_ctrl.route_last_assignment(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment))
       where upcase(table_name)=upcase("&inDS");

     insert
       into stg_ctrl.route_last_assignment %fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment, add_parens=Y)
        set table_name=upcase("&inDS");

     update stg_ctrl.route_last_assignment(%fcf_get_ds_opts(lib_name=stg_ctrl, ds_name=route_last_assignment))
         set last_user_assigned=(select investigator_name
                                 from route_last_assignment a
                                 join investigator_numbers b
                                 on a.investigator_num=b.investigator_num)
         where upcase(table_name)=upcase("&inDS");
   quit;

   %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to update stg_ctrl.route_last_assignment with last investigator;
        %fcf_put_error;
        %goto MACRO_END;
   %end;

   
  /*------------------- 
  * Clean up work lib
  *-------------------*/
  proc datasets library=work nolist;
     delete investigator_numbers;
     delete need_to_route;
     delete route_last_assignment;
     delete routing_info;
  quit;

 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_route_entity_alert;


