/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_sa_ods_test_output.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    This macro is called at the end of the stored process that executes
|          the scenarios when the execution was successful. It is invoked
|          after %STPBEGIN macro was executed, so it allows customization of 
|          the output that will be sent back to the browser using ODS output.
+------------------------------------------------------------------------------+
| USAGE:   %fcf_sa_ods_test_output
|              (seg_kc_lib, runasofdate, msg_encoding, display_alerts,
|               fcf_sa_all_alerts_ds, display_transactions,
|               fcf_sa_all_trans_key)
+------------------------------------------------------------------------------+
| PARAMETERS:
|     seg_kc_lib
|         library that contains all scenario related datasest/tables
|             FSK_SCENARIO
|             FSK_HEADER
|             etc.
|
|     runasofdate - run date entered by the user
|
|     samplesize - sample size entered by the user
|
|     msg_encoding - Scenario Message Encoding selected by the user
|
|     display_alerts
|         if Y then the user checked the Display Alerts Table radio button
|
|     fcf_sa_all_alerts_ds
|         data set name that contains all the generated alerts. The data set
|         may not exist or be empty.
|
|     display_transactions
|         if Y then the user checked the Display Transactions Table radio button
|
|     fcf_sa_all_trans_key
|         data set that contains all triggering transactions for the generated 
|         alerts if they were saved. Data set may not exist or be empty. The
|         triggering transactions are only saved when display_transactions is Y.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%macro fcf_sa_ods_test_output(seg_kc_lib=work, runasofdate=, samplesize=, msg_encoding=, display_alerts=,
                              fcf_sa_all_alerts_ds=, display_transactions=, fcf_sa_all_trans_keys=, user_locale= );

    %put &abc;

    %let fcssa_msg1=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_success_msg, N));
    %let fcssa_msg2=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_parm_msg, N, &runasofdate, &samplesize, &msg_encoding ));
    %let fcssa_msg3=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_no_alerts_msg, N));
    %let fcssa_msg_chart_title=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_chart_title, N));
    %let fcssa_msg_chart_label_name=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_chart_label_name, N));
    %let fcssa_msg_chart_label_count=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_chart_label_count, N));

    %let alert_num=0;
    %if %sysfunc(exist(&fcf_sa_all_alerts_ds)) %then %do;
        proc sql noprint;
           select count(*) into :alert_num from &fcf_sa_all_alerts_ds(%fcf_get_ds_opts(ds_name=&fcf_sa_all_alerts_ds));
        quit;
    %end;

    data ok_msg;
        length msg $256;
        msg = "&fcssa_msg1";
        output;
        msg = "&fcssa_msg2";
        output;
        if &alert_num eq 0 then do;
            msg = "&fcssa_msg3";
            output;
        end;
    run;

    proc print data=ok_msg NOOBS LABEL;
        Var   msg;
        Label msg='00'x;
        Title;
    run;

    proc sql;
        create table scr_list as
        select h.header_name, s.scenario_name
        from work.fsk_header h
        inner join work.fsk_scenario s
        on s.header_id = h.header_id
        order by header_name, scenario_name
        ;
    quit;

    %let scenario_name_label=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_scenario_name_label, N));
    %let header_name_label=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_header_name_label, N));
    %let executed_scenarios_title=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_executed_scenarios_title, N));
    proc print data=scr_list NOOBS LABEL;
        Label scenario_name=&scenario_name_label;
        Label header_name=&header_name_label;
        Title "&executed_scenarios_title";
    run;

    %if &alert_num ne 0 %then %do;
        proc sql;
            create table scr_alt as
                    SELECT
                            s.scenario_name as NAME,
                            count(a.scenario_id) as ALERT_COUNT
                    FROM
                            &seg_kc_lib..&fcf_sa_all_alerts_ds(%fcf_get_ds_opts(lib_name=&seg_kc_lib, ds_name=&fcf_sa_all_alerts_ds)) a
                    JOIN    &seg_kc_lib..fsk_scenario(%fcf_get_ds_opts(lib_name=&seg_kc_lib, ds_name=fsk_scenario)) s
                    ON a.scenario_id = s.scenario_id
                    GROUP BY name
                    ORDER BY name, ALERT_COUNT
                    ;
        quit;

        %let max_alert_num=0;
        proc sql noprint;
            select max(ALERT_COUNT) into :max_alert_num from scr_alt;
        quit;

        %put &max_alert_num;
/* order=(0 to &max_alert_num); */
/* TODO: graph does not work for Chrome when STP is invoked via rest service
        %if %sysprod(GRAPH) = 1 %then %do;
            axis1 label=NONE value=(height=2) minor=none;
            axis2 label=none ;
            title "&fcssa_msg_chart_title" ;
            proc gchart data=scr_alt ;
                label name="&fcssa_msg_chart_label_name";
                label ALERT_COUNT="&fcssa_msg_chart_label_count";

                vbar3d name / sumvar=ALERT_COUNT
                        raxis=axis1
                        maxis=axis2;
            run;
            quit;
        %end;
        %else  %do;
            proc print data=scr_alt label;
                label name="&fcssa_msg_chart_label_name";
                label ALERT_COUNT="&fcssa_msg_chart_label_count";
                Title "&fcssa_msg_chart_title";
            run;
        %end;
*/
        proc print data=scr_alt label;
            label NAME="&fcssa_msg_chart_label_name";
            label ALERT_COUNT="&fcssa_msg_chart_label_count";
            Title "&fcssa_msg_chart_title";
        run;

        %if &display_alerts eq Y %then %do;
            %let fcssa_msg4=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_alerts_hdr, N));
            %let rundate_label=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_rundate_label, N));
            %let act_val_label=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_act_val_text_label, N));
            %let subj_type=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_subj_type_label, N));
            %let subj_num=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_subj_num_label, N));
            %let scenario_id=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_scenario_id_label, N));
            %let scenario_name=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_scenario_name_label, N));

            %fcf_get_aml_lookup_table(tabName=RT_ENTITY_LEVEL, locale=&user_locale);

            proc sql;
                create table alerts
                as select t.rundate, t.actual_values_text,
                          coalesce(l.LOV_TYPE_DESC,  t.entity_level_code) as entity_level,
                          t.primary_entity_number, t.scenario_id,
                          s.scenario_name
                from &seg_kc_lib..&fcf_sa_all_alerts_ds(%fcf_get_ds_opts(lib_name=&seg_kc_lib, ds_name=&fcf_sa_all_alerts_ds)) t
                left join &seg_kc_lib..fsk_scenario(%fcf_get_ds_opts(lib_name=&seg_kc_lib, ds_name=fsk_scenario)) s
                ON t.scenario_id = s.scenario_id
                left join RT_ENTITY_LEVEL l
                ON t.entity_level_code = l.LOV_TYPE_CODE
                ;
            quit;

            proc print data=alerts label;
                label rundate=&rundate_label;
                label actual_values_text=&act_val_label;
                label entity_level=&subj_type;
                label primary_entity_number=&subj_num;
                label scenario_id=&scenario_id;
                label scenario_name=&scenario_name;
                Title "&fcssa_msg4";
            run;
            Title;
        %end;

        %if &display_transactions eq Y and %sysfunc(exist(&fcf_sa_all_trans_keys)) %then %do;
            %let fcssa_msg5=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_sp_trans_hdr, N));
            %let subj_num=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_subj_num_label, N));
            %let scenario_id=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_scenario_id_label, N));
            %let trn_key=%sysfunc(sasmsg(sashelp.fcfmsg, sa_test_alerts_trn_key_label, N));
            proc print data=&seg_kc_lib..&fcf_sa_all_trans_keys(%fcf_get_ds_opts(lib_name=&seg_kc_lib, ds_name=&fcf_sa_all_trans_keys)) label;
                label primary_entity_number=&subj_num;
                label scenario_id=&scenario_id;
                label transaction_key=&trn_key;
                Title "&fcssa_msg5";
            run;
            Title;
        %end;
    %end;
%mend fcf_sa_ods_test_output;
