/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_sar_90_day_review_process.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Analyze filed SARs and create AML cases for review in 90 days
+------------------------------------------------------------------------------+
| USAGE:    requires AML autoxec to be executed before invoking
|           %fcf_sar_90_day_review_process()
|           Parameters
|                    worklib - optional, default is WORK library, change it if
|                              need to save intermediate data sets for debugging
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2017 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------------
 * This macro is intended to be used in a batch process that is executed at least once every 90 days
 * and performs analysis of SARs filed in the last 90 days.
 * This program can be run daily as a batch process (see fcf.svr.sar90.lookbackDays preference).
 * It requires cdd_autoexec be executed.
 * The following actions are executed:
 *   	When a SAR is submitted to FinCEN, i.e. efile has been generated, report status is set
 *      to G (generated) and when efile is accepted then report status may be set to A (accepted).
 *      The program analyzes all SAR reports submitted to FinCEN (status A) in the last N days
 *      (N is the value of fcf.svr.sar90.lookbackDays) and captures the parties that were used on
 *      the SAR as suspects.  Only suspects from original SAR based on a case are used, not an
 *      amended SARs.
 *      Report DCN is saved in the case summary field.
 *      New case will be assigned to the same investigator who owned the original SAR case.
 *
 *      New SAR90 case (in suspended status - S) is created for each report. All report suspects
 *      are linked to the new case. Original report is linked to a case using FSK_RC_REPORT.SAR90_CASE_ID.
 *      Efile transmission date will be used as sar90 create date  and this date will be used for
 *      counting number of days the case has been suspended.
 *
 *	    All SAR90 cases that are past the suspension period defined by fcf.svr.sar90.suspendedDays
 *      preference are activated (status set to O – open) and the activation date is set to the current date.
 *
 * Preferences:
 *
 * fcf.svr.sar90.lookbackDays=90
 *       This value defines how many days back the SARs are analyzed. It should be set to a value
 *       greater than the run frequency of fcf_sar_90_day_review_process.sas.
 *
 *  fcf.svr.sar90.suspendedDays=60
 *       This value defines how long SAR case stays suspended. After the case is activated the
 *       countdown starts from the activation date. This value is also used to calculate maximum
 *       countdown for SAR90 cases, it is calculated as sum of this value and caseCountdown.countdownMaximum
 *
 * The following does not apply to default AML 7.1 behaviour.
 * By default AML 7.1 uses new_case_for_each_report=Y
 *  ****  6.3M1 like behaviour when new_case_for_each_report=N *****
 * If there is suspended or active existing SAR90 (case category) case  with one of the
 * SAR suspects attached to it then all SAR suspects will be attached to the existing SAR90 case.
 * If multiple not closed SAR90 cases found for the suspects from the same report then the oldest
 * case will be used for all report suspects.
 * If there is no existing SAR90 case then a new SAR90 case will be created with suspended status
 * with all the SAR suspects attached to new SAR90 case.
 * Data integrity, duplicates:
 *    If the same entity can be attached to multiple existing cases, then attach only to the case with
 *    the smallest ID - i.e. the oldest case
 *    This situation would not happen if SAR90 cases are only created and updated by this batch process,
 *    but manual updates to the cases and reports may break the integrity of the data.
 *    If the same entity is attached to multiple reports then it will be attached only to one
 *    SAR90 case, earliest report or earliest case in most cases.
 *    This situation typically is not expected but may happen if SAR report volume is high.
 * ------------------------------------------------------------------------------------------- */

%macro fcf_sar_90_day_review_process(worklib=work);

    %let rr_filed_ready_cd='A';                    /* list of report status values for FILED reports */
    %let rr_type_cd_lst='SAR' 'SARX';              /* list of SAR type codes in FORM_CONFIG */
    %let SAR90_case_suspended_status=S;            /* code for suspended case status */
    %let fcf_case_active_status=O;                 /* code for active/open case status */
    %let aml_inv_type=AML;                         /* AML case type */
    %let aml_sar90_cat=SAR90;                      /* AML SAR90 case category */
    %let aml_pty_level_list='PTY' 'EXT';                 /* List of entity levels for report suspects*/
    %let sar90_create_date=TRANSMISSION_DATE;      /* use either TRANSMISSION_DATE or EFILE_CREATE_DATE */
    %let new_case_for_each_report=Y;               /* if set to Y then each new report will have new sar90
                                                    * case created, if N then report might be attached to an existing case */

    /* read meatadata settings */
    %global fcf_SAR_num_of_days_to_process;        /* number of days to look back for filed SARs */
    %global fcf_SAR90_case_suspended_days;         /* number of days SAR90 case stays suspended */

    %fcf_get_property_value(property=fcf.svr.sar90.lookbackDays,
                            output_property_value=fcf_SAR_num_of_days_to_process,
                            output_status=prop_status);
    %if &prop_status ne 0 %then %do;
        %put WARNING: Number of days to look back for SARs not found in preferences. Using default value 90 days.;
        %let fcf_SAR_num_of_days_to_process=90;
    %end;
    %put NOTE: All reports less than &fcf_SAR_num_of_days_to_process days will be analyzed.;


    %fcf_get_property_value(property=fcf.svr.sar90.suspendedDays,
                            output_property_value=fcf_SAR90_case_suspended_days,
                            output_status=prop_status);
    %if &prop_status ne 0 %then %do;
        %put WARNING: Number of days to suspend SAR review cases not found in preferences. Using default value 90 days.;
        %let fcf_SAR90_case_suspended_days=90;
    %end;
    %put NOTE: All SAR90 case older than &fcf_SAR90_case_suspended_days days will be activated.;

    /* reset error codes to zero */
    %let trans_rc = 0;
    %let sqlrc = 0;
    %let sqlxrc = 0;
    %let sysrc = 0;

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;

    /* Get the column names of fsk_case_entity */
    proc contents data=seg_kc.fsk_case_entity out=fsk_case_entity_columns (keep=name) noprint;
    run;
    /* Get the column names of fsk_case_entity_event */
    proc contents data=seg_kc.fsk_case_entity_event out=fsk_case_entity_event_columns (keep=name) noprint;
    run;

    %let fsk_case_entity_keep_list=;
    %let fsk_case_entity_event_keep_list=;

    proc sql noprint;
        SELECT distinct upcase(name) INTO :fsk_case_entity_keep_list separated by ' '
            FROM fsk_case_entity_columns
        ;
        SELECT distinct upcase(name) INTO :fsk_case_entity_event_keep_list separated by ' '
            FROM fsk_case_entity_event_columns
        ;
    quit;


    /*  Find all filed SARs.
     *  New SARs have FSK_RC_REPORT.SAR90_CASE_ID equal to NULL
     *  and  REPORT_READY_CODE equal to 'A' if efile was accepted
     *  Bridge table fsk_rc_efile_report may have multiple records for the same report key,
     *  use the one with max efile key (latest)
    */

    proc sql;
        create table &worklib..all_filed_sar as
            select
                r.*,
                e.EFILE_KEY,
                e.EFILE_STATUS_CODE,
                e.TRANSMISSION_DATE,
                e.CREATE_DATE as EFILE_CREATE_DATE,
                c.OWNER_USER_LONG_ID,
                er.REPORT_AGENCY_REF_ID
            from
                seg_kc.fsk_rc_report r
                inner join seg_kc.fsk_rc_efile_report er
                      on r.report_key = er.report_key
                inner join seg_kc.fsk_rc_efile e
                      on er.efile_key = e.efile_key
                inner join seg_kc.fsk_case c
                      on c.LOGICAL_DELETE_IND ne 'Y'
                         and r.PARENT_OBJECT_NAME='CASE'
                         and c.CASE_ID = r.PARENT_OBJECT_KEY
            where   r.sar90_case_id is null
                and r.correction_ind ne 'Y'
                and r.form_config_key in
                    ( select form_config_key  from seg_kc.fsk_rc_form_config
                                  where form_type_code in (&rr_type_cd_lst) )
                and report_ready_code in (&rr_filed_ready_cd)
            order by r.report_key,  e.efile_key DESC
        ;
    quit;
    %let all_sar_cnt = &SQLOBS;

    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to retrieve filed SARs;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /*  Keep only the latest efile, i.e. max efile_key.
     */
    proc sort data=&worklib..all_filed_sar out=&worklib..all_filed_sar_latest nodupkey;
         by report_key;
    run;

    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to process latest SARs;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /*  drop SARs outside look back period.
     */
    data &worklib..all_new_sar;
         set &worklib..all_filed_sar_latest;
            if (today() - datepart(TRANSMISSION_DATE)) < &fcf_SAR_num_of_days_to_process then output;
    run;

    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to process SARs for lookback period;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /****************************************************/
    /* if there is no new SARs then just activate cases */
    /****************************************************/
    %if &SYSNOBS le 0 %then %goto  ACTIVATE_SAR_CASES;

    /* create empty table to track cases assigned to reports */
    proc datasets lib = seg_kc nolist nowarn memtype = (data view);
       delete temp_rc_report;
    quit;
    proc sql;
        create table seg_kc.temp_rc_report
        as select REPORT_KEY,
                  SAR90_CASE_ID as CASE_ID
        from  seg_kc.fsk_rc_report
        where report_key IS NULL;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to create seg_kc.temp_rc_report table;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /* get all suspects attached to new SARs */
    proc sql;
        create table &worklib..rr_suspects_all_1 as
            select
                    re.REPORT_KEY,
                    re.ENTITY_LEVEL_CODE,
                    re.ENTITY_KEY,
                    re.ENTITY_NUMBER,
                    re.ENTITY_NAME,
                    p.EMPLOYEE_IND
            from  seg_kc.FSK_RC_REPORT_ENTITY re
            left join seg_kc.fsc_party_dim p
                      on re.ENTITY_KEY = p.PARTY_KEY
                         and re.ENTITY_LEVEL_CODE='PTY'
                         and p.CHANGE_CURRENT_IND = 'Y'
            where     re.REPORT_KEY in (select REPORT_KEY from &worklib..all_new_sar)
                  and re.LOGICAL_DELETE_IND ne 'Y'
                  and re.ENTITY_LEVEL_CODE in (&aml_pty_level_list)
        ;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to retrieve SAR suspects;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /* add efile dates - this will be needed to create SAR90 case with the same date */
    proc sql;
        create table &worklib..rr_suspects_all as
            select
                    re.REPORT_KEY,
                    re.ENTITY_LEVEL_CODE,
                    re.ENTITY_KEY,
                    re.ENTITY_NUMBER,
                    re.ENTITY_NAME,
                    re.EMPLOYEE_IND,
                     r.EFILE_CREATE_DATE,
                     r.TRANSMISSION_DATE,
                     r.OWNER_USER_LONG_ID,
                     r.REPORT_AGENCY_REF_ID
            from  &worklib..rr_suspects_all_1 re
            left join &worklib..all_new_sar r
                     on re.report_key = r.report_key
            order by re.REPORT_KEY;
        ;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to join with efile info;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /* for 7.1 map each new SAR report suspects to a new SAR9 case
     * To enable behaviour similar to 6.3 M1 set  new_case_for_each_report value to  N
     */
    %if &new_case_for_each_report eq Y %then %do;
        /* keep all suspects, even duplicates */
        data &worklib..rr_suspects_no_cases;
           set &worklib..rr_suspects_all;
           CASE_ID = .;
        run;
        %GOTO PROCESS_NEW_SAR90_CASES;
    %end;

/*************************************************************************************/
/* The following section is not executed for AML 7.1 */

    /* find all suspects that are already exists on open SAR90 cases */
    proc sql;
        create table &worklib..existing_SAR90C_suspects as
            select ce.case_id,
                   ce.ENTITY_LEVEL_CODE,
                   ce.ENTITY_KEY,
                   ce.ENTITY_NUMBER
            from seg_kc.fsk_case c
                 inner join seg_kc.FSK_CASE_ENTITY ce
                       on c.case_id = ce.case_id
                       and c.CASE_TYPE_CODE = "&aml_inv_type"
                       and c.CASE_CATEGORY_CODE = "&aml_sar90_cat"
                       and (    c.CASE_STATUS_CODE not in (&fcf_closed_case_status_lst)
                    	     or c.CASE_STATUS_CODE is null )
                       and c.LOGICAL_DELETE_IND ne 'Y'
                       and ce.ENTITY_LEVEL_CODE in (&aml_pty_level_list)
        ;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to retrieve filed SAR cases suspects;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    /* map new sar90 suspects to existing cases */
    proc sql;
        create table &worklib..rr_suspect_x_case as
          select rs.*, cs.CASE_ID
          from &worklib..rr_suspects_all rs
               left join &worklib..existing_SAR90C_suspects cs
                    on      rs.ENTITY_LEVEL_CODE = cs.ENTITY_LEVEL_CODE
                       and  rs.ENTITY_NUMBER = cs.ENTITY_NUMBER
          order by  ENTITY_LEVEL_CODE, ENTITY_NUMBER, CASE_ID, REPORT_KEY
        ;
    quit;

    /* drop duplicate records if the same entity can be attached to multiple cases,
     * leave the case with smallest ID - oldest case
     * This will also drop entity from one of the reports if it is attached to multiple reports.
     * Both situation should not happen if SAR90 case are only updated by this batch process,
     * but manual updates to the cases may break the integrity of the data.
    */
    proc sort data=&worklib..rr_suspect_x_case out=&worklib..rr_suspect_x_case_nodup NODUPKEY;
        by  ENTITY_LEVEL_CODE ENTITY_NUMBER;
    run;

    /* for each new report attach all suspects from the same report to the same existing case
     * find suspects that have no existing case to be attached to
     * keep record all report attached to case to update FSK_RC_REPORT.SAR90_CASE_ID
    */

    proc sort data=&worklib..rr_suspect_x_case_nodup;
        by REPORT_KEY descending CASE_ID;
    run;

    data &worklib..rr_suspect_x_case_per_report
         &worklib..rr_suspects_no_cases
         &worklib..report_x_existing_case (keep=report_key case_id);

         set &worklib..rr_suspect_x_case_nodup;
         by REPORT_KEY;

         length report_case_id 8;
         retain report_case_id 0;
         drop report_case_id;

         if  first.REPORT_KEY then do;
             report_case_id = case_id;
             if case_id ne . then output &worklib..report_x_existing_case;
         end;
         case_id = report_case_id;

         if case_id eq .
         then output &worklib..rr_suspects_no_cases;
         else output &worklib..rr_suspect_x_case_per_report;
    run;

    proc append base=seg_kc.temp_rc_report data=&worklib..report_x_existing_case FORCE;
    run;

    /* do not attach suspects from new reports that are already attached to existing cases
     * to avoid duplicate relationships
     */
    proc sql;
        delete from &worklib..rr_suspect_x_case_per_report rs
        where exists (select 1 from &worklib..existing_SAR90C_suspects cs
                               where    rs.case_id = cs.case_id
                                    and rs.ENTITY_LEVEL_CODE = cs.ENTITY_LEVEL_CODE
                                    and rs.ENTITY_NUMBER = cs.ENTITY_NUMBER)
        ;
    quit;

    %let new_entity_count = %fcf_get_observation_count(table=&worklib..rr_suspect_x_case_per_report, nonotes=N, sasdataset=Y);

    /* ****************************************************************** */
    /*  attach entities to existing cases                                 */
    /* ****************************************************************** */
    %if &new_entity_count eq 0 %then %GOTO PROCESS_NEW_SAR90_CASES;

    data &worklib..new_case_entity (keep=&fsk_case_entity_keep_list)
         &worklib..new_case_entity_event (keep=&fsk_case_entity_event_keep_list);
         if 0 then
            SET seg_kc.fsk_case_entity
                seg_kc.fsk_case_entity_event;

        set &worklib..rr_suspect_x_case_per_report;
        /*  already populated from the input dataset
         *  CASE_ID
         *  ENTITY_KEY
         *  ENTITY_LEVEL_CODE
         *  ENTITY_NAME
         *  ENTITY_NUMBER
         */
           RELATION_DESCRIPTION = 'Suspect on report ' || trim(left(put(REPORT_KEY, BEST32.)));
           LSTUPDATE_DATE = ("&current_dt"dt);
           LSTUPDATE_USER_ID = "&SYSUSERID";
           PRIMARY_SUSPECT_IND = 'N';
           RR_IND = 'N';
           VERSION_NUMBER = 1;
           output &worklib..new_case_entity;

           EVENT_ID = .;
           EVENT_TYPE_CODE = 'ADD';
           CREATE_USER_ID = "&SYSUSERID";
           CREATE_DATE = ("&current_dt"dt);
           EVENT_DESCRIPTION = "Add report " || trim(left(put(REPORT_KEY, BEST32.)))
                               || " suspect " || trim(ENTITY_LEVEL_CODE)
                               || " - " || trim(ENTITY_NUMBER) || " - " || trim(ENTITY_NAME);
           output &worklib..new_case_entity_event;
    run;

    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to create new relationships for SAR suspects;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    proc append base=seg_kc.fsk_case_entity data=&worklib..new_case_entity FORCE;
    run;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to append new relationships for SAR suspects;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    %let num_events = %fcf_get_observation_count(table=&worklib..new_case_entity_event, nonotes=N, sasdataset=Y);
    %put NOTE: Number of event ids requested: &num_events;
    %if &num_events ne 0 %then %do;
        /*--------------------------------
        * get sequence IDs for new records
        *--------------------------------*/
        %fcf_get_sequence_ids(numIds=&num_events, globals=first);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to obtain event ids from &seg_kc_libname..FSK_SEQ.;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        data &worklib..new_case_entity_event;
            set &worklib..new_case_entity_event;
            EVENT_ID = _n_ + &firstSeqID -1;
        run;

        proc append base=seg_kc.fsk_case_entity_event data=&worklib..new_case_entity_event FORCE;
        run;
        %fcf_chkrc
        %if &bat_abort=Y %then %do;
            %let _fcfmsg_=ERROR: FCF: SAR90: Unable to append events for SAR suspects;
            %fcf_put_error;
            %goto MACRO_END;
        %end;
    %end;

    /****************************************************/
    /* create new SAR90 cases                           */
    /****************************************************/
%PROCESS_NEW_SAR90_CASES:

    /* get new cases count */
    %let num_new_cases=0;
    proc sql noprint;
      select count(distinct report_key) into :num_new_cases from &worklib..rr_suspects_no_cases;
    quit;

    /* if there is no new SARs then check if FSK_RC_REPORT table update if needed and then activate SAR90 cases */
    %if &num_new_cases eq 0 %then %goto  FINALIZE_SAR_PROCESS;

    /*--------------------------------
    * get sequence IDs for new records
    *--------------------------------*/
    %fcf_get_sequence_ids(numIds=&num_new_cases, globals=first);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: Unable to obtain case ids from &seg_kc_libname..FSK_SEQ.;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /* get the column names of fsk_case */
    proc contents data=seg_kc.fsk_case out=fsk_case_columns (keep=name) noprint;
    run;
    /* get the column names of fsk_case_event */
    proc contents data=seg_kc.fsk_case_event out=fsk_case_event_columns (keep=name) noprint;
    run;

    %let fsk_case_keep_list=;
    %let fsk_case_event_keep_list=;

    proc sql noprint;
        SELECT distinct upcase(name) INTO :fsk_case_keep_list separated by ' '
            FROM fsk_case_columns
        ;
        SELECT distinct upcase(name) INTO :fsk_case_event_keep_list separated by ' '
            FROM fsk_case_event_columns
        ;
    quit;

    /* Note: duplicate parties are already removed when rr_suspect_x_case_nodup was created.  */
    /* Create records to add new cases, their entities and events */
    data &worklib..new_sar90_case (keep=&fsk_case_keep_list)
         &worklib..new_sar90_case_event (keep=&fsk_case_event_keep_list)
         &worklib..new_sar90_case_entity (keep=&fsk_case_entity_keep_list)
         &worklib..new_sar90_case_entity_event (keep=&fsk_case_entity_event_keep_list)
         &worklib..report_x_new_case  (keep=report_key case_id);

         if 0 then
            SET seg_kc.fsk_case_entity
                seg_kc.fsk_case_entity_event
                seg_kc.fsk_case
                seg_kc.fsk_case_event
                ;

        SET &worklib..rr_suspects_no_cases;
        by REPORT_KEY;

        length report_case_id case_count 8;
        retain report_case_id case_count 0;
        retain suspect_employee_ind;
        drop report_case_id case_count suspect_employee_ind;

        if  first.REPORT_KEY then do;
            case_count = case_count+1;
            report_case_id = case_count + &firstSeqID - 1;
        end;
        /*  already populated:
         *  ENTITY_KEY
         *  ENTITY_LEVEL_CODE
         *  ENTITY_NAME
         *  ENTITY_NUMBER
         */

        CASE_ID = report_case_id;

        if  first.REPORT_KEY then do;
            suspect_employee_ind = 'N';
        end;
        if employee_ind ='Y' then suspect_employee_ind = employee_ind;

        if  last.REPORT_KEY then do;
            /* create case */
            CASE_CATEGORY_CODE="&aml_sar90_cat";
            CASE_SUB_CATEGORY_CODE="&aml_sar90_cat";
            CASE_PRIORITY_CODE="H";
            CASE_TYPE_CODE="&aml_inv_type";
            LE_CONTACTED_IND="N";
            CREATE_DATE  = &sar90_create_date;
            CREATE_USER_ID = "&SYSUSERID";
            OPENED_IND = 'Y';
            MODIFIED_IND='N';
            EMPLOYEE_IND = suspect_employee_ind; /* use FSC_PARTY_DIM value, if one report party has employee_ind = 'Y' */
            CASE_STATUS_CODE="&SAR90_case_suspended_status";
            VERSION_NUMBER=1;
            LOGICAL_DELETE_IND='N';
            LSTUPDATE_DATE = ("&current_dt"dt);
            CASE_DESCRIPTION="SAR90 for report " || trim(left(put(REPORT_KEY, BEST32.)));
            CASE_SUMMARY= trim(CASE_DESCRIPTION) || ". Report DCN=" || trim(left(REPORT_AGENCY_REF_ID));
            /* nullable FSK_CASE columns
                CASE_CLOSE_REASON_CODE
                AGGREGATE_AMT
                LE_CONTACT_AGENCY
                LE_CONTACT_NAME
                LE_CONTACT_PHONE
                LE_CONTACT_PHONE_EXT
                LE_CONTACT_DATE
                FIRST_TRANSACTION_DATE
                LAST_TRANSACTION_DATE
            */
            LSTUPDATE_USER_ID = "&SYSUSERID";
            /* OWNER_USER_LONG_ID use the value pulled from original report case */
            output &worklib..new_sar90_case;

            /* add case event */
            /*  already populated
                CASE_ID
                CREATE_USER_ID
            */
            CREATE_DATE  = ("&current_dt"dt);
            EVENT_DESCRIPTION = "SAR90 case for report " || trim(left(put(REPORT_KEY, BEST32.)));
            EVENT_ID = CASE_ID; /* it is safe to re-use case_id because the tables use the same sequence */
            EVENT_TYPE_CODE = "ADD";
            output &worklib..new_sar90_case_event;

            output &worklib..report_x_new_case ;
        end;

        /* add entity to case relationships */
        CASE_ID = report_case_id;
        RELATION_DESCRIPTION = 'Suspect on report ' || left(put(REPORT_KEY, BEST32.));
        LSTUPDATE_DATE = ("&current_dt"dt);
        LSTUPDATE_USER_ID = "&SYSUSERID";
        PRIMARY_SUSPECT_IND = 'N';
        RR_IND = 'N';
        VERSION_NUMBER = 1;
        output &worklib..new_sar90_case_entity;

        /* add entity to case relationships events */
        CASE_ID = report_case_id;
        EVENT_ID = .;
        EVENT_TYPE_CODE = 'ADD';
        CREATE_USER_ID = "&SYSUSERID";
        CREATE_DATE = ("&current_dt"dt);
        EVENT_DESCRIPTION = "Report " || trim(left(put(REPORT_KEY, BEST32.)))
                            || " Suspect " || trim(ENTITY_LEVEL_CODE)
                            || " - " || trim(ENTITY_NUMBER) || " - " || trim(ENTITY_NAME);
        output &worklib..new_sar90_case_entity_event;


    run;

    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to create new records for SAR 90 cases;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    proc append base=seg_kc.temp_rc_report data=&worklib..report_x_new_case FORCE;
    run;


    proc append base=seg_kc.fsk_case data=&worklib..new_sar90_case FORCE;
    run;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to append new SAR 90 cases;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    proc append base=seg_kc.fsk_case_event data=&worklib..new_sar90_case_event FORCE;
    run;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to append new SAR 90 case events;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    proc append base=seg_kc.fsk_case_entity data=&worklib..new_sar90_case_entity FORCE;
    run;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
		%let _fcfmsg_=ERROR: FCF: SAR90: Unable to append new relationships for SAR suspects;
		%fcf_put_error;
		%goto MACRO_END;
    %end;

    %let num_events = %fcf_get_observation_count(table=&worklib..new_sar90_case_entity_event, nonotes=N, sasdataset=Y);
    %put NOTE: Number of event ids requested: &num_events;
    %if &num_events ne 0 %then %do;
        /*--------------------------------
        * get sequence IDs for new records
        *--------------------------------*/
        %fcf_get_sequence_ids(numIds=&num_events, globals=first);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to obtain event ids from &seg_kc_libname..FSK_SEQ.;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        data &worklib..new_sar90_case_entity_event;
            set &worklib..new_sar90_case_entity_event;
            EVENT_ID = _n_ + &firstSeqID -1;
        run;

        proc append base=seg_kc.fsk_case_entity_event data=&worklib..new_sar90_case_entity_event FORCE;
        run;
        %fcf_chkrc
        %if &bat_abort=Y %then %do;
            %let _fcfmsg_=ERROR: FCF: SAR90: Unable to append events for SAR suspects;
            %fcf_put_error;
            %goto MACRO_END;
        %end;
    %end;

%FINALIZE_SAR_PROCESS:
    /* update Sar90_case_id for all reports that have been linked to SAR90 cases */

    proc sql feedback;
        connect to &dbflavor as &dbflavor (authdomain="&segkcauthdomain." &segKCDBConnOpts);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: SAR90: Cannot connect to RDBMS.;
             %fcf_put_error;
             %goto MACRO_END;
        %end;
        execute (
            merge into fsk_rc_report r
                  using ( select REPORT_KEY, CASE_ID from temp_rc_report) tmp
            on (tmp.REPORT_KEY = r.REPORT_KEY)
            when matched then update set
                r.SAR90_CASE_ID  = tmp.CASE_ID
            %if %upcase(&dbflavor) eq ODBC %then %do;
                ;
            %end;
        ) by &dbflavor;
        disconnect from &dbflavor;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to update FSK_RC_REPORT;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

%ACTIVATE_SAR_CASES:

    proc sql;
        create table &worklib..suspended_cases as
            select case_id, create_date, ACTIVATE_DATE, case_status_code
            from seg_kc.fsk_case
            where case_status_code = "&SAR90_case_suspended_status"
        ;
    quit;
    %if &SQLOBS eq 0 %then %do;
       %put NOTE: There are no suspended SAR90 cases.;
       %GOTO MACRO_END;
    %end;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to get suspended SAR90 cases;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    data &worklib..sar90_cases_to_activate (keep=case_id ACTIVATE_DATE case_status_code);
         set &worklib..suspended_cases;
            ACTIVATE_DATE=datetime();
            case_status_code = "&fcf_case_active_status";
            if (today() - datepart(create_date)) > &fcf_SAR90_case_suspended_days then output;
    run;

    %if &SYSNOBS le 0 %then %do;
       %put NOTE: There are no suspended SAR90 cases to activate.;
       %GOTO MACRO_END;
    %end;
    %else %do;
       %put NOTE: There are &SYSNOBS suspended SAR90 cases to activate.;
    %end;

    /* create empty table to ACTIVATE cases */
    proc datasets lib = seg_kc nolist nowarn memtype = (data view);
       delete temp_CASE;
    quit;
    proc sql;
        create table seg_kc.temp_CASE
        as select CASE_ID,
                  ACTIVATE_DATE,
                  case_status_code
        from  seg_kc.fsk_CASE
        where CASE_ID IS NULL;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to create seg_kc.temp_CASE table;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc append base=seg_kc.temp_CASE data=&worklib..sar90_cases_to_activate FORCE;
    run;
   %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to append to seg_kc.temp_CASE table;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc sql feedback;
        connect to &dbflavor as &dbflavor (authdomain="&segkcauthdomain." &segKCDBConnOpts);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: SAR90: Cannot connect to RDBMS.;
             %fcf_put_error;
             %goto MACRO_END;
        %end;
        execute (
            merge into fsk_case c
                  using ( select case_id, ACTIVATE_DATE, case_status_code from temp_case) tmp
            on (tmp.case_id = c.case_id)
            when matched then update set
                c.ACTIVATE_DATE  = tmp.ACTIVATE_DATE,
                c.case_status_code = tmp.case_status_code
            %if %upcase(&dbflavor) eq ODBC %then %do;
                ;
            %end;
        ) by &dbflavor;
        disconnect from &dbflavor;
    quit;
    %fcf_chkrc
    %if &bat_abort=Y %then %do;
        %let _fcfmsg_=ERROR: FCF: SAR90: Unable to update activated cases in FSK_CASE;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    /* add records to fsk_case_event */
    data &worklib..new_activate_case_event (keep=&fsk_case_event_keep_list);

        if 0 then SET seg_kc.fsk_case_event;

        SET &worklib..sar90_cases_to_activate;
            /* add case event */
            /*  already populated
                CASE_ID
            */
            CREATE_USER_ID = "&SYSUSERID";
            CREATE_DATE  = datetime();
            EVENT_DESCRIPTION = "Activate SAR90 case.";
            EVENT_ID = .;
            EVENT_TYPE_CODE = "ACT";
            output;
     run;
    %let num_events = &SYSNOBS;
    %put NOTE: Number of event ids requested: &num_events;
    %if &num_events ne 0 %then %do;
        /*--------------------------------
        * get sequence IDs for new records
        *--------------------------------*/
        %fcf_get_sequence_ids(numIds=&num_events, globals=first);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: FCF: Unable to obtain event ids from &seg_kc_libname..FSK_SEQ.;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        data &worklib..new_activate_case_event;
            set &worklib..new_activate_case_event;
            EVENT_ID = _n_ + &firstSeqID -1;
        run;

        proc append base=seg_kc.fsk_case_event data=&worklib..new_activate_case_event FORCE;
        run;
        %fcf_chkrc
        %if &bat_abort=Y %then %do;
            %let _fcfmsg_=ERROR: FCF: SAR90: Unable to append events for activated SAR90 cases;
            %fcf_put_error;
            %goto MACRO_END;
        %end;
    %end;

%MACRO_END:

%mend fcf_sar_90_day_review_process;
