/* MAINDOC
+-------------------------------------------------------------------------------------------------------+
| NAME:    fcf_solr_index_delete.sas
| PRODUCT: SAS Anti-Money Laundering
| SYSTEM:  Windows, Unix
| DESC:    Call SOLR rest service to delete index entities after they are removed from database. 
|          Macro will execute curl command directly to the Solr REST service and perform an update 
|          and pass an XML file with the delete and id statements.
|          The Solr url, port and collection will be queries from the AML preferences, so  
|          version of fcf_autoexec will determine the Solr collection to delete records.
+-------------------------------------------------------------------------------------------------------+
| USAGE: (R) entity_type   = the type of entity to remove (ex. case, alert, etc.)
|        (R) id_dataset    = dataset with list of primary keys to delete. 
|        (R) id_variable   = the unique primary key variable for the delete (ex. case-id) 
|            work_location = directory to store deletion xml and log file from delete. Will retrieve from 
|                            option, then preferences and last resort will be SASWORK.
|            curl_command  = full pathname to curl executable. For Windows, should be the curl.exe
|                            pathname.
|            cert_file     = for SSL enable https Solr. Pass the certificate file for connection. 
|            debug         = (T or F).  Enabled extra tracing output.
|
| *R will denote a required parameter.
|
| Example:
|     %fcf_solr_index_delete ( entity_type=case
|                             ,id_dataset=work.case_delete
|                             ,id_variable=case_id
|						      ,work_location=/usr/tmp/solr/deletes
|						      ,cert_file=/install/solr-5.3.1/server/etc/solr-ssl.cacert.pem
|						      ,curl_command=/usr/bin/curl
|						      ,debug=T );
+--------------------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2019 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
%macro fcf_solr_index_delete ( entity_type=
							  ,id_dataset=
							  ,id_variable=
							  ,work_location=
							  ,curl_command=
							  ,cert_file=
							  ,debug=
                              );
 

    /* Set Module, message and return code. */
    %let trans_rc=0;
    %let _fcfmsg_=;
    %local _fcfmodule_;
    %let _fcfmodule_=FCF_SOLR_INDEX_DELETE;

    /* set some variables based on OS type */
    %if &sysscp = WIN %then %do;
       %let slash = %quote(\);
    %end;
    %else %do;
       %let slash = %quote(/);
    %end;


	/* Review list of required parameters to ensure they are populated. */
	%if %nrbquote(&entity_type) = %nrbquote() %then %do;
        %let _fcfmsg_=ERROR: Parameter entity_type is required.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;
	%if %nrbquote(&id_dataset) = %nrbquote() %then %do;
        %let _fcfmsg_=ERROR: Parameter id_dataset is required.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;
	%if %nrbquote(&id_variable) = %nrbquote() %then %do;
        %let _fcfmsg_=ERROR: Parameter id_variable is required.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;

	/* Make sure can get to the id dataset. */
	%let check_dset=%sysfunc(exist(&id_dataset.));
	%if &check_dset = 0 %then %do;
        %let _fcfmsg_=ERROR: Dataset &id_dataset. does not exist.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;

    /* Check for records in the dataset.  If the dataset is empty, then just end macro */
    /* because the empty XML input file will cause a REST error.                       */
    %local n_obs ;
    %let dsid = %sysfunc (open (&id_dataset.(%fcf_get_ds_opts(ds_name=&id_dataset)))) ;
    %if &dsid = 0 %then %do;
        %let _fcfmsg_=ERROR: Unable to open &id_dataset. for observation count.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
        %end;
    %else %do;
        %let n_obs = %sysfunc (attrn (&dsid, nlobs)) ;
        %let rc   = %sysfunc (close (&dsid)) ;
        %end;
    %if &n_obs = 0 %then %do;
         %put NOTE: Dataset &id_dataset. is empty and will not generate any deletes.;
         %goto MACRO_END;
         %end;
	
	/* If pass in a cert_file, make sure can find it. */
	%if %nrbquote(&cert_file.) ne %nrbquote() %then %do;
	    %let check_cert_file=%sysfunc(fileexist(&cert_file.));
		%if &check_cert_file = 0 %then %do;
           %let _fcfmsg_=ERROR: certficate file &cert_file. does not exist.;
           %fcf_put_error;
           %fcf_rcset(3);
           %goto MACRO_END;
		   %end;
    %end;

	/* Make sure can see the id variable that was provided in dataset. */
	%let dsid = %sysfunc(open(&id_dataset.(%fcf_get_ds_opts(ds_name=&id_dataset))));
    %let var_exist = %sysfunc(varnum(&dsid,&id_variable.));
    %let rc = %sysfunc(close(&dsid));
	%if &var_exist = 0 %then %do;
        %let _fcfmsg_=ERROR: Variable &id_variable. does not exist in &id_dataset..;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;

	/* Check the entity_type to make sure one of the supported types. */
	%let valid_entities=%str(alert,case,comment,customer,account,hhd,report,attachment,address,cddreview,risk);
    %let locate_entity=%index(&valid_entities.,&entity_type.);
	%if &locate_entity = 0 %then %do;
        %let _fcfmsg_=ERROR: Entity_type &entity_type. is not a supported type.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;

	/* Set a work location for scripts.  If blank, will use SAS WORK */
	%if %nrbquote(&work_location) = %nrbquote() %then %do;    
        /* Look in preferences first, and if not found, default to WORK directory. */	
	    %fcf_get_property_value(property=solrConfig.amlPurgeLogDirectory,
                    output_property_value=log_dir,
                    output_status=prop_status);
	    %if &prop_status ne 0 %then %do;
            %put WARNING: Purge log directory not found in preferences. Using default value of SAS WORK directory.;
            %let work_location=%trim(%sysfunc(getoption(work)));
            %end;
		%else %do;
		    %let work_location=%trim(&log_dir.);
		    %end;
        %put NOTE: Directory &work_location. will default for scripts and log files.;
        %end;
	%else %do;
	    %put NOTE: Directory &work_location. will be used for scripts and log files.;
	    %end;

	/* If curl_command is passed in, use that pathname and check for existence. */
	/* If not, default the command based on operating system type.              */
	%if %nrbquote(&curl_command) = %nrbquote() %then %do;
	    %if &sysscp eq WIN %then %do;
	        %let curl_exe=curl.exe;
	        %end;
	    %else %do;
	        %let curl_exe=curl;
	        %end;
		%end;
	%else %do;
		%let check_exe=%sysfunc(fileexist(&curl_command.));
	    %if &check_exe = 0 %then %do;
            %let _fcfmsg_=ERROR: Curl executable &curl_command. cannot be found.;
            %fcf_put_error;
            %fcf_rcset(3);
            %goto MACRO_END;
            %end;
	    %let curl_exe=%trim(&curl_command.);
	    %end;

	/* Lookup values from the AML Admin preferences for the SOLR */
	/* rest end point.                                           */
	%fcf_get_property_value(property=solrConfig.collection,
                        output_property_value=solr_collection,
                        output_status=prop_status);
    %if &prop_status ne 0 %then %do;
       %put WARNING: Name of Solr collection not found in preferences. Using default value CS-FCFBU1.;
       %let solr_collection=CS-FCFBU1;
       %end;
    %put NOTE: Using Solr collection: &solr_collection.;
	%fcf_get_property_value(property=solrConfig.protocol,
                        output_property_value=solr_protocol,
                        output_status=prop_status);
	%if &prop_status ne 0 %then %do;
       %put WARNING: Solr protocol not found in preferences. Using default value http.;
       %let solr_protocol=http;
       %end;
	%put NOTE: Using Solr protocol: &solr_protocol.;
	%fcf_get_property_value(property=solrConfig.serverPort,
                        output_property_value=solr_port,
                        output_status=prop_status);
	%if &prop_status ne 0 %then %do;
       %put WARNING: Solr Server Port not found in preferences. Using default value 8983.;
       %let solr_port=8983;
       %end;
    %put NOTE: Using Solr server port: &solr_port.;
	%fcf_get_property_value(property=solrConfig.serverUrl,
                        output_property_value=solr_url,
                        output_status=prop_status);
   %if &prop_status ne 0 %then %do;
      %put WARNING: Solr Server URL not found in preferences. Using default value localhost.;
      %let solr_url=localhost;
      %end;
   %put NOTE: Using Solr server URL: &solr_url.; 
   
   /* Check if cert_file was provided and url is not https, then exit. */
   /* Else, modify the curl command to pass the --cacert arg.          */
   %let cert_option=;
   %if %nrbquote(&cert_file.) ne %nrbquote() %then %do;
       %if %nrbquote(&solr_protocol.) = http %then %do;
	       %let _fcfmsg_=ERROR: Solr protocol &solr_protocol. is not supported when using cert_file option.;
           %fcf_put_error;
           %fcf_rcset(3);
           %goto MACRO_END;
	       %end;
	   %else %do;
	       %let cert_option=%str(--cacert &cert_file.);
	       %end;
       %end;
	
   %let solr_rest_url=&solr_protocol.://&solr_url.:&solr_port./solr/&solr_collection./update?commit=true; 

   %if &debug.=T %then %do;
      %put TRACE: SOLR REST url: &solr_rest_url.; 
	  %put TRACE: curl command: &curl_exe.; 
	  %end;
    
   /* Build an XML file of delete statements that will be input to */
   /* REST service update call.                                    */
   /* Use pid number (sysjobid), date and time for uniqueness.     */
   %let dt=%sysfunc(datetime(),b8601dt19.3);
   %let xml_tmp_file=&work_location.&slash.delete_ids_&dt._&sysjobid..xml;
   %let rest_output=&work_location.&slash.delete_ids_&dt._&sysjobid..log;
   %let win_bat=&work_location.&slash.delete_ids_&dt._&sysjobid..bat;
   filename xml_list "&xml_tmp_file.";

   /* This needs to be generated based on a passed list/dataset. */
   data _null_;
      file xml_list recfm=v lrecl=512;
	  set &id_dataset.(%fcf_get_ds_opts(ds_name=&id_dataset)) end=done;
	  format id_key $512.;
	  /* certain entity types will NOT use entity name, but use abbreviation. */
	  /* And for these character keys, make sure to add an extra trim.        */
      if trim(upcase("&entity_type.")) = 'CUSTOMER' then
          id_key=trim(cat("<id>","pty","-",trim(&id_variable.),"</id>"));
      else if trim(upcase("&entity_type.")) = 'ACCOUNT' then
          id_key=trim(cat("<id>","acc","-",trim(&id_variable.),"</id>"));
      else if trim(upcase("&entity_type.")) = 'HHD' then
          id_key=trim(cat("<id>","hhd","-",trim(&id_variable.),"</id>"));
      else if trim(upcase("&entity_type.")) = 'EXT' then
          id_key=trim(cat("<id>","ext","-",trim(&id_variable.),"</id>"));
      else
          id_key=trim(cat("<id>",symget("entity_type"),"-",&id_variable.,"</id>"));
	  if _n_=1 then 
	     put "<delete>";
	  put id_key;
	  if done then 
	     put "</delete>";
   run;
   
   /* Build a curl command to run the POST update to delete ids.         */
   /* Make sure can find the curl executable before trying to proceed.   */
   /* Windows will be curl.exe and Unix will be curl for executables.    */  
   data _null_;
       format full_command $2048.;
       ostype=trim("&sysscp.");
	   /* Windows needs double quoting and .exe for curl, and Unix does not. */
	   if ostype='WIN' then do;
	      full_command=cat('"',"&curl_exe.",'"', ' -X POST'
		                  ,' ',"&cert_option."
                          ,' ',"&solr_rest_url."
                          ,' -H "Content-Type: application/xml"'
                          ,' --data-binary '
						  ,'"@',"&xml_tmp_file.",'"'
						  ,' >','"',"&rest_output.",'"',' 2>&1'
                          );
          end;
	   else do;
          full_command=cat("&curl_exe",' -X POST'
		                   ,' ',"&cert_option."
         				   ,' ',"&solr_rest_url."
                           ,' -H "Content-Type: application/xml"'
						   ,' --data-binary @'
						   ,"&xml_tmp_file."
                           ,' >',"&rest_output.",' 2>&1'
                           );
	       end;
	   full_command=trim(full_command);
       call symput('pipecmd',trim(full_command));
	   %if &debug=T %then %do;
           put "TRACE: Will be executing" full_command;
		   %end;
       run;

   /* One more check, if its Windows, need to change the pipe to be a */
   /* bat file to execute.                                            */
   %if &sysscp eq WIN %then %do;
       /* First, create the bat file with current pipe command. */
       data _null_;
	      format curl_line $2048.;	  
	      file "&win_bat.";	  
		  curl_line=trim(symget("PIPECMD"));
		  put curl_line;
	      run;
       /* Second, change the pipe command to be execute bat file. */
       data _null_;
	      format bat_cmd $2048.;
		  bat_cmd=trim(cat('"',trim(symget("WIN_BAT")),'"'));
		  put 'NOTE: New command ' bat_cmd;
		  call symput('pipecmd',trim(bat_cmd));
	   run;
       %end;

   %let filerf=runcmd;
   %let rc=%sysfunc(filename(filerf, %str(&pipecmd.), pipe));
   %if &rc ne 0 %then %do;
       %let _fcfmsg_=ERROR: Unable to execute system pipe command to execute script.;
       %fcf_put_error;
	   %fcf_rcset(3);
	   %GOTO MACRO_END;
       %end;		
   data _null_;
       infile runcmd lrecl=1024 recfm=v missover;
       input @1 line $1024.;
	   %if &debug=T %then %do;
           put 'TRACE: Command output: ' line=;
		   %end;
   run;
             
   %fcf_rcset(&syserr);

   /* Test for an output file to make sure command ran. */
   %if %sysfunc(fileexist("&rest_output.")) eq 0 %then %do;
       %let _fcfmsg_=Error: Log file &rest_output was not created.;
	   %fcf_put_error;     
       %fcf_rcset(3);
       %goto MACRO_END;
	   %end;

   /* Look for a successful return code in log file xml. */
   %let successYN=N;
   filename rest_out "&rest_output.";
   data _null_;
       format successFlag 8. data_line $2048.;
       retain successFlag 0;
       infile rest_out lrecl=2048 pad missover recfm=v end=done;
       input data_line $2048.;
	   %if &debug=T %then %do;
	       put 'TRACE: Execution Log: ' data_line;
		   %end;
       if index(data_line,'<int name="status">0') > 0 then do;
	       successFlag=1;
		   end;
	   if done then do;
	       if successFlag=1 then do;
		      call symput('successYN','Y');
		      end;
	       end;
	   run;
   %if &successYN=N %then %do;
       %let _fcfmsg_=Error: Delete script did not run successful. Check log file &rest_output.;
       %fcf_put_error;
       %fcf_rcset(3);
       %goto MACRO_END;
	   %end; 
   %else %do;
       %put Delete script was successful. Check log file &rest_output. for results;
       %end; 
                        
   %fcf_rcset(&syserr);                             


%MACRO_END:
      

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
        %fcf_execute_macro(fcf_error);
    %else 
        %fcf_execute_macro(fcf_success);

	%if %sysfunc(fileref(XML_LIST))=0 %then %do;
	   filename xml_list clear;
	   %end;
	%if %sysfunc(fileref(REST_OUT))=0 %then %do;
	   filename rest_out clear;
	   %end;
              
%mend;
