/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_wl_load.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Load watchlist data into Core DBMS
+------------------------------------------------------------------------------+
| USAGE:  %fcf_wl_load(watchlist=, stage_ds_lib=, stage_ds_name=, core_table=);
+------------------------------------------------------------------------------+
| PARAMETERS:
|     watchlist
|         The name of the watchlist that is being loaded. 
|         example values: hmt, dwjn, ofac
|
|     stage_ds_lib
|         The name of library containing the watch list data set to be loaded.
|
|     stage_ds_name
|         The name of the watch list data set to be loaded.
|
|     core_table = the name of the table to be updated
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_load(watchlist=,
                   stage_ds_lib=STG_WTCH,
                   stage_ds_name=,
                   core_table=FSC_ENTITY_WATCH_LIST_DIM);

  /*---------------
   * USER SETTINGS
   *---------------*/

  /* Set DBMS locking option */
  %let LOCK_OPTIONS=;
  %if %upcase(&dbflavor) eq DB2 %then %do;
    %let LOCK_OPTIONS=%str(UPDATE_ISOLATION_LEVEL=RR);
  %end;
  %else %do;
    %let LOCK_OPTIONS=%str(UPDATE_LOCK_TYPE=TABLE REREAD_EXPOSURE=YES);
  %end;

  /*----------------------
   * END of USER SETTINGS
   *----------------------*/

  /*------------
   * Set Module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=fcf_WL_LOAD;

  /*---------------------
   * Call pre stub macro
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*-----------------------------
   * Set start time for jobstats
   *-----------------------------*/
  %fcf_section_header(section=&_fcfmodule_, message_format=verbose);

  %let watchlist=%upcase(%cmpres(&watchlist));
  %let core_table=%upcase(%cmpres(&core_table));
  %let nkey=ENTITY_WATCH_LIST_NUMBER;
  %let skey=ENTITY_WATCH_LIST_KEY;
  %let stage_ds_full_name = &stage_ds_lib..&stage_ds_name;


  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc=0;
  %let _fcfmsg_=;

  %let _fcfmsg_=NOTE: FCF: Loading data for watchlist: %upcase(&watchlist).;
  %put &_fcfmsg_;

  /* Make sure input dataset name is not too long */
  %if %length(&stage_ds_name) gt 28 %then %do;
    %let _fcfmsg_=ERROR: FCF: Input data set name is too long: &stage_ds_name;
    %put &_fcfmsg_;
    %let trans_rc=999999;
    %goto MACRO_END;
  %end;

  /* Make sure runasofdate is set */
  %if not %symexist(runasofdate) %then %do;
    %fcf_get_runasofdate;
    %if not %symexist(runasofdate) %then %do;
      %let _fcfmsg_=ERROR: FCF: Runasofdate could not be set.  &_fcfmodule_ exiting.;
      %put &_fcfmsg_;
      %let trans_rc=999999;
      %goto MACRO_END;
    %end;
  %end;
  %if &runasofdate eq %str() %then %do;
    %let _fcfmsg_=ERROR: FCF: Runasofdate is not set correctly.  &_fcfmodule_ exiting.;
    %put &_fcfmsg_;
    %let trans_rc=999999;
    %goto MACRO_END;
  %end;
  %let _fcfmsg_=NOTE: FCF: Runasofdate: &runasofdate;
  %put &_fcfmsg_;


  /* Cleanup from previous run */
  %let _fcfmsg_=NOTE: FCF: Clean-up from previous load processes.;
  %put &_fcfmsg_;
  proc datasets library=STG_WTCH nolist nowarn;
    delete wl_&watchlist._NEW;
    delete wl_&watchlist._TYPE2CLOSE;
    delete wl_&watchlist._EXISTS;
  quit;

  proc datasets library=&stage_ds_lib nolist nowarn;
    delete &stage_ds_name._tmp;
  quit;


  /******************************************/
  /* Initialize data column designation     */
  /******************************************/
  %fcf_wl_column_types;

  /*
   NOTE:

   The watch list data can have multiple entries with the same
   natural key value.  This is because of aliases, alternate spellings,
   multiple entries for fields such as country_of_birth, address, etc.

   The columns in the "key columns" macro variable are the set of unique values
   for a watch list entry.  If -any- of these columns are different, a new
   entry is created in the database.

   However, if all of the values in the "key columns" are the same, then changes
   to a value of any of the columns in the "data columns" cause an SCD-type 2
   update to the information in the database.
   
  */


  /********************************************/
  /* Remove any duplicates from incoming data */
  /********************************************/

  /* Check for existence of transformed watchlist table */
  %fcf_get_observation_count(table=&stage_ds_full_name);
  %if &fcf_obs_count eq -1 %then %do;
    %let _fcfmsg_=ERROR: FCF: %upcase(&stage_ds_full_name) does not exist or is not accessible.;
    %let trans_rc=999999;
     %fcf_put_error;
    %goto MACRO_END;
  %end;
  %if &fcf_obs_count eq 0 %then %do;
    %let _fcfmsg_=ERROR: FCF: %upcase(&stage_ds_full_name) has 0 observations.;
    %let trans_rc=999999;
     %fcf_put_error;
    %goto MACRO_END;
  %end;
  %let fcf_extracted_records=&fcf_obs_count;
  
  /* Compute digest values on incoming data for merge */
  data &stage_ds_full_name(%fcf_get_ds_opts(ds_name=&stage_ds_full_name, output_ds=Y));
    set &stage_ds_full_name(%fcf_get_ds_opts(ds_name=&stage_ds_full_name));
    datacol_stage_digest=put(md5(cats(of &fcf_wl_data_columns)),$hex32.);
    keycol_digest=put(md5(cats(of &fcf_wl_key_columns)),$hex32.);
  run;

  /* Sort incoming data removing duplicates */
  proc sort data=&stage_ds_full_name(%fcf_get_ds_opts(ds_name=&stage_ds_full_name))
             out=&stage_ds_full_name._tmp(%fcf_get_ds_opts(ds_name=&stage_ds_full_name._tmp, output_ds=Y)) nodup
          dupout=&stage_ds_full_name._DUP(%fcf_get_ds_opts(ds_name=&stage_ds_full_name._DUP, output_ds=Y));
    by keycol_digest;
  run;
  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: An error occurred while sorting the staged data: &stage_ds_full_name.;
     %fcf_put_error;
    %goto MACRO_END;
  %end;
  %fcf_get_observation_count(table=&stage_ds_full_name._DUP);
  %let fcf_extracted_dup_records=&fcf_obs_count;


  /******************************************/
  /* Initialize Xref Tables                 */
  /******************************************/
  data work.wl_&watchlist._lookup_tmp(keep=&nkey
                                           &skey
                                           datacol_change_digest
                                           keycol_digest
                                      compress=yes);
    attrib datacol_change_digest keycol_digest length=$32;

    set &db_core_libname..&core_table(where=(change_current_ind='Y' and watch_list_name="&watchlist"));

    datacol_change_digest=put(md5(cats(of &fcf_wl_data_columns)),$hex32.) ;
    keycol_digest=put(md5(cats(of &fcf_wl_key_columns)),$hex32.) ;
  run;
  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: An error occurred while creating the lookup tables.;
     %fcf_put_error;
    %goto MACRO_END;
  %end;

  proc sort data=work.wl_&watchlist._lookup_tmp
             out=stg_xref.wl_&watchlist._LOOKUP(%fcf_get_ds_opts(lib_name=stg_xref, ds_name=wl_&watchlist._LOOKUP, output_ds=Y)) nodup;
    by keycol_digest;
  run;
  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: An error occurred while sorting the lookup tables.;
     %fcf_put_error;
    %goto MACRO_END;
  %end;

  proc datasets library=work nolist nowarn;
    delete wl_&watchlist._lookup_tmp;
  quit;
  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: An error occurred while deleting the temporary lookup tables.;
     %fcf_put_error;
    %goto MACRO_END;
  %end;

  /******************************************/
  /* Get max surrogate key                  */
  /******************************************/
  %fcf_get_max_surrogate_key(lib=seg_kc, dimension=&core_table, surrogatekey=&skey.);
  %let _fcfmsg_=NOTE: FCF: Max Surrogate Key for watch list dimension: &maxsurrogatekey;
  %put &_fcfmsg_;


  /***************************************************************************/
  /*                        BEGIN WATCHLIST PROCESSING                       */
  /***************************************************************************/

  %let _fcfmsg_=NOTE: FCF: Begin watch list data update process.;
  %put &_fcfmsg_;

  data STG_WTCH.wl_&watchlist._NEW         (%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_&watchlist._NEW, output_ds=Y)
                                            drop=datacol_stage_digest keycol_digest datacol_change_digest index=(&skey ))
       STG_WTCH.wl_&watchlist._type2Close  (%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_&watchlist._type2Close, output_ds=Y)
                                            drop=datacol_stage_digest keycol_digest datacol_change_digest index=(&skey))
       STG_WTCH.wl_&watchlist._type2New    (%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_&watchlist._type2New, output_ds=Y)
                                            drop=datacol_stage_digest keycol_digest datacol_change_digest index=(&skey))
       STG_WTCH.wl_&watchlist._EXISTS      (%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_&watchlist._EXISTS, output_ds=Y));

    drop   new_records changed_records existing_records deleted_records;
    retain new_records changed_records existing_records deleted_records 0;

    format &skey best32.;
    format change_end_date   datetime20.;
    format change_begin_date datetime20.;

    drop new_surrogate_key;
    
    /* Max of the surrogate key from core watchlist table */
    retain new_surrogate_key &maxSurrogateKey;

    /* Merge sorted staged table with XREF table */
    merge &stage_ds_full_name._tmp (%fcf_get_ds_opts(ds_name=&stage_ds_full_name._tmp) in=inSTAGE)
          stg_xref.wl_&watchlist._LOOKUP (%fcf_get_ds_opts(lib_name=stg_xref, ds_name=wl_&watchlist._LOOKUP) in=inXREF) end=LASTREC;
    by keycol_digest;

    /* Existing records */
    if inSTAGE and inXREF then do;

      if kstrip(datacol_stage_digest) ne kstrip(datacol_change_digest) then do;
        /* Data column changed
           need new record for audit purposes (SCD2-type change)
        */

       /* Output record to be "CLOSED" */
        change_current_ind='N';
        change_end_date=dhms(input(put(&runasofdate,8.),yymmdd8.),00,00,00) - 1;
        output STG_WTCH.wl_&watchlist._type2Close;

        /* Output record with new values */
        new_surrogate_key + 1;
        &skey.=new_surrogate_key;
        
        change_begin_date=dhms(input(put(&runasofdate,8.),yymmdd8.),00,00,00);
        change_end_date='01JAN5999:00:00:00'dt;
        change_current_ind='Y';

        changed_records + 1;
        output STG_WTCH.wl_&watchlist._type2New;
      end;
      else do;
        /* Record exists unchanged in DBMS */
        output STG_WTCH.wl_&watchlist._EXISTS;
        existing_records + 1;
      end;
    end;

    else if inSTAGE and not inXREF then do;

      /* New records */

      new_surrogate_key + 1;
      &skey.=new_surrogate_key;
      change_begin_date=dhms(input(put(&runasofdate,8.),yymmdd8.),00,00,00);
      change_end_date='01JAN5999:00:00:00'dt;
      change_current_ind='Y';
      output STG_WTCH.wl_&watchlist._NEW;

      new_records + 1;
    end;

    if LASTREC then do;
      put 40* "-";
      put "NOTE: FCF: Staging record count for watchlist: &watchlist";
      put 40* "-";
      put "NOTE: FCF: Extracted watchlist records: %cmpres(&fcf_extracted_records)";
      put "NOTE: FCF: Duplicate watchlist records: %cmpres(&fcf_extracted_dup_records)";
      put "NOTE: FCF: Watchlist records processed: %cmpres(%eval(&fcf_extracted_records - &fcf_extracted_dup_records))";
      put 40* "-";
      put "NOTE: FCF: New records to be added:     " new_records;
      put "NOTE: FCF: Records with SCD1 changes:   " changed_records;
      put "NOTE: FCF: Records that already exist:  " existing_records;
      put 40* "-";
    end;
  run;
  %fcf_rcset(&syserr);
  %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
    %let _fcfmsg_=ERROR: FCF: An error occurred while merging the staged &watchlist data.;
    %fcf_put_error;
    %let _fcfmsg_=ERROR: FCF: The FCF database will not be updated with the information in &stage_ds_full_name..;
    %put &_fcfmsg_;
    %goto MACRO_END;
  %end;

  /*--------------------------
   * END WATCHLIST PROCESSING
   *--------------------------*/


  /*----------------------
   * BEGIN WATCHLIST LOAD
   *----------------------*/

  /* Get bulkload options */
  %let fcf_bulk_load_options=%fcf_bulk_load(module=&_fcfmodule_._&watchlist);

  /* type2Close - Close out existing records with data column changes */
  %fcf_get_observation_count(table=STG_WTCH.wl_&watchlist._type2Close);
  %if &fcf_obs_count gt 0 %then %do;

    data &db_core_libname..&core_table;
      modify &db_core_libname..&core_table(dbkey=&skey &LOCK_OPTIONS) 
             STG_WTCH.WL_&watchlist._type2Close(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=WL_&watchlist._type2Close)
                                                rename=(change_end_date=update_end_date));
      by &skey;
      if _iorc_=0 then do;
        change_end_date = update_end_date;
        change_current_ind = 'N';
        replace;
      end;

      else do;
        _error_=0;
        put 'WARNING: FCF: Could not update database for watchlist &watchlist: ' &skey=;
      end;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Cannot close out records with data column changes in &db_core_libname..&core_table..;
      %fcf_put_error;
      %goto MACRO_END;
    %end;

  %end;
  /* End of SCD2 Close out -- STG_WTCH.wl_&watchlist._type2Close has obs */


  /* NEW - SCD2 - type2New dimension records */
  %fcf_get_observation_count(table=STG_WTCH.wl_&watchlist._type2New);
  %if &fcf_obs_count gt 0 %then %do;
  
    /* CORE: Add new records to &db_core_libname */
    proc append base=&db_core_libname..&core_table(&fcf_bulk_load_options)
                data=STG_WTCH.wl_&watchlist._type2New(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=wl_&watchlist._type2New))
                force;
    run;
  %end;

  /* NEW - watchlist records */
  %fcf_get_observation_count(table=STG_WTCH.WL_&watchlist._NEW);
  %if &fcf_obs_count gt 0 %then %do;
     /* CORE: Add new records to seg_kc */
     proc append base=db_core.&core_table(&fcf_bulk_load_options)
                 data=STG_WTCH.WL_&watchlist._NEW(%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=WL_&watchlist._NEW))
       force;
      run;
  %end;


  /*----------------------------------------------------------------
   * Reporting on records that already exist in database and
   * were included in input file.
   * No changes were detected in staged record and database record.
   *----------------------------------------------------------------*/
  %fcf_get_observation_count(table=STG_WTCH.WL_&watchlist._EXISTS);
  %if &fcf_obs_count gt 0 %then %do;
      %put NOTE: FCF: Staged &watchlist watchlist records already exist in the database.;

      data _null_;
        call symput('key_columns_nocomma',tranwrd("&fcf_wl_key_columns",',',' '));
      run;

      title 'Existing Watch List Records';
      title2 "Data set: STG_WTCH.WL_&watchlist._EXISTS";
      proc print data=STG_WTCH.WL_&watchlist._EXISTS (%fcf_get_ds_opts(lib_name=STG_WTCH, ds_name=WL_&watchlist._EXISTS)
                                                      keep=&nkey &key_columns_nocomma) width=MIN noobs;
      run;
      title;
      title2;
  %end;


  /*--------------------
   * END WATCHLIST LOAD
   *--------------------*/

  %MACRO_END:

   /* Set end time for jobstats */
   %fcf_section_footer(section=&_fcfmodule_, message_format=verbose);

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      /* Call post stub macro */
      %fcf_execute_macro(postwl_&watchlist._load);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_wl_load;
