/*==============================================================================
| NAME:    fcf_wl_nbci_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - National Bureau of Criminal Intelligence, New Zealand Police
WATCH LIST NAME     - Designated Terrorist Entities
WATCH LIST CODE     - NBCI
WEB LINK            - http://www.police.govt.nz/service/counterterrorism/designated-terrorists.html

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_nbci_transform;

  /*------------
   * Set module
   *------------*/
  %let _fcfmodule_=fcf_WL_NBCI_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc=0;
  %let _fcfmsg_=;

   /*------------------------------------------
    * get column names for watch_list_dim
    *------------------------------------------*/
    %fcf_get_column_names(table=seg_kc.fsc_entity_watch_list_dim);

/******************************************************************************/
/* BEGIN: INDIVIDUALS                                                         */
/******************************************************************************/


  proc contents data=stg_wtch.wl_nbci_ind_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_extract))
                 out=cols (keep=name varnum) noprint;
  run;

  proc sql noprint;
	  select strip(name) into :col1_name
      from cols 
	  where varnum=1;
  quit;

  %let rename_stmt= rename=(&col1_name=nbci_id) ;
	

/*******************************************************************************
 STEP DESCRIPTION:

   This step processes and outputs a distinct record for each column involved
   in a series (example DOB_1 - DOB_9)

 INPUTS:

   stg_wtch.wl_nbci_ind_extract

 OUTPUTS:

   stg_wtch.wl_nbci_ind_transform_01 - (place of birth)
   stg_wtch.wl_nbci_ind_transform_02 - (date of birth)
   stg_wtch.wl_nbci_ind_transform_03 - (citizenship/nationality)
   stg_wtch.wl_nbci_ind_transform_04 - (passport)
   stg_wtch.wl_nbci_ind_transform_05 - (address)
   stg_wtch.wl_nbci_ind_transform_06 - (tax_id)

 NOTES:

   The following column series are processed by this step:

   - POB_1-POB_2
   - POB_1_COUNTRY-POB_2_COUNTRY
   - DOB_1-DOB_9
   - NATIONALITY_1-NATIONALITY_3
   - PASSPORT_1-PASSPORT_4
   - ADDRESS-ADDRESS_5

   Additional customizations may be required when processing NID column.

*******************************************************************************/

data

   stg_wtch.wl_nbci_ind_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_01, output_ds=Y)
                                     keep=
      entity_watch_list_number
      place_of_birth
   )

   stg_wtch.wl_nbci_ind_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_02, output_ds=Y)
                                     keep=
      entity_watch_list_number
      nbci_id
      dob
   )

   stg_wtch.wl_nbci_ind_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_03, output_ds=Y)
                                     keep=
      entity_watch_list_number
      citizenship_country_name
      nationality_country_name
   )

   stg_wtch.wl_nbci_ind_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_04, output_ds=Y)
                                     keep=
      entity_watch_list_number
      identification_id
      identification_type_desc
   )

   stg_wtch.wl_nbci_ind_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_05, output_ds=Y)
                                     keep=
      entity_watch_list_number
      full_address
   )

   stg_wtch.wl_nbci_ind_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_06, output_ds=Y)
                                     keep=
      entity_watch_list_number
      tax_id
   );

   attrib dob length=$50  informat=$50.  format=$50.;

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_nbci_ind_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_extract)
                                    keep=&col1_name
                                         pob_1-pob_2
                                         pob_1_country
                                         pob_2_country
                                         dob_1-dob_9
                                         nationality_1-nationality_3
                                         passport_1-passport_4
                                         address
                                         address_2
                                         address_3
                                         address_4
                                         address_5
                                         nid

										 &rename_stmt
                                   );

   if kindex(nbci_id,'.') then 
      entity_watch_list_number=kstrip(ksubstr(nbci_id,1,kindex(nbci_id,'.')-1));
   else entity_watch_list_number=kstrip(nbci_id);

   /*********************************************/
   /* POB_1-POB_2 - POB_1_COUNTRY-POB_2_COUNTRY */
   /*********************************************/

   /* POB_1 - POB_1_COUNTRY */
   if not missing(pob_1) then do;
      if kindex(pob_1_country, '(' ) then
         pob_1_country=kstrip(kscan(pob_1_country,1,'('));

      pob_1=kstrip(compbl(upcase(pob_1)));
      pob_1_country=kstrip(compbl(upcase(pob_1_country)));

      if indexw(pob_1,pob_1_country) then
          place_of_birth=pob_1;
      else
         place_of_birth=kstrip(pob_1) || ", " || kstrip(pob_1_country);

      output stg_wtch.wl_nbci_ind_transform_01;
   end;
   /* POB_2 - POB_2_COUNTRY */
   if not missing(pob_2) then do;
      if kindex(pob_2_country, '(' ) then
         pob_2_country=kstrip(kscan(pob_2_country,1,'('));

      pob_2=kstrip(compbl(upcase(pob_2)));
      pob_2_country=kstrip(compbl(upcase(pob_2_country)));

      if indexw(pob_2,pob_2_country) then
          place_of_birth=pob_2;
      else
         place_of_birth=kstrip(pob_2) || ", " || kstrip(pob_2_country);

      output stg_wtch.wl_nbci_ind_transform_01;
   end;

   /*****************/
   /* DOB_1 - DOB_9 */
   /*****************/

   /* DOB_1 */
   if not missing(dob_1) then do;
      dob=kstrip(dob_1);
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_2 */
   if not missing(dob_2) then do;
      dob=kstrip(dob_2);
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_3 */
   if not missing(dob_3) then do;
      dob=kstrip(dob_3);
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_4 */
   if not missing(dob_4) then do;
      dob=kstrip(dob_4);
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_5 */
   if not missing(dob_5) then do;
   	  /* Convert to character value before strip function */
   	  dob = kstrip(put(dob_5, best8.));
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_6 */
   if not missing(dob_6) then do;
   	  /* Convert to character value before strip function */
   	  dob = kstrip(put(dob_6, best8.));
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_7 */
   if not missing(dob_7) then do;
   	  /* Convert to character value before strip function */
   	  dob = kstrip(put(dob_7, best8.));
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_8 */
   if not missing(dob_8) then do;
   	  /* Convert to character value before strip function */
   	  dob = kstrip(put(dob_8, best8.));
      output stg_wtch.wl_nbci_ind_transform_02;
   end;
   /* DOB_9 */
   if not missing(dob_9) then do;
   	  /* Convert to character value before strip function */
   	  dob = kstrip(put(dob_9, best8.));
      output stg_wtch.wl_nbci_ind_transform_02;
   end;

   /*********************************/
   /* NATIONALITY_1 - NATIONALITY_3 */
   /*********************************/

   /* NATIONALITY_1 */
   if not missing(nationality_1) then do;
      citizenship_country_name=upcase(compbl(kstrip(nationality_1)));
      nationality_country_name=upcase(compbl(kstrip(nationality_1)));
      output stg_wtch.wl_nbci_ind_transform_03;
   end;
   /* NATIONALITY_2 */
   if not missing(nationality_2) then do;
      citizenship_country_name=upcase(compbl(kstrip(nationality_2)));
      nationality_country_name=upcase(compbl(kstrip(nationality_2)));
      output stg_wtch.wl_nbci_ind_transform_03;
   end;
   /* NATIONALITY_3 */
   if not missing(nationality_3) then do;
      citizenship_country_name=upcase(compbl(kstrip(nationality_3)));
      nationality_country_name=upcase(compbl(kstrip(nationality_3)));
      output stg_wtch.wl_nbci_ind_transform_03;
   end;

   /***************************/
   /* PASSPORT_1 - PASSPORT_4 */
   /***************************/

   /* PASSPORT_1 */
   if not missing(passport_1) then do;
      if kindex(passport_1,';') then do;
         num_passports=countc(passport_1,';') + 1;
         do i=1 to num_passports;
            identification_id=kstrip(kscan(passport_1,i,';'));
            identification_type_desc='PASSPORT';
            output stg_wtch.wl_nbci_ind_transform_04;
         end;
      end;
      else do;
         identification_id=kstrip(passport_1);
         identification_type_desc='PASSPORT';
         output stg_wtch.wl_nbci_ind_transform_04;
      end;
   end;
   /* PASSPORT_2 */
   if not missing(passport_2) then do;
      if kindex(passport_2,';') then do;
         num_passports=countc(passport_2,';') + 1;
         do i=1 to num_passports;
            identification_id=kstrip(kscan(passport_2,i,';'));
            identification_type_desc='PASSPORT';
            output stg_wtch.wl_nbci_ind_transform_04;
         end;
      end;
      else do;
         identification_id=kstrip(passport_2);
         identification_type_desc='PASSPORT';
         output stg_wtch.wl_nbci_ind_transform_04;
      end;
   end;
   /* PASSPORT_3 */
   if not missing(passport_3) then do;
      if kindex(passport_3,';') then do;
         num_passports=countc(passport_3,';') + 1;
         do i=1 to num_passports;
            identification_id=kstrip(kscan(passport_3,i,';'));
            identification_type_desc='PASSPORT';
            output stg_wtch.wl_nbci_ind_transform_04;
         end;
      end;
      else do;
         identification_id=kstrip(passport_3);
         identification_type_desc='PASSPORT';
         output stg_wtch.wl_nbci_ind_transform_04;
      end;
   end;
   /* PASSPORT_4 */
   if not missing(passport_4) then do;
      if kindex(passport_4,';') then do;
         num_passports=countc(passport_4,';') + 1;
         do i=1 to num_passports;
            identification_id=kstrip(kscan(passport_4,i,';'));
            identification_type_desc='PASSPORT';
            output stg_wtch.wl_nbci_ind_transform_04;
         end;
      end;
      else do;
         identification_id=kstrip(passport_4);
         identification_type_desc='PASSPORT';
         output stg_wtch.wl_nbci_ind_transform_04;
      end;
   end;

   /***********************/
   /* ADDRESS - ADDRESS_5 */
   /***********************/

   /* ADDRESS */
   if not missing(address) then do;
      address=tranwrd(address,":","");
      address=tranwrd(address,"a)","");
	  address=tranwrd(address,"b)","|");
	  address=tranwrd(address,"c)","|");
	  address=tranwrd(address,"d)","|");
	  address=tranwrd(address,"e)","|");

	  if kindex(address,'|') then do;
         num_address=countc(address,'|') + 1;
         do i=1 to num_address;
            full_address=compbl(kstrip(kscan(address,i,'|')));
            output stg_wtch.wl_nbci_ind_transform_05;
		 end;
	  end;
	  else do;
	     full_address=compbl(kstrip(address));
         output stg_wtch.wl_nbci_ind_transform_05;
	  end;
   end;
   /* ADDRESS_2 */
   if not missing(address_2) then do;
      full_address=compbl(kstrip(address_2));
      output stg_wtch.wl_nbci_ind_transform_05;
   end;
   /* ADDRESS_3 */
   if not missing(address_3) then do;
      full_address=compbl(kstrip(address_3));
      output stg_wtch.wl_nbci_ind_transform_05;
   end;
   /* ADDRESS_4 */
   if not missing(address_4) then do;
      full_address=compbl(kstrip(address_4));
      output stg_wtch.wl_nbci_ind_transform_05;
   end;
   /* ADDRESS_5 */
   if not missing(address_5) then do;
      full_address=compbl(kstrip(address_5));
      output stg_wtch.wl_nbci_ind_transform_05;
   end;

   /* NID */
   if not missing(nid) then do;
	  nid=tranwrd(nid,' a)','');
	  nid=tranwrd(nid,' b)',';');
	  nid=tranwrd(nid,' c)',';');
	  nid=tranwrd(nid,'(','');
	  nid=tranwrd(nid,')','');
	  nid=upcase(compbl(kstrip(nid)));
	  nid=tranwrd(nid,"SSN:","SSN");
      orig_nid=nid;
      nid=tranwrd(nid,'BRITISH','');
      nid=tranwrd(nid,'UNITED','');
      nid=tranwrd(nid,'KINGDOM','');
	  nid=tranwrd(nid,'STATES','');
	  nid=tranwrd(nid,'SOCIAL','');
	  nid=tranwrd(nid,'SECURITY','');
      nid=tranwrd(nid,'CHINESE','');
      nid=tranwrd(nid,'GERMAN','');
	  nid=tranwrd(nid,'ITALIAN','');
	  nid=tranwrd(nid,'FISCAL','');
      nid=tranwrd(nid,'PAKISTANI','');
	  nid=tranwrd(nid,'PAKISTAN','');
      nid=tranwrd(nid,'YEMENI','');
      nid=tranwrd(nid,'SAUDI','');
      nid=tranwrd(nid,'ARABIAN','');
	  nid=tranwrd(nid,'QATARI','');
	  nid=tranwrd(nid,'QATAR','');
	  nid=tranwrd(nid,'TUNISIAN','');
      nid=tranwrd(nid,'GEORGIAN','');
	  nid=tranwrd(nid,'TAZKIRA','');
	  nid=tranwrd(nid,'AGHAN','');
	  nid=tranwrd(nid,'LIBYAN','');
	  nid=tranwrd(nid,'MALASIAN','');
      nid=tranwrd(nid,'MALIAN','');
	  nid=tranwrd(nid,'INDONESIAN','');
	  nid=tranwrd(nid,'RUSSIAN','');
      nid=tranwrd(nid,'NATIONAL','');
      nid=tranwrd(nid,'INSURANCE','');
      nid=tranwrd(nid,'NUMBERS','');
      nid=tranwrd(nid,'NUMBER:','');
	  nid=tranwrd(nid,'NUMBER','');
	  nid=tranwrd(nid,'CODE:','');
      nid=tranwrd(nid,'SSN:','');
      nid=tranwrd(nid,'IDENTIFICATION','');
      nid=tranwrd(nid,'IDENTITY','');
      nid=tranwrd(nid,'CARD','');
      nid=tranwrd(nid,'FEDERAL','');
      nid=tranwrd(nid,'NID','');
	  nid=tranwrd(nid,'ID','');
	  nid=tranwrd(nid,'ENTITITY','');
	  nid=tranwrd(nid,'KUWAITI','');
	  nid=tranwrd(nid,'KUWAIT','');
	  nid=tranwrd(nid,'CIVIL','');
	  nid=tranwrd(nid,'AFGHAN','');
	  nid=tranwrd(nid,'RES ENTIAL','');
	  nid=tranwrd(nid,'ALTERNATE','');
	  nid=tranwrd(nid,'PASSPORT','');
	  nid=tranwrd(nid,'BIRTH','');
	  nid=tranwrd(nid,'CERTIFICATE','');
	
      if kindex(nid,';') then do;
         num_nid=countc(nid,';') + 1;
         do i=1 to num_nid;
            tax_id=kstrip(kscan(nid,i,';'));
            if not missing(tax_id)          and
			   anydigit(tax_id) and
               not indexw(tax_id,'EXPIRES') and
			   not indexw(tax_id,'EXPIRED') and
               not indexw(tax_id,'UNTIL') then do;
			      if indexw(tax_id,"OFFICIALLY") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"OFFICIALLY")-1);
				  if indexw(tax_id,"IN") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"IN")-1);
				  if kindex(tax_id,"ISSUE") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"ISSUE")-1);
                  output stg_wtch.wl_nbci_ind_transform_06;
				  orig_nid_part=kstrip(kscan(orig_nid,i,';'));
				  if indexw(orig_nid_part,tax_id) eq 1 then do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,length(identification_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
				  else do;
				     identification_id=tax_id;
					 if kindex(orig_nid_part,tax_id) then identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,1,kindex(orig_nid_part,tax_id)-1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
			   end;
         end;
      end;
      else if kindex(nid,',') then do;
         num_nid=countc(nid,',') + 1;
         do i=1 to num_nid;
            tax_id=kstrip(kscan(nid,i,','));
            if not missing(tax_id)          and
			   anydigit(tax_id) and
               not indexw(tax_id,'EXPIRES') and
			   not indexw(tax_id,'EXPIRED') and
               not indexw(tax_id,'ISSUED')  and
               not indexw(tax_id,'UNTIL')  then do;
			      if indexw(tax_id,"OFFICIALLY") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"OFFICIALLY")-1);
				  if indexw(tax_id,"IN") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"IN")-1);
				  if kindex(tax_id,"ISSUE") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"ISSUE")-1);
                  output stg_wtch.wl_nbci_ind_transform_06;
				  orig_nid_part=kstrip(kscan(orig_nid,i,','));
				  if indexw(orig_nid_part,tax_id) eq 1 then do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid,length(identification_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
				  else do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,1,kindex(orig_nid_part,tax_id)-1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
			   end;
         end;
      end;
	  else if kindex(nid,':') then do;
         num_nid=countc(nid,':') + 1;
         do i=1 to num_nid;
            tax_id=kstrip(kscan(nid,i,':'));
            if not missing(tax_id)          and
			   anydigit(tax_id) and
               not indexw(tax_id,'EXPIRES') and
			   not indexw(tax_id,'EXPIRED') and
               not indexw(tax_id,'ISSUED ON')  and
               not indexw(tax_id,'UNTIL')  then do;
			      if indexw(tax_id,"OFFICIALLY") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"OFFICIALLY")-1);
				  if indexw(tax_id,"IN") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"IN")-1);
				  if kindex(tax_id,"ISSUE") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"ISSUE")-1);
                  output stg_wtch.wl_nbci_ind_transform_06;
				  orig_nid_part=kstrip(kscan(orig_nid,i,':'));
				  if indexw(orig_nid_part,tax_id) eq 1 then do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,length(identification_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
				  else do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,1,length(orig_nid_part)-length(tax_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
			   end;
         end;
      end;
	  else if kindex(nid,'OR') then do;
         num_nid=countc(nid,'OR') + 1;
         do i=1 to num_nid;
            tax_id=kstrip(kscan(nid,i,'OR'));
            if not missing(tax_id)          and
			   anydigit(tax_id) and
               not indexw(tax_id,'EXPIRES') and
			   not indexw(tax_id,'EXPIRED') and
               not indexw(tax_id,'ISSUED')  and
               not indexw(tax_id,'UNTIL')  then do;
			      if indexw(tax_id,"OFFICIALLY") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"OFFICIALLY")-1);
				  if indexw(tax_id,"IN") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"IN")-1);
				  if kindex(tax_id,"ISSUE") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"ISSUE")-1);
                  output stg_wtch.wl_nbci_ind_transform_06;
				  orig_nid_part=kstrip(kscan(orig_nid,i,'OR'));
				  if indexw(orig_nid_part,tax_id) eq 1 then do;
				     identification_id=tax_id;
                     if i eq 1 then identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,length(identification_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
				  else do;
				     identification_id=tax_id;
					 if i eq 1 then if kindex(orig_nid_part,identification_id) > 1 then 
                        identification_type_desc=upcase(kstrip(ksubstr(orig_nid_part,1,kindex(orig_nid_part,tax_id)-1)));
					 else identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
			   end;
         end;
      end;
      else do;
         tax_id=kstrip(nid);
         if not missing(tax_id) and
		       anydigit(tax_id) and
               not indexw(tax_id,'EXPIRES') and
			   not indexw(tax_id,'EXPIRED') and
               not indexw(tax_id,'UNTIL')  then do;
			      if indexw(tax_id,"OFFICIALLY") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"OFFICIALLY")-1);
				  if indexw(tax_id,"IN") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"IN")-1);
				  if kindex(tax_id,"ISSUE") then tax_id=ksubstr(tax_id,1,kindex(tax_id,"ISSUE")-1);
                  output stg_wtch.wl_nbci_ind_transform_06;
				  if indexw(orig_nid,tax_id) eq 1 then do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid,length(identification_id)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
				  else do;
				     identification_id=tax_id;
					 identification_type_desc=upcase(kstrip(ksubstr(orig_nid,1,length(orig_nid)-length(nid)+1)));
					 if missing(identification_type_desc) then identification_type_desc="NATIONAL ID";
					 output stg_wtch.wl_nbci_ind_transform_04;
				  end;
			   end;
      end;
   end;

   /* Set all columns in if set 1=0 column to missing */
   call missing(&col_name_list);

run;

/* Delete duplicate records */
proc sort data=stg_wtch.wl_nbci_ind_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_01))
          out=stg_wtch.wl_nbci_ind_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_01, output_ds=Y))
          nodupkey;
by entity_watch_list_number place_of_birth;
run;

proc sort data=stg_wtch.wl_nbci_ind_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_02))
          out=stg_wtch.wl_nbci_ind_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_02, output_ds=Y))
          nodupkey;
by entity_watch_list_number dob;
run;

proc sort data=stg_wtch.wl_nbci_ind_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_03))
          out=stg_wtch.wl_nbci_ind_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_03, output_ds=Y))
          nodupkey;
by entity_watch_list_number citizenship_country_name;
run;

proc sort data=stg_wtch.wl_nbci_ind_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_04))
          out=stg_wtch.wl_nbci_ind_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_04, output_ds=Y))
          nodupkey;
by entity_watch_list_number identification_id;
run;

proc sort data=stg_wtch.wl_nbci_ind_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_05))
          out=stg_wtch.wl_nbci_ind_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_05, output_ds=Y))
          nodupkey;
by entity_watch_list_number full_address;
run;

proc sort data=stg_wtch.wl_nbci_ind_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_06))
          out=stg_wtch.wl_nbci_ind_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_06, output_ds=Y))
          nodupkey;
by entity_watch_list_number tax_id;
run;

/*******************************************************************************
 STEP DESCRIPTION:

   Cast date of birth into SAS date value

 INPUTS:

   stg_wtch.wl_nbci_ind_transform_02

 OUTPUTS:

   stg_wtch.wl_nbci_ind_transform_07
   stg_err.wl_nbci_errors_01

 NOTES:

   This step casts the date of birth column into a SAS date value.
   Date of birth formats supported by this step include:

   MM/DD/YYYY    - example, 01/31/1963
   YYYY          - example, 1963
   YYYY- YYYY    - example, 1963-1968
   YYYY (Approx) - example, 1963 (Approx)
   circa YYYY    - example, circa 1963
   
   Depending on the engine used to extract the data, some date fields will
   be text and others numeric.  For text fields, some Excel values will be
   their numeric representation, days since 1900.  Care must be taken when
   converting dates.
   
*******************************************************************************/
data
   stg_wtch.wl_nbci_ind_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_07, output_ds=Y)
                                     keep=
      entity_watch_list_number
      date_of_birth
      year_of_birth
   )

   stg_err.wl_nbci_errors_01(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_nbci_errors_01, output_ds=Y)
                             keep=
      error_watch_list_name
      error_record_id
      error_record_id_value
      error_file
      error_field
      error_value
      error_message
      error_datetime
   );

   retain error_flag 0;

   attrib
      first_dob                length=8   informat=best32.     format=best32.
      last_dob                 length=8   informat=best32.     format=best32.

      /* ERROR FIELDS */

      error_watch_list_name length=$4
                            informat=$4.
                            format=$4.
                            label='Error Watch List Name'

      error_record_id       length=$32
                            informat=$32.
                            format=$32.
                            label='Error Record ID'

      error_record_id_value length=$35
                            informat=$35.
                            format=$35.
                            label='Error Record Value'

      error_file            length=$200
                            informat=$200.
                            format=$200.
                            label='Error File'

      error_field           length=$32
                            informat=$32.
                            format=$32.
                            label='Error Field'

      error_value           length=$1000
                            informat=$1000.
                            format=$1000.
                            label='Error Value'

      error_message         length=$200
                            informat=$200.
                            format=$200.
                            label='Error Message'

      error_datetime        length=8
                            informat=nldatm.
                            format=nldatm.
                            label='Error Datetime'
    ;

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_nbci_ind_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_02)) end=eof;

   dob=upcase(compbl(kstrip(dob)));
   dob=tranwrd(dob,'CIRCA','');
   dob=tranwrd(dob,'(APPROX)','');
   dob=tranwrd(dob,'APPROXIMATELY','');
   dob=tranwrd(dob,'APPROX.','');
   dob=tranwrd(dob,'APPROX','');
   dob=tranwrd(dob,'DOB','');
   dob=tranwrd(dob,'(UNDER THIS ALIAS)','');
   dob=tranwrd(dob,'BETWEEN','');
   dob=kstrip(dob);

   if lengthn(dob) eq 9 and
      lengthn(kcompress(dob,'-')) eq 8 and
      anyalpha(kcompress(dob,'-')) eq 0 then do;

      first_dob=input(kscan(dob,1,'-'),best32.);
      last_dob=input(kscan(dob,2,'-'),best32.);

      do i=first_dob to last_dob;
         date_of_birth=.;
         year_of_birth=i;
         output stg_wtch.wl_nbci_ind_transform_07;
      end;
   end;

   /* Convert from Excel format, i.e. 31028 => 12DEC1984 */
   else if lengthn(dob) = 5 then do;
	  date_of_birth = dhms(input(dob,best.)-21916,0,0,0);
      year_of_birth=year(datepart(date_of_birth));
	  output stg_wtch.wl_nbci_ind_transform_07;
   end;

   /* Range of years ex 1977/79 */
   else if lengthn(dob) = 7 and 
        anyalpha(kcompress(dob,'/')) eq 0 then do;
	  first_dob=input(kscan(dob,1,'/'),best32.);
      last_dob=input(kscan(dob,2,'/'),best32.);
	  last_dob=int(first_dob/100)*100+last_dob;

      do i=first_dob to last_dob;
         date_of_birth=.;
         year_of_birth=i;
         output stg_wtch.wl_nbci_ind_transform_07;
      end;
   end;

   /* Between 1975 and 1979 */
   else if indexw(dob,"AND") then do;
	  first_dob=input(kscan(dob,1,'AND'),best32.);
      last_dob=input(kscan(dob,2,'AND'),best32.);
	 
      do i=first_dob to last_dob;
         date_of_birth=.;
         year_of_birth=i;
         output stg_wtch.wl_nbci_ind_transform_07;
      end;
   end;

   else if anyalpha(dob) eq 0 and
      lengthn(dob) eq 4 then do;
      date_of_birth=.;
      year_of_birth=input(dob,best32.);
      output stg_wtch.wl_nbci_ind_transform_07;
   end;

   else do;
      date_of_birth=input(dob,??mmddyy10.);

      if not missing(date_of_birth) then do;
         date_of_birth=dhms(date_of_birth,00,00,00);
         year_of_birth=year(datepart(date_of_birth));
         output stg_wtch.wl_nbci_ind_transform_07;
      end;
      /* WARNING: Invalid date of birth. */
      else if missing(date_of_birth) and missing(year_of_birth) then do;
         error_watch_list_name='NBCI';
         error_record_id='ID';
         error_record_id_value=ncbi_id;
         error_file="&nbci_file";
         error_record_number=ncbi_id;
         error_field='DOB';
         error_value=kstrip(dob);
         error_message='WARNING: Invalid date of birth.';
         error_datetime=datetime();
         error_flag=1;

         putlog 'WARNING: FCF: Invalid date of birth. ' entity_watch_list_number= dob=;
         output stg_err.wl_nbci_errors_01;
      end;
   end;

   /* Set all columns in if set 1=0 column to missing */
   call missing(&col_name_list);


   if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_NBCI_ERRORS_01 table.';
   end;

run;

proc sort data=stg_wtch.wl_nbci_ind_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_07)) nodupkey;
   by entity_watch_list_number
      date_of_birth
      year_of_birth;
run;

/*******************************************************************************
 STEP DESCRIPTION:

   Get indvidual NAME and TITLE

 INPUTS:

   stg_wtch.wl_nbci_ind_extract

 OUTPUTS:

   stg_wtch.wl_nbci_ind_transform_08

 NOTES:

*******************************************************************************/

data stg_wtch.wl_nbci_ind_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_08, output_ds=Y)
                                       keep=
   entity_watch_list_number
   watch_list_name
   category_desc
   type_desc
   first_name
   last_name
   middle_name
   entity_name
   entity_title
   create_date
   exclude_ind
);

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_nbci_ind_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_extract)
                                    keep=&col1_name
                                        name_1-name_4
                                        lastname
                                        title
                                        entry_date

                                        &rename_stmt
                                    );

   /* ENTITY_WATCH_LIST_NUMBER */
   if kindex(nbci_id,'.') then 
      entity_watch_list_number=kstrip(ksubstr(nbci_id,1,kindex(nbci_id,'.')-1));
   else entity_watch_list_number=kstrip(nbci_id);

   /* WATCH_LIST_NAME */
   watch_list_name='NBCI';

   /* CATEGORY_DESC */
   category_desc='TERRORISM';

   /* TYPE_DESC */
   type_desc='INDIVIDUAL';

   /* FIRST_NAME */
   first_name=upcase(compbl(kstrip(name_1)));

   /* LAST_NAME */
   last_name=upcase(compbl(kstrip(lastname)));

   /* MIDDLE_NAME */
   middle_name=upcase(compbl(kstrip(catx(' ',name_2, name_3, name_4))));

   /* ENTITY_NAME */
   entity_name=catx(' ',first_name, middle_name, last_name);

   /* ENTITY_TITLE */
   entity_title=compbl(kstrip(title));
   if kindex(entity_title,'(') then entity_title=ksubstr(entity_title,1,kindex(entity_title,'(')-1);

   /* CREATE_DATE */
   entry_date=kstrip(entry_date);
   if lengthn(entry_date) = 5 then do;
	  create_date = dhms(input(entry_date,best.)-21916,0,0,0);
   end;
   else if kindex(entry_date,'.') then do;
       entry_date=ksubstr(entry_date,1,kindex(entry_date,'.')-1);
       create_date=dhms(input(entry_date,??ddmmyy10.),0,0,0);
   end;
   else create_date=.;
  
   /* EXCLUDE_IND */
   exclude_ind='N';

run;

/*******************************************************************************
 STEP DESCRIPTION:

   Create records which include all permutations of name, place of birth,
   date of birth, citizenship/nationality, passport, address and tax id.

 INPUTS:

   stg_wtch.wl_nbci_ind_transform_01 - (place of birth0
   stg_wtch.wl_nbci_ind_transform_03 - (citizenship/nationality)
   stg_wtch.wl_nbci_ind_transform_04 - (passport)
   stg_wtch.wl_nbci_ind_transform_05 - (address)
   stg_wtch.wl_nbci_ind_transform_06 - (tax_id)
   stg_wtch.wl_nbci_ind_transform_07 - (date of birth)
   stg_wtch.wl_nbci_ind_transform_08 - (name/title)

 OUTPUTS:

   stg_wtch.wl_nbci_ind_transform_14 - all permutations

 NOTES:

*******************************************************************************/

proc sql noprint;

   create table stg_wtch.wl_nbci_ind_transform_09 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_09, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.place_of_birth
      from stg_wtch.wl_nbci_ind_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_08)) as a full join
           stg_wtch.wl_nbci_ind_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_01)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_nbci_ind_transform_10 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_10, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.citizenship_country_name,
             b.nationality_country_name
      from stg_wtch.wl_nbci_ind_transform_09(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_09)) as a full join
           stg_wtch.wl_nbci_ind_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_03)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_nbci_ind_transform_11 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_11, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.identification_id,
             b.identification_type_desc
      from stg_wtch.wl_nbci_ind_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_10)) as a full join
           stg_wtch.wl_nbci_ind_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_04)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_nbci_ind_transform_12 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_12, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.full_address
      from stg_wtch.wl_nbci_ind_transform_11(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_11)) as a full join
           stg_wtch.wl_nbci_ind_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_05)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_nbci_ind_transform_13 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_13, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.tax_id
      from stg_wtch.wl_nbci_ind_transform_12(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_12)) as a full join
           stg_wtch.wl_nbci_ind_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_06)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

   create table stg_wtch.wl_nbci_ind_transform_14 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_14, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.date_of_birth,
             b.year_of_birth
      from stg_wtch.wl_nbci_ind_transform_13(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_13)) as a full join
           stg_wtch.wl_nbci_ind_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_07)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;


quit;

/******************************************************************************/
/* END: INDIVIDUALS                                                           */
/******************************************************************************/


/******************************************************************************/
/* BEGIN: ORGANIZATIONS                                                       */
/******************************************************************************/

proc contents data=stg_wtch.wl_nbci_org_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_extract))
                                                out=cols (keep=name varnum) noprint;
  run;

  proc sql noprint;
	  select strip(name) into :col1_name
      from cols 
	  where varnum=1;
  quit;

  %let rename_stmt= rename=(&col1_name=nbci_id) ;

/*******************************************************************************
 STEP DESCRIPTION:

   This step processes and outputs a distinct record for each column involved
   in a series (example DOB_1 - DOB_9)

 INPUTS:

   stg_wtch.wl_nbci_ind_extract

 OUTPUTS:

   stg_wtch.wl_nbci_ind_transform_01 - (place of birth)
   stg_wtch.wl_nbci_ind_transform_02 - (date of birth)
   stg_wtch.wl_nbci_ind_transform_03 - (citizenship/nationality)
   stg_wtch.wl_nbci_ind_transform_04 - (passport)
   stg_wtch.wl_nbci_ind_transform_05 - (address)
   stg_wtch.wl_nbci_ind_transform_06 - (tax_id)

 NOTES:

   The following column series are processed by this step:

   - POB_1-POB_2
   - POB_1_COUNTRY-POB_2_COUNTRY
   - DOB_1-DOB_9
   - NATIONALITY_1-NATIONALITY_3
   - PASSPORT_1-PASSPORT_4
   - ADDRESS-ADDRESS_5

   Additional customizations may be required when processing NID column.

*******************************************************************************/

data

   stg_wtch.wl_nbci_org_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_01, output_ds=Y)
                                     keep=
      entity_watch_list_number
      full_address
   );

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_nbci_org_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_extract)
                                    keep=&col1_name address_1-address_14
                                    &rename_stmt);

   /* Remove blank records */
   if not(missing(nbci_id));
   
   if kindex(nbci_id,'.') then 
      entity_watch_list_number=kstrip(ksubstr(nbci_id,1,kindex(nbci_id,'.')-1));
   else entity_watch_list_number=kstrip(nbci_id);

   /**************************/
   /* ADDRESS_1 - ADDRESS_14 */
   /**************************/

   /* ADDRESS_1 */
   if not missing(address_1) then do;
      full_address=compbl(kstrip(address_1));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_2 */
   if not missing(address_2) then do;
      full_address=compbl(kstrip(address_2));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_3 */
   if not missing(address_3) then do;
      full_address=compbl(kstrip(address_3));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_4 */
   if not missing(address_4) then do;
      full_address=compbl(kstrip(address_4));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_5 */
   if not missing(address_5) then do;
      full_address=compbl(kstrip(address_5));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_6 */
   if not missing(address_6) then do;
      full_address=compbl(kstrip(address_6));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_7 */
   if not missing(address_7) then do;
      full_address=compbl(kstrip(address_7));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_8 */
   if not missing(address_8) then do;
      full_address=compbl(kstrip(address_8));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_9 */
   if not missing(address_9) then do;
      full_address=compbl(kstrip(address_9));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_10 */
   if not missing(address_10) then do;
      full_address=compbl(kstrip(address_10));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_11 */
   if not missing(address_11) then do;
      full_address=compbl(kstrip(address_11));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_12 */
   if not missing(address_12) then do;
      full_address=compbl(kstrip(address_12));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_13 */
   if not missing(address_13) then do;
      full_address=compbl(kstrip(address_13));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

   /* ADDRESS_14 */
   if not missing(address_14) then do;
      full_address=compbl(kstrip(address_14));
      output stg_wtch.wl_nbci_org_transform_01;
   end;

run;

/* Delete duplicate records */
proc sort data=stg_wtch.wl_nbci_org_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=stg_wtch.wl_nbci_org_transform_01))
          out=stg_wtch.wl_nbci_org_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=stg_wtch.wl_nbci_org_transform_01, output_ds=Y))
          nodupkey;
by entity_watch_list_number full_address;
run;


/*******************************************************************************
 STEP DESCRIPTION:

   Get organization NAME

 INPUTS:

   stg_wtch.wl_nbci_org_extract

 OUTPUTS:

   stg_wtch.wl_nbci_org_transform_02

 NOTES:

*******************************************************************************/

data stg_wtch.wl_nbci_org_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_02, output_ds=Y)
                                       keep=
   entity_watch_list_number
   watch_list_name
   category_desc
   type_desc
   entity_name
   create_date
   exclude_ind
);

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
      set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set stg_wtch.wl_nbci_org_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_extract)
                                    keep=&col1_name name entry_date
                                    &rename_stmt);

   /* Remove blank records */
   if not(missing(nbci_id));

   /* ENTITY_WATCH_LIST_NUMBER */
   if kindex(nbci_id,'.') then 
      entity_watch_list_number=kstrip(ksubstr(nbci_id,1,kindex(nbci_id,'.')-1));
   else entity_watch_list_number=kstrip(nbci_id);

   /* WATCH_LIST_NAME */
   watch_list_name='NBCI';

   /* CATEGORY_DESC */
   category_desc='TERRORISM';

   /* TYPE_DESC */
   type_desc='ORGANIZATION';

   /* ENTITY_NAME */
   entity_name=upcase(compbl(kstrip(name)));

   /* CREATE_DATE */
   entry_date=kstrip(entry_date);
   entry_date=tranwrd(entry_date,"*","");
   entry_date=kstrip(entry_date);
   if lengthn(entry_date) = 5 then do;
	  create_date = dhms(input(entry_date,best.)-21916,0,0,0);
   end;
   else if kindex(entry_date,'.') then do;
       entry_date=ksubstr(entry_date,1,kindex(entry_date,'.')-1);
       create_date=dhms(input(entry_date,??ddmmyy10.),0,0,0);
   end;
   else if anyalpha(entry_date) eq 0 then create_date=dhms(input(entry_date,??ddmmyy10.),0,0,0);
   else create_date=.;
  

   /* EXCLUDE_IND */
   exclude_ind='N';

   if not missing(entity_watch_list_number) then
      output;

run;


/*******************************************************************************
 STEP DESCRIPTION:

   Create records which include all permutations of name and address.

 INPUTS:

   stg_wtch.wl_nbci_org_transform_01 - (address)
   stg_wtch.wl_nbci_org_transform_02 - (name)

 OUTPUTS:

   stg_wtch.wl_nbci_org_transform_03 - all permutations

 NOTES:

*******************************************************************************/

proc sql noprint;

   create table stg_wtch.wl_nbci_org_transform_03 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_03, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.full_address
      from stg_wtch.wl_nbci_org_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_02)) as a full join
           stg_wtch.wl_nbci_org_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_01)) as b
      on a.entity_watch_list_number=b.entity_watch_list_number;

quit;

/******************************************************************************/
/* END: ORGANIZATIONS                                                         */
/******************************************************************************/

/*******************************************************************************
 STEP DESCRIPTION:

   Append individuals and organizations.

 INPUTS:

   stg_wtch.wl_nbci_ind_transform_14 - (final individual)
   stg_wtch.wl_nbci_org_transform_03 - (final organization)

 OUTPUTS:

   stg_wtch.wl_nbci_transform - final individal/organization

 NOTES:

*******************************************************************************/

/*  proc append base=stg_wtch.wl_nbci_transform*/
/*              data=stg_wtch.wl_nbci_ind_transform_14*/
/*              force;*/
/*   run;*/
/**/
/*  proc append base=stg_wtch.wl_nbci_transform*/
/*              data=stg_wtch.wl_nbci_org_transform_03*/
/*              force;*/
/*   run;*/

data stg_wtch.wl_nbci_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_transform, output_ds=Y));
    set stg_wtch.wl_nbci_ind_transform_14(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_ind_transform_14))
        stg_wtch.wl_nbci_org_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_nbci_org_transform_03));
run;

  %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(post&_fcfmodule_);
      %fcf_execute_macro(fcf_success);
   %end;

%mend fcf_wl_nbci_transform;
