/*==============================================================================
| NAME:    fcf_wl_osfi_transform
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

WATCH LIST PROVIDER - Office of the Superintendent of Financial Institutions Canada
WATCH LIST NAME     - Terrorism Financing (Lists of Names)
WATCH LIST CODE     - OSFI
WEB LINK            - http://www.osfi-bsif.gc.ca/osfi/index_e.aspx?DetailID=525

MODULE TYPE         - TRANSFORM
MODULE PURPOSE      - Transform data in prepartion for data load.

MODULE NOTES
------------
None

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_wl_osfi_transform;

  /*------------
   * Set module
   *------------*/
  %local _fcfmodule_;
  %let _fcfmodule_=fcf_WL_OSFI_TRANSFORM;

  /*---------------------
   * Call pre macro stub
   *---------------------*/
  %fcf_execute_macro(pre&_fcfmodule_);

  /*------------------------------------------
   * Set error code to 0 and message to blank
   *------------------------------------------*/
  %let trans_rc = 0;
  %let _fcfmsg_=;

/******************************************************************************/
/* BEGIN: Obtain maximum field lengths for ID, POB, DOB and NATIONALITY       */
/*                                                                            */
/* DESCRIPTION: The maximum field length values for these fields is required  */
/* by the normalization step below.                                           */
/******************************************************************************/
proc sql noprint;
   select max(id_L),
          max(pob_L),
          max(dob_L),
          max(nationality_L)
   into   :L1,
          :L2,
          :L3,
          :L4
   from STG_WTCH.wl_osfi_indstld_extract_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_extract_01));
quit;
/******************************************************************************/
/* END: Obtain maximum field lengths for ID, POB, DOB and NATIONALITY         */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_indstld_transform_01 - POB                         */
/*        STG_WTCH.wl_osfi_indstld_transform_02 - DOB                         */
/*        STG_WTCH.wl_osfi_indstld_transform_03 - NATIONALITY                 */
/*                                                                            */
/* DESCRIPTION: This step generates distinct records for each instance of a   */
/* non-null POB, DOB and NATIONALITY value for a given ID                     */
/*                                                                            */
/* NOTE: The OSFI ID consists of two components. The first component          */
/* represents a unique entity number. The second component represents a       */
/* sequence number that is incremented by 1 for each alias name instance.     */
/* The two components comprising the ID number are separated by a semi-colon. */
/******************************************************************************/
data

STG_WTCH.wl_osfi_indstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_01, output_ds=Y)
                                      keep=
   id_2
   pob
)

STG_WTCH.wl_osfi_indstld_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_02, output_ds=Y)
                                      keep=
   id_2
   dob
)

STG_WTCH.wl_osfi_indstld_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_03, output_ds=Y)
                                      keep=
   id_2
   nationality
)
;

   length
      id_2        $ &L1
      pob         $ &L2
      dob         $ &L3
      nationality $ &L4
   ;

   set STG_WTCH.wl_osfi_indstld_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_extract)
                                        keep=id
                                             pob
                                             alt_pob
                                             dob
                                             alt_dob_1
                                             alt_dob_2
                                             alt_dob_3
                                             nationality
                                             alt_nationality_1
                                             alt_nationality_2
                                       );

   id_2=kstrip(kscan(id,1,'.'));
   
   if not missing(pob) then output STG_WTCH.wl_osfi_indstld_transform_01;

   if not missing(alt_pob) then do;
      pob=kstrip(alt_pob);
      output STG_WTCH.wl_osfi_indstld_transform_01;
   end;

    if not missing(dob) then do;
	  dob=kstrip(dob);
      if kindex(dob,'-') and not kindex(dob,'/') then do;
		 dob=kstrip(tranwrd(upcase(dob),"APPROXIMATELY",""));
		 loc=find(dob,'-');
	     year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+1));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else if kindex(upcase(dob),'BETWEEN') then do;
	     dob=tranwrd(upcase(dob),'BETWEEN','');
         loc=find(upcase(dob),'AND');
		 year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+3));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else output STG_WTCH.wl_osfi_indstld_transform_02;
   end;

   if not missing(alt_dob_1) then do;
	  dob=kstrip(alt_dob_1);
	  if kindex(dob,'-') and not kindex(dob,'/') then do;
		 dob=kstrip(tranwrd(upcase(dob),"APPROXIMATELY",""));
		 loc=find(dob,'-');
	     year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+1));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		  end;
	  end;
	  else if kindex(upcase(dob),'BETWEEN') then do;
	     dob=tranwrd(upcase(dob),'BETWEEN','');
         loc=find(upcase(dob),'AND');
		 year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+3));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else output STG_WTCH.wl_osfi_indstld_transform_02;
   end;

   if not missing(alt_dob_2) then do;
      dob=kstrip(alt_dob_2);
	  if kindex(dob,'-') and not kindex(dob,'/') then do;
		 dob=kstrip(tranwrd(upcase(dob),"APPROXIMATELY",""));
		 loc=find(dob,'-');
	     year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+1));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else if kindex(upcase(dob),'BETWEEN') then do;
	     dob=tranwrd(upcase(dob),'BETWEEN','');
         loc=find(upcase(dob),'AND');
		 year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+3));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else output STG_WTCH.wl_osfi_indstld_transform_02; 
   end;

   if not missing(alt_dob_3) then do;
      dob=kstrip(alt_dob_3);
	  if kindex(dob,'-') and not kindex(dob,'/') then do;
		 dob=kstrip(tranwrd(upcase(dob),"APPROXIMATELY",""));
		 loc=find(dob,'-');
	     year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+1));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		  end;
	  end;
	  else if kindex(upcase(dob),'BETWEEN') then do;
	     dob=tranwrd(upcase(dob),'BETWEEN','');
         loc=find(upcase(dob),'AND');
		 year1=kstrip(substr(dob,1,loc-1));
		 year2=kstrip(substr(dob,loc+3));
		 if notdigit(year1) eq 0 then do;
            do i=year1 to year2;
		       dob=put(i,4.);
               output STG_WTCH.wl_osfi_indstld_transform_02;
		    end;
		 end;
	  end;
	  else output STG_WTCH.wl_osfi_indstld_transform_02; 
   end;

   if not missing(nationality) then output STG_WTCH.wl_osfi_indstld_transform_03;

   if not missing(alt_nationality_1) then do;
      nationality=kstrip(alt_nationality_1);
      output STG_WTCH.wl_osfi_indstld_transform_03;
   end;

   if not missing(alt_nationality_2) then do;
      nationality=kstrip(alt_nationality_2);
      output STG_WTCH.wl_osfi_indstld_transform_03;
   end;

run;

/* Remove duplicate records */
proc sort data=STG_WTCH.wl_osfi_indstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_01))
          out=STG_WTCH.wl_osfi_indstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_01, output_ds=Y))
          nodupkey;
   by id_2 pob;
run;

proc sort data=STG_WTCH.wl_osfi_indstld_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_02))
          out=STG_WTCH.wl_osfi_indstld_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_02, output_ds=Y))
          nodupkey;
   by id_2 dob;
run;

proc sort data=STG_WTCH.wl_osfi_indstld_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_03))
          out=STG_WTCH.wl_osfi_indstld_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_03, output_ds=Y))
          nodupkey;
   by id_2 nationality;
run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_indstld_transform_01 - POB                           */
/*      STG_WTCH.wl_osfi_indstld_transform_02 - DOB                           */
/*      STG_WTCH.wl_osfi_indstld_transform_03 - NATIONALITY                   */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_indstld_transform_04                               */
/*                                                                            */
/* DESCRIPTION: Create individuals table which excludes POB, DOB and          */
/* NATIONALITY fields as this table will be used for joining purposes.        */
/******************************************************************************/
data STG_WTCH.wl_osfi_indstld_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_04, output_ds=Y));

   length id_2 $ &L1;

   set STG_WTCH.wl_osfi_indstld_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_extract)
                                        drop=pob
                                             alt_pob
                                             dob
                                             alt_dob_1
                                             alt_dob_2
                                             alt_dob_3
                                             nationality
                                             alt_nationality_1
                                             alt_nationality_2
                                       );

   id_2=kstrip(kscan(id,1,'.'));

run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_indstld_transform_04                                 */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_indstld_transform_05                               */
/*        STG_WTCH.wl_osfi_indstld_transform_06                               */
/*        STG_WTCH.wl_osfi_indstld_transform_07                               */
/*        STG_WTCH.wl_osfi_indstld_transform_08                               */
/*                                                                            */
/* DESCRIPTION: The final output table 'STG_WTCH.wl_osfi_indstld_transform_08'*/
/* from this step contains all possible data permutations for each POB, DOB   */
/* and NATIONALITY value.                                                     */
/******************************************************************************/
proc sql noprint;

   create table STG_WTCH.wl_osfi_indstld_transform_05 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_05, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.pob
      from STG_WTCH.wl_osfi_indstld_transform_04(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_04)) as a full join
           STG_WTCH.wl_osfi_indstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_01)) as b
      on a.id_2=b.id_2;

   create table STG_WTCH.wl_osfi_indstld_transform_06 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_06, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.dob
      from STG_WTCH.wl_osfi_indstld_transform_05(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_05)) as a full join
           STG_WTCH.wl_osfi_indstld_transform_02(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_02)) as b
      on a.id_2=b.id_2;

   create table STG_WTCH.wl_osfi_indstld_transform_07 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_07, output_ds=Y, add_parens=Y)  as
      select a.*,
             b.nationality
      from STG_WTCH.wl_osfi_indstld_transform_06(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_06)) as a full join
           STG_WTCH.wl_osfi_indstld_transform_03(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_03)) as b
      on a.id_2=b.id_2;

   create table STG_WTCH.wl_osfi_indstld_transform_08 %fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_08, output_ds=Y, add_parens=Y)  as
      select *
      from STG_WTCH.wl_osfi_indstld_transform_07(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_07))
      where pob ne '' or
            dob ne '' or
            nationality ne '';

quit;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_indstld_transform_05                                 */
/*      STG_WTCH.wl_osfi_indstld_transform_06                                 */
/*      STG_WTCH.wl_osfi_indstld_transform_07                                 */
/*      STG_WTCH.wl_osfi_indstld_transform_08                                 */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_indstld_transform_09                               */
/*                                                                            */
/* DESCRIPTION: Re-build the individuals table to include the new rows        */
/* generated as a result of the normalizing POB, DOB and NATIONALITY.         */
/******************************************************************************/
/*proc append base=STG_WTCH.wl_osfi_indstld_transform_09*/
/*            data=STG_WTCH.wl_osfi_indstld_extract(drop=alt_pob*/
/*                                                       alt_dob_1*/
/*                                                       alt_dob_2*/
/*                                                       alt_dob_3*/
/*                                                       alt_nationality_1*/
/*                                                       alt_nationality_2)*/
/*            force nowarn;*/
/*run;*/

proc append base=STG_WTCH.wl_osfi_indstld_transform_09(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_09, output_ds=Y))
            data=STG_WTCH.wl_osfi_indstld_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_08) drop=id_2)
            force nowarn;
run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_indstld_transform_09                                 */
/*                                                                            */
/* DESCRIPTION: Re-build the individuals table to include the new rows        */
/* generated as a result of the normalizing POB, DOB and NATIONALITY.         */
/******************************************************************************/


/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_indstld_transform_10                               */
/*                                                                            */
/* DESCRIPTION: Final transform table for Part A - Individuals extract.       */
/******************************************************************************/
data

STG_WTCH.wl_osfi_indstld_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_10, output_ds=Y)
                                      keep=
   entity_watch_list_key
   entity_watch_list_number
   watch_list_name
   category_desc
   type_desc
   programs
   tax_id
   tax_id_type_code
   identification_id
   identification_type_desc
   date_of_birth
   year_of_birth
   place_of_birth
   deceased_ind
   first_name
   last_name
   middle_name
   entity_name
   match_code_individual
   match_code_organization
   entity_title
   occupation_desc
   address
   city_name
   state_name
   postal_code
   country_code
   country_name
   full_address
   match_code_full_address
   citizenship_country_code
   citizenship_country_name
   match_code_citizenship
   nationality_country_code
   nationality_country_name
   match_code_nationality
   org_country_of_business_code
   org_country_of_business_name
   match_code_org_country
   politically_exposed_person_ind
   create_date
   update_date
   exclude_ind
   change_begin_date
   change_end_date
   change_current_ind
)

STG_ERR.wl_osfi_errors(%fcf_get_ds_opts(lib_name=stg_err, ds_name=wl_osfi_errors, output_ds=Y)
                       keep=
   entity_watch_list_number
   watch_list_name
   error_file
   error_record_number
   error_field
   error_value
   error_message
   error_datetime
);

   retain error_flag 0;
   
   attrib
      /* ERROR FIELDS */
      error_file          length=$200
                          informat=$200.
                          format=$200.
                          label='Error File'

      error_record_number length=8
                          informat=best32.
                          format=best32.
                          label='Error Record Number'

      error_field         length=$32
                          informat=$32.
                          format=$32.
                          label='Error Field'

      error_value         length=$1000
                          informat=$1000.
                          format=$1000.
                          label='Error Value'

      error_message       length=$200
                          informat=$200.
                          format=$200.
                          label='Error Message'

      error_datetime      length=8
                          informat=nldatm.
                          format=nldatm.
                          label='Error Datetime'
   ;

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
         set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set STG_WTCH.wl_osfi_indstld_transform_08(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_08)
                                             rename=(first_name=osfi_first_name
                                                     last_name=osfi_last_name)) end=eof;


/******************************************************************************/
/* NOTE:                                                                      */
/*                                                                            */
/* Initial values for the fields listed below are set to missing as these     */
/* fields values are set by later processes.                                  */
/******************************************************************************/

   /* Fields set by load */
   entity_watch_list_key=.;
   change_begin_date=.;
   change_end_date=.;
   change_current_ind='';

/* ************************************************************************ */
/* NOTE:                                                                    */
/*                                                                          */
/* The following fields are set to null as this watch list extract does not */
/* currently include this data.                                             */
/* ************************************************************************ */
   programs='';
   deceased_ind='';
   citizenship_country_name='';
   politically_exposed_person_ind='';
   create_date=.;
   update_date=.;

/* ************************************************************************ */
/* NOTE:                                                                    */
/*                                                                          */
/* The following fields are set to null due to the complexity of required   */
/* transformations. This does not preclude users from writing               */
/* transformation code which could populate these fields.                   */
/* ************************************************************************ */
   tax_id='';                   /* source: OTHER_INFO */
   tax_id_type_code='';         /* source: set=S when tax_id is populated */
   identification_id='';        /* source: OTHER_INFO */
   identification_type_desc=''; /* source: set value when identification_id */
                                /*         is populated */
   entity_title='';             /* source: OTHER_INFO */
   occupation_desc='';          /* source: OTHER_INFO */
   address='';                  /* source: OTHER_INFO */
   city_name='';                /* source: OTHER_INFO */
   state_name='';               /* source: OTHER_INFO */
   postal_code='';              /* source: OTHER_INFO */
   country_code='';             /* source: OTHER_INFO */
   country_name='';             /* source: OTHER_INFO */
   full_address='';             /* source: OTHER_INFO */


/* ************************************************************************ */
/* NOTE:                                                                    */
/*                                                                          */
/* The following fields are not applicable where type_desc='ORGANIZATION'.  */
/* These fields are therefore set to null.                                  */
/* ************************************************************************ */
   org_country_of_business_code='';
   org_country_of_business_name='';

   /* ENTITY_WATCH_LIST_NUMBER */
   entity_watch_list_number=kstrip(kscan(id,1,'.'));

   /* WATCH_LIST_NAME */
   watch_list_name='OSFI';

   /* CATEGORY_DESC */
   category_desc='TERRORISM';

   /* TYPE_DESC */
   type_desc='INDIVIDUAL';

   /* DATE_OF_BIRTH
      YEAR_OF_BIRTH */
   /* NOTE: OSFI DOB format=DD/MM/YY */

   /* Missing DOB */
   orig_dob=dob;
   if missing(dob) then do;
      date_of_birth=.;
      year_of_birth=.;
   end;

   /* Non-missing DOB */
   else if not missing(dob) then do;
      dob=kstrip(tranwrd(upcase(dob),'APPROXIMATELY',''));
      dob=kstrip(tranwrd(upcase(dob),'APPROXIMITELY',''));

      /* Only year_of_birth is available */
      if lengthn(dob) eq 4 and not anyalpha(dob) then do;
         date_of_birth=.;
         year_of_birth=input(dob,best32.);
      end;

      /* Valid DD/MM/YY format */
      else if lengthn(kstrip(kcompress(dob,'/'))) eq 6 and
         not anyalpha(kcompress(dob,'/')) then do;

         date_of_birth=input(dob,??ddmmyy8.);

         if not missing(date_of_birth) then do;
            date_of_birth=dhms(date_of_birth,00,00,00);
            year_of_birth=year(datepart(date_of_birth));
         end;

         /* WARNING: Invalid date of birth */
         else if missing(date_of_birth) then do;

            year_of_birth=.;

            error_file="&osfi_indstld_file";
            error_record_number=_N_;
            error_field='DOB';
            error_value=kstrip(orig_dob);
            error_message='WARNING: Invalid date of birth.';
            error_datetime=datetime();
            error_flag=1;

            putlog 'WARNING: FCF: Invalid date of birth. ' id= orig_dob=;
            output STG_ERR.wl_osfi_errors;

         end; /* end: Invalid date of birth */
      end; /* end: valid DD/MM/YY format */

      /* Invalid DD/MM/YY format */
      /* WARNING: Invalid date of birth */
      else if lengthn(kstrip(kcompress(dob,'/'))) ne 6 or
         anyalpha(kcompress(dob,'/')) then do;

         date_of_birth=.;
         year_of_birth=.;

         error_file="&osfi_indstld_file";
         error_record_number=_N_;
         error_field='DOB';
         error_value=kstrip(orig_dob);
         error_message='WARNING: Invalid date of birth.';
         error_datetime=datetime();
         error_flag=1;

         putlog 'WARNING: FCF: Invalid date of birth. ' id= orig_dob=;
         output STG_ERR.wl_osfi_errors;

      end; /* end: Invalid DD/MM/YY format */
   end; /* end: Non-missing date of birth */

   /* PLACE_OF_BIRTH */
   pob=tranwrd(upcase(pob),"IN THE AREA ","");
   place_of_birth=upcase(compbl(pob));

   /* FIRST_NAME */
   first_name=upcase(compbl(osfi_first_name));

   /* LAST_NAME */
   last_name=upcase(compbl(osfi_last_name));

   /* MIDDLE_NAME */
   middle_name=upcase(compbl(catx(' ',second_name,third_name,fourth_name)));

   /* NAME */
   entity_name=catx(' ',first_name,middle_name,last_name);

   /* NATIONALITY_COUNTRY_NAME */
   nationality_country_name=upcase(compbl(kstrip(nationality)));

   /* EXCLUDE_IND */
   exclude_ind='N';

   output STG_WTCH.wl_osfi_indstld_transform_10;
             
   if (eof) and (error_flag) then do;
      putlog ' ';
      putlog 'WARNING: FCF: Invalid values were encountered processing the watch list data';
      putlog 'WARNING: FCF: Data values were written to the STG_ERR.WL_OSFI_ERRORS_01 table.';
   end;

run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_indstld_transform_10                                 */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_entstld_transform_01                               */
/*                                                                            */
/* DESCRIPTION: Final transform table for Part B - Entities extract.          */
/******************************************************************************/
data STG_WTCH.wl_osfi_entstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_entstld_transform_01, output_ds=Y)
                                           keep=
   entity_watch_list_number
   watch_list_name
   category_desc
   type_desc
   entity_name
   full_address
   exclude_ind
);

   /* FSC_ENTITY_WATCH_LIST_DIM */
   /* Sets variable attribs to match variables in DBMS */
   if 1 eq 0 then
         set seg_kc.FSC_ENTITY_WATCH_LIST_DIM;

   set STG_WTCH.wl_osfi_entstld_extract(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_entstld_extract) rename=(address=osfi_address));

   /* ENTITY_WATCH_LIST_NUMBER */
   entity_watch_list_number=kstrip(kscan(id,1,'.'));

   /* WATCH_LIST_NAME */
   watch_list_name='OSFI';

   /* CATEGORY_DESC */
   category_desc='TERRORISM';

   /* TYPE_DESC */
   type_desc='ORGANIZATION';

   /* NAME */
   entity_name=upcase(compbl(kstrip(entity)));

   /* EXCLUDE_IND */
   exclude_ind='N';

   /* FULL_ADDRESS */
   /* Parse this field to remove phone numbers and other information */
   /* This field may include multiple addressess                     */
   full_address=upcase(compbl(kstrip(osfi_address)));
    num=countc(osfi_address,';')+1;
	do i = 1 to num;
	   if kindex(osfi_address,';') eq 0 then full_address=compbl(osfi_address);
	   else full_address=compbl(strip(scan(osfi_address,i,';')));
	   full_address=tranwrd(full_address,'Address: ','');
	   if kindex(upcase(full_address),'OTHER INFORMATION') 
         or kindex(upcase(full_address),'PHONE')
         or kindex(upcase(full_address),'FAX') then do;
          full_address='';
		  if i eq 1 and num eq 1 then output;
	   end;
	   else output;
	end;

run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_entstld_transform_01                                 */
/******************************************************************************/

/******************************************************************************/
/* BEGIN: STG_WTCH.wl_osfi_transform                                          */
/*                                                                            */
/* DESCRIPTION: Final transform table for Part A - Individuals extract        */
/*              and Part B - Entities extract                                 */
/******************************************************************************/
data STG_WTCH.wl_osfi_transform(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_transform, output_ds=Y));
     set STG_WTCH.wl_osfi_indstld_transform_10(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_indstld_transform_10))
         STG_WTCH.wl_osfi_entstld_transform_01(%fcf_get_ds_opts(lib_name=stg_wtch, ds_name=wl_osfi_entstld_transform_01));
run;
/******************************************************************************/
/* END: STG_WTCH.wl_osfi_transform                                            */
/******************************************************************************/

  %if &trans_rc eq 3 or &trans_rc ge 5 %then
     %fcf_execute_macro(fcf_error);
  %else %do;
     %fcf_execute_macro(post&_fcfmodule_);
     %fcf_execute_macro(fcf_success);
  %end;

%mend fcf_wl_osfi_transform;

