/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_driver.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Driver program for generating finCEN e-files. 
|          Multiple e-files will be generated if the number of reports is 
|          more than the maximum allowed report counts defined in SEG_KC.FORM_CONFIG.  
|          
+-----------------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key (e.g. 301, 312)
|          form_type_code  = form type (e.g. SAR, CTRX)
|          efile_key       = e-file key for regenerating existing e-file  
|                            if this parameter is blank, all reports with READY_CODE='R' 
|                            will be included in the e-file.
|          testYN          = Y to create e-file(s) without updating database data
|                            N to create e-file(s) with database update 
+------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%macro rc_efile_fincen_driver(form_config_key=,
                              efile_key=,
                              testYN=N
                             );
     

    /*------------
     * Set Module
     *------------*/
    %let _fcfmodule_=RC_EFILE_FINCEN_DRIVER;

    /*-----------------------------
     * Set start time for jobstats
     *-----------------------------*/
    %fcf_section_header(section=&_fcfmodule_, message_format=verbose);
    %put;

    %let trans_rc=0;
    %let _fcfmsg_=;

    %if %str(&form_config_key)=%str() %then %do;
        %let _fcfmsg_=ERROR: Parameter form_config_key is required.;
        %fcf_put_error;
        %fcf_rcset(5);
        %goto MACRO_END;
    %end;
 
    /* set missing option to blank to avoid '.' being used as missing values */
    options missing=' '; 

    %let form_type_code=;

    /* retrieve efile config info */ 
    proc sql noprint;
         select  
             upcase(trim(form_type_code))
             into 
             :form_type_code                     
         from seg_kc.fsk_rc_form_config
         where form_config_key=&form_config_key;
    quit;

    %let form_type_code=%trim(&form_type_code);

    %if %nrbquote(&form_type_code)=%nrbquote() %then %do;
        %let _fcfmsg_=Error: RC: Form key &form_config_key is not defined in SEG_KC.FSK_RC_FORM_CONFIG.;
        %fcf_put_error;
        %fcf_rcset(3);
        %goto MACRO_END;
    %end;
     

    /*-------------------------------------------
     * Retrieve the list of reports to process
     *-------------------------------------------*/
   
   
    %let report_list=;
    %let report_cnt=0;


    %if &efile_key = %then 
    %do; /* if e-file key is not provided, all 'ready to submit' reports are included */

        proc sql noprint;
             select  strip(put(report_key,best.)) into :report_list separated by ' '
             from seg_kc.fsk_rc_report
             where report_ready_code='R' and form_config_key=&form_config_key
             ;
             %let report_cnt=&sqlobs; 
        quit;

    %end;
    %else 
    %do; /* if e-file key is provided, retrieve the reports under this e-file */
        proc sql noprint;
             select  strip(put(erpt.report_key,best.)),
                     strip(put(ef.form_config_key,best.)) 
             into :report_list separated by ' ',
                  :efile_form_config_key
             from seg_kc.fsk_rc_efile_report as erpt,
                  seg_kc.fsk_rc_efile as ef
             where ef.efile_key=&efile_key and
                   erpt.efile_key=ef.efile_key
             ;
             %let report_cnt=&sqlobs;
            
        quit;
        
        %if &efile_form_config_key ne &form_config_key %then %do;
            %let _fcfmsg_=ERROR: Parameter form_config_key (%trim(&form_config_key)) does not match with form_config_key (%trim(&efile_form_config_key)) in fsk_rc_efile.;
            %fcf_put_error;
            %fcf_rcset(5);
            %goto MACRO_END;
        %end;
    %end;

    %if &report_cnt=0 %then %do;
        %put;
        %put **********************************************************************;
        %put * NOTE: No &form_type_code reports are found for submission.          ;
        %put **********************************************************************;
        %put; 
        %goto MACRO_END;
    %end;

    %if &form_type_code = SAR or &form_type_code=CTR or &form_type_code=DOEP  %then
    %do; /* for the .dat based FinCEN forms */

       /*--------------------------------------------------
        * retreive and summarize report and location data
        *-------------------------------------------------*/
   
        %rc_efile_fincen_data(report_list=%superq(report_list),
                             efile_key=&efile_key,
                             form_config_key=&form_config_key,
                             form_type_code=&form_type_code
                             );

        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

       /*--------------------------------------------------
        * Use the output data from rc_efile_fincen_data to
        * generate .dat e-files.
        *-------------------------------------------------*/ 
 
        %rc_efile_fincen_generate( form_config_key=&form_config_key,
                                   form_type_code=&form_type_code,
                                   efile_key=&efile_key,
                                   efile_outdsn=rc_efile,
                                   efile_report_outdsn=rc_efile_report
                                 );
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

       /*--------------------------------------------------
        * Create SAR attachment .zip file 
        *-------------------------------------------------*/ 

        %if &form_type_code=SAR %then %do;
            %rc_efile_fincen_zip_sar(form_config_key=&form_config_key,
                                     form_type_code=&form_type_code, 
                                     efile_indsn=rc_efile
                                    );
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;

        %end;

    %end;

    %else
    %do; /* for the .xml based FinCEN forms */
 
       /*--------------------------------------------------
        * Retrive report and location data 
        *-------------------------------------------------*/ 
        %if &form_type_code=CTRX %then
        %do;
            /* assign efile keys if none is provided*/ 
            /* use regulatoryReports/nodes REST service to get data for all efiles */
            %rc_efile_fincen_data_x(report_list=%superq(report_list),
                                    efile_key=&efile_key,
                                    form_config_key=&form_config_key,
                                    form_type_code=&form_type_code
                                   );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;

            %rc_efile_fincen_generate_x( form_config_key=&form_config_key,
                                         form_type_code=&form_type_code,
                                         efile_key=&efile_key,
                                         efile_outdsn=rc_efile,
                                         efile_report_outdsn=rc_efile_report
                                 );
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;
          
        %end;
        %else
        %do; 
            /* assign e-file key to reports */         
            %rc_efile_assign_efile_key(report_list=%superq(report_list),
                                       form_config_key=&form_config_key,
                                       form_type_code=&form_type_code,
                                       efile_key=&efile_key,
                                       out_dsn=seq_report
                                       );

            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;
            
 

            /* use regulatoryReports/efileCollection rest service to get data for each efile */
            /* then generate .xml efile(s)                                                   */       
            %rc_efile_fincen_generate_x2( form_config_key=&form_config_key,
                                          form_type_code=&form_type_code,
                                          efile_key=&efile_key,
                                          efile_outdsn=rc_efile,
                                          efile_report_outdsn=rc_efile_report
                                        );
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end;
        %end;     
    %end;
    



   /*--------------------------------------------------
    * Generate e-files for submission via SDTM  
    *-------------------------------------------------*/ 
    %if %upcase(&&&rc_sdtm_YN_&form_type_code)=Y %then 
    %do;
        %rc_efile_fincen_sdtm(form_config_key=&form_config_key, 
                              form_type_code=&form_type_code, 
                              efile_indsn=rc_efile
                              );
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;
     
    %end;
 


  
   /*--------------------------------------------------
    * Update database data if it is not a test  
    *-------------------------------------------------*/ 
    %if &testYN ne Y %then
    %do;
        %if %nrbquote(&efile_key)=%nrbquote() %then %let newYN=Y;
        %else %let newYN=N; 
        %rc_efile_db_updt( form_config_key=&form_config_key,
                           form_type_code=&form_type_code,
                           efile_indsn=rc_efile,
                           efile_report_indsn=rc_efile_report,
                           newYN=&newYN
						  );
        %let efile_cnt=0;
        proc sql noprint;
             select put(count(*),best.) into :efile_cnt  
             from rc_efile;
        quit;
		
       %put;
       %put **********************************************************************;
       %put * NOTE: %trim(&efile_cnt) &form_type_code e-file(s) generated.        ;
       %put **********************************************************************;
       %put;      
      
  %end;

 

%MACRO_END:
   quit;
   %put;
   
  /*-----------------------------
   * Set end time for jobstats
   *-----------------------------*/

   %fcf_section_footer(Section=RC_EFILE_FINCEN_DRIVER);
  
   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;

 
%mend;

 
