/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_entity_info.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    This macro
|          1. gets the lastest transmitter information by forms with efile_key.
|          2. gets the default institution for 2A. 
|             If no default institution (DEFAULT_IND='Y') is found, used the 
|             latest one.
|          4. gets institutions for SAR 2B using WORK.SAR_2B.USER_FIELD. 
|             For SARs from the older version of AML which institution2bKey is not part of 
|             the report.UIJson, use 2A institution for 2B.  
|          
+------------------------------------------------------------------------------+
| USAGE:   report_form_config_key = form key 
|          form_type_code  = form type code (defined in FSK_RC_FORM_CONFIG)
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_efile_fincen_entity_info(form_config_key=, form_type_code=);

    proc sql;
         /* get all transmitters that are effective at e-file generation time.           */
        create table _tmp_fincen_1A as 
        select *
        from  rc_cfg.fincen_transmitter(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_transmitter))
        where form_config_key=&form_config_key and datetime() between effective_from_date and effective_to_date
        order by effective_from_date desc 
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

        /* pick the most recent transmitter and repeat it for each efile */
        create table fincen_1A as
        select seq.efile_key, 
               fincen.*
        from seq_batch as seq,
              _tmp_fincen_1A (obs=1) as fincen
        ; 
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;           

        /* get all institutions that are effective at e-file generation time.           */
        create table fincen_institution_list as
        select *
        from rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution))
        where form_config_key=&form_config_key and datetime() between effective_from_date and effective_to_date           
        order by default_ind desc, effective_from_date desc
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

        /* pick the default one or the most recent one for 2A */
        /* And repeat it for each efile                       */
        create table fincen_2A as
        select seq.efile_key, 
               fincen.*
        from seq_batch as seq,
              fincen_institution_list (obs=1) as fincen
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end; 
        quit;
  

    %if &form_type_code=SAR %then
    %do;
         
         /* get the default institution */
         %let default_institution_key=;
         data _null_;
             set fincen_2A;
             call symput('default_institution_key',strip(put(institution_key,best.)));
             stop; 
         run;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;    


         proc sql;
              /* use the default institution for 2B if it is not selected by the user */
              /* Note: user_field is used to store the institution2bKey               */
              update SAR_2B 
              set user_field="&default_institution_key"
              where user_field = ''
              ;
              %fcf_rcset(&sqlrc);
              %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %goto MACRO_END;
              %end; 

             /* get institution info for each report by institution_key */ 
             create table fincen_SAR_2B as 
             select mtr.efile_key, 
                    mtr.report_key, 
                    inst.* 
             from SAR_2B as mtr , 
                  rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution)) as inst
             where input(mtr.user_field,best.)=inst.institution_key                    
             order by mtr.efile_key, mtr.report_key 
            ;
            %fcf_rcset(&sqlrc);
            %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
            %end; 
         quit;
    %end;

 %MACRO_END:  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    
  

%mend;


