/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_gen_rawxml.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    This macro generates report data in xml format.
|          The xml will be used as input to PROC XSL to
|          transform the xml into the e-filing ready tags. 
| NOTE:    The macro is called by rc_efile_fincen_generate_x.sas.
|          All data used in this macro is created by rc_efile_fincen_detail_x or
|          rc_efile_fincen_sum_x.
|          
+-----------------------------------------------------------------------------------------+
| PARAMETERS:   
|          form_config_key = form key
|          form_type_code  = form type (e.g. CTRX)
|          efile_key       = efile key if it is a regeneration of existing efile
|          in_dsn          = input data set containing report data 
|          outfref         = file reference for the output xml
+------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_efile_fincen_gen_rawxml(form_config_key=,
                                  form_type_code=,
                                  efile_key=,
                                  in_dsn=rc_report_node_more_x,
                                  outfref= 
                               );

     proc sql noprint;

    /*-----------------------------
     * sum up dollar amount 
     *-----------------------------*/ 

    %let totalAmount=0;
        select strip(put(sum(input(node_value,best.)),best.)) into :totalAmount
        from &in_dsn(%fcf_get_ds_opts(ds_name=&in_dsn))
        where efile_key=&efile_key
              and entity_level_code='REPORT' 
              and node_name in ('usCashInAmount' 'usCashOutAmount' 'foreignCashInAmount' 'foreignCashOutAmount')
         ;
    %if &rc_debug=Y %then %put totalAmount=&totalAmount;

    /*-----------------------------
     * Get filing date
     *-----------------------------*/

    %let FilingDate=%sysfunc(today(),yymmddn8.);
    %if &rc_debug=Y %then %put FilingDate=&FilingDate;


    /*----------------------------------------
     * Count number of activities (reports)
     *---------------------------------------*/

    %let activityCount=0;
    proc sql noprint;
         select strip(put(count(distinct report_key),best.)) into :activityCount 
         from &in_dsn(%fcf_get_ds_opts(ds_name=&in_dsn))
         where efile_key=&efile_key 
               and entity_level_code='REPORT' 
         ;
    %if &rc_debug=Y %then %put activityCount=&activityCount;

    /*----------------------------------------------------------------------
     * count number of parties including transmitter, transmitter contact,
     * filing institution, filing institution contact, branches, and subjects
     *-----------------------------------------------------------------------*/

    %let PartyCount=0;
        select strip(put(count(*)+&activityCount * 4,best.)) into :PartyCount 
        from seg_kc.fsk_rc_report_entity 
              where report_key in (select distinct report_key from &in_dsn(%fcf_get_ds_opts(ds_name=&in_dsn))
                                   where efile_key=&efile_key  
                                   )
					and logical_delete_ind='N'
         ;
    quit;

    %if &rc_debug=Y %then %put partyCount=&PartyCount;

    %if %str(&outfref) = %str() %then
    %do; /* if outfref is not provided, create the xml in saswork */
         filename _rawxml "%sysfunc(pathname(work))/%lowcase(&form_type_code).Raw._&efile_key..xml";
         %let outfref=_rawxml;
    %end;

    data _null_;
     
         file &outfref;       
         set &in_dsn(%fcf_get_ds_opts(ds_name=&in_dsn))
             report_branch end=eof; /* include branch entity number */ 
         by report_key source_entity_level_code entity_key form_efile_record_id node_path;
         where efile_key=&efile_key;
         length stmt $1024;
         length transformed_node_value  $ 2000;
         if _n_=1 then link INIT;
         if first.report_key then link REPORT_INIT;
         if first.entity_key and entity_key ne report_key  then link ENTITY_INIT;      
         if first.node_path then link NODE_PATH_INIT;
         link node;
         if last.node_path then link NODE_PATH_TERM;     
         if last.entity_key and entity_key ne report_key then link ENTITY_TERM;
         if last.report_key then link REPORT_TERM;
         if eof then link TERM;
         
         return;

    INIT:
        stmt= '<?xml version="1.0" ?>';
        put stmt;
        stmt='<collection>';
        put stmt;
        stmt='<items>';
        put stmt;
        link GLOBAL;
        link TRANSMITTER; 
        link FILER;
    return;

    REPORT_INIT: 
       /* Initilization for each report by group */
        stmt='<report>';
        put stmt;
        stmt='<report_key>'||strip(put(report_key,best.))||'</report_key>';
        put stmt;
    return;

    ENTITY_INIT:
     /* begin of each report entity by group */
        stmt= '<'||strip(source_entity_level_code)||'>';
        put stmt;
        stmt='<entity_key>'||strip(put(entity_key,best.))||'</entity_key>';
        put stmt;
        
        if source_entity_level_code='BRANCH' then link BRANCH;
    return;

    NODE_PATH_INIT:
    /* begin of each report + entity + node path by group */
        if node_path ne '' then
        do;
            stmt= '<'||strip(form_efile_record_id)||'>';
            put stmt;
        end;
         
    return;


    NODE:
     /* write tag for each node name/value pair    */ 
     /* adjust node value based on node type       */

        if node_name='filingTypeCode' then 
        do; /* filing type is special,                              */
            /* creating 'Y' boolean tag with node name + node value */      
           stmt= '<'||strip(node_name)||upcase(strip(node_value))||'>Y</'
                 ||strip(node_name)||upcase(strip(node_value))||'>';
           put stmt;
        end;
        else
        do;
            select (upcase(source_field_type));
            when ('DATE') transformed_node_value=compress(scan(node_value,1,'T'),'-');
            when ('BOOLEAN') transformed_node_value=ifc(node_value='true','Y','');
            when ('ARRYCAT') transformed_node_value=ifc(node_value='true','Y','');
            when ('CODE') /* convert code value if a sas format with the same name exists */
            do;
                if cexist('work.formats.'||strip(node_name)||'.formatc') then
                   transformed_node_value=strip(putc(node_value,node_name)); 
                else 
                   transformed_node_value=strip(node_value);  
            end;
            otherwise           
                transformed_node_value=strip(node_value); 
            end;

            if transformed_node_value > '' then
            do;
                stmt= '<'||strip(node_name)||'>'||strip(transformed_node_value)||'</'||strip(node_name)||'>';
                put stmt;
            end;

        end;
        
    return;

    NODE_PATH_TERM:
    /* end of each report + entity + node_path by group */ 
        if node_path ne '' then
        do;
            stmt= '</'||strip(form_efile_record_id)||'>';
            put stmt;
        end;
    return;

     
    ENTITY_TERM:
    /* end of each report + entity by group */
        stmt= '</'||strip(source_entity_level_code)||'>';
        put stmt;
    return;

    REPORT_TERM:
    /* end of each report by group */
        stmt= '</report>';
        put stmt;
    return;

    TERM:
    /* end of all nodes  */
        stmt='</items>';
        put stmt;
        stmt='</collection>';
        put stmt;
    return;

    GLOBAL:
    /* tags global to all reports */
        put '<global>';
        
        put "<FilingDate>%trim(&FilingDate)</FilingDate>";    
        put "<TotalAmount>%trim(&totalAmount)</TotalAmount>";
        put "<ActivityCount>%trim(&activityCount)</ActivityCount>";
        put "<PartyCount>%trim(&partyCount)</PartyCount>";        
        
        put '</global>';
    return;

        
    TRANSMITTER:
    /* write transmitter tags */
    /* transmitter is created in rc_efile_fincen_sum_x */
       set transmitter (obs=1);
        put '<transmitter>';
        %&form_type_code._transmitter;
        put '</transmitter>';
    return;

    FILER:
    /* write filer tags */
    /* filer is created in rc_efile_fincen_sum_x */
        put '<filer>';
        set filer (obs=1);
        %&form_type_code._filer;  
        put '</filer>';
    return;

    BRANCH:
    /* extract branch data from branch_list */
    /* branch_list is created in rc_efile_fincen_sum_x */
        set branch_list key=entity_key/UNIQUE;
        if  _iorc_ = 0 then 
        do;
           link WRT_BRANCH;
        end;
        else 
        do;
           _error_=0;
           stmt='Branch '||strip(put(entity_key,best.))||' cannot be found';
           putlog stmt;
        end;
         
    return;

    WRT_BRANCH:
    /* write branch tags */

       %&form_type_code._branch;

    return;

    run;

           
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: RC: Failed to create &form_type_code.Raw.xml;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 



%MACRO_END:
  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    

%mend;
 
