/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_sdtm.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Driver program for generating a FINCEN e-file for SDTM           
|         (secure direct transfer mode)   
| SAR efile: BSARST.<yyyymmddhhmmss>.<organizationname>                           
| CTR efile: BCTRST.<yyyymmddhhmmss>.<organizationname> 
| CTRX efile: CTRXST.<yyyymmddhhmmss>.<organizationname>.xml 
| SARX efile: SARXST.<yyyymmddhhmmss>.<organizationname>.xml 
| SAR attachment zip: : BSARST.<yyyymmddhhmmss>.zip                               
| The final zip file:  BSARST.<yyyymmddhhmmss>.<organizationname>.zip. for sar 
| SARX attachment zip: : SARXST.<yyyymmddhhmmss>.zip                               
| The final zip file:  SARXST.<yyyymmddhhmmss>.<organizationname>.zip. for sarx   
+-----------------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key
|          form_type_code  = form type (e.g. SAR, CTR, SARX, ...)
|          efile_indsn    =  input table with efile info  
+------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2009-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_efile_fincen_sdtm(   efile_indsn=rc_efile,
                               form_config_key=,
                               form_type_code=
                            );
 
    
    %let trans_rc=0;
    %let _fcfmsg_=;

 
    %let zip_cnt=0;
    %let efile_file_prefix=;
    proc sql noprint;

         select trim(efile_file_prefix)                
                into :efile_file_prefix  
           from seg_kc.fsk_rc_form_config
           where form_config_key=&form_config_key;
           %let efile_file_prefix=%trim(&efile_file_prefix);
           %if %nrbquote(&efile_file_prefix)=%nrbquote() %then %do;
               %let _fcfmsg_=Note: RC: efile prefix is not defined for &form_type_code form in SEG_KC.FSK_RC_FORM_CONFIG.;
               %fcf_put_error;
               %fcf_rcset(3);
               %goto MACRO_END;
           %end;
        

         create table old_sdtm_efile  as
              select distinct efile_key, strip(direct_transfer_efile_name) as old_sdtm_efile_name 
              from seg_kc.fsk_rc_efile 
              where efile_key in (select distinct efile_key from &efile_indsn(%fcf_get_ds_opts(ds_name=&efile_indsn)))
              order by efile_key
              ;      

         %fcf_rcset(&sqlrc);                       
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: failed to old_sdtm_efile;
             %fcf_put_error;
             %goto MACRO_END;
         %end;
     
         select distinct put(efile_key,best.)
                into :zip_efile_key_1 -                 
         from &efile_indsn(%fcf_get_ds_opts(ds_name=&efile_indsn))
         where output_zip_name ne ''
        ;
         %let zip_cnt=&sqlobs;

         %fcf_rcset(&sqlrc);                       
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: failed to set up list of efile keys for winzip;
             %fcf_put_error;
             %goto MACRO_END;
         %end;

    quit;        
     

    %let rc_rc=0;
    proc sql;
    %do izip=1 %to &zip_cnt;
    
        %rc_mkdir(pathname=%sysfunc(pathname(work))&slash&form_type_code._&&&zip_efile_key_&izip,rcvar=rc_rc,resetYN=Y);
        %if %str(&rc_rc) ne 1 %then %do;
            %let _fcfmsg_=ERROR: RC: "%sysfunc(pathname(work))&slash&form_type_code._&&&zip_efile_key_&izip" cannot be created.;
            %fcf_put_error;
            %fcf_rcset(3);
            %goto MACRO_END;
        %end;
     
    %end;

          
    


   /* Retrieve  the generated efile names and locations */
   /*  and define the SDTM file names and locations.    */
 
    filename zippgm temp;

    data new_sdtm_efile (keep=efile_key direct_transfer_efile_name) ;
         length sdtm_dttm 
                $ 14;
         length rc_output_fullpath 
                sdtm_output_fullpath
                sdtm_dat_name 
                sdtm_zip_name
                sdtm_final_zip_name
                tmp_zip_fullpath
                sdtm_transmission_id
                
                $ 1024
                ;
         merge &efile_indsn (%fcf_get_ds_opts(ds_name=&efile_indsn)
                             keep=efile_key output_create_date lstupdate_date
                                  output_path_name output_name output_zip_name
                                  direct_transfer_efile_name
                                  )
               old_sdtm_efile
               end=eof;                      
         by efile_key;
         
         file zippgm;

         sdtm_dttm=compress(put(lstupdate_date,B8601DT15.),'T'); 
         %if &form_type_code=SARX or &form_type_code=CTRX or &form_type_code=DOEPX %then
         %do;
            /* do not add leading B for xml format forms */
            sdtm_prefix=strip("&form_type_code")||'ST';
         %end;
         %else
         %do;
             sdtm_prefix='B'||strip("&form_type_code")||'ST';
         %end;
         organization_name="&&&rc_sdtm_suffix_&form_type_code";
         sdtm_output_fullpath="&&&rc_sdtm_efile_path_&form_type_code";
      

         rc_output_fullpath=strip("&rc_efile_path")||"&slash"||strip(output_path_name);
         

         sdtm_transmission_id=strip(sdtm_prefix)
                               ||'.'
                               ||sdtm_dttm
                               ||'_'
                               ||strip(put(efile_key,best.))
                               ||'.'
                               ||strip(organization_name);

         %if &form_type_code=SARX or &form_type_code=CTRX or &form_type_code=DOEPX %then
         %do;
            /* add .xml for xml format forms */
            sdtm_transmission_id = strip(sdtm_transmission_id)||'.xml'; 
         %end;
         sdtm_dat_name=strip(sdtm_transmission_id);

          link DELETE_OLD;         
                        
        
         if output_zip_name ne '' then 
         do;
            

            /* attachment zip does not have the organization name to distinguish it from the final .zip */
            sdtm_zip_name=strip(sdtm_prefix)
                        ||'.'
                        ||sdtm_dttm
                        ||'_'
                        ||strip(put(efile_key,best.))
                        ||'.zip'; 
            sdtm_final_zip_name=strip(sdtm_transmission_id)
                        ||'.zip';
                        
            tmp_zip_fullpath=strip(pathname('WORK'))
                             ||"&slash"
                             ||"&form_type_code"||'_'
                             ||strip(put(efile_key,best.))
                         ;
            direct_transfer_efile_name=sdtm_final_zip_name;
            link COPY2WORK;
            link WRTZIPCMD;  
         end;
         else 
         do;
            direct_transfer_efile_name=sdtm_dat_name;
            link COPY2SDTM;
         end;
         
         if eof then link TERM;
         return;

    DELETE_OLD:
          
         if  old_sdtm_efile_name ne ' '  then 
         do;         
          fref='OLDFILE';
          rc=filename(fref, strip(sdtm_output_fullpath)||"&slash"||strip(old_sdtm_efile_name));
          if rc = 0 and fexist(fref) then
          rc=fdelete(fref);
          rc=filename(fref);
         end;

    return;     

    COPY2SDTM:
        rc=filename('src',strip(rc_output_fullpath)||"&slash"||strip(output_name),,'TERMSTR=CRLF LRECL=60000000');
        rc=filename('target',strip(sdtm_output_fullpath)||"&slash"||strip(sdtm_dat_name),,'TERMSTR=CRLF LRECL=60000000');
        rc=fcopy('src','target');
        rc=filename('src');
        rc=filename('target');
    return;

    COPY2WORK:
        rc=filename('src',strip(rc_output_fullpath)||"&slash"||strip(output_name),,'TERMSTR=CRLF LRECL=60000000');
        rc=filename('target',strip(tmp_zip_fullpath)||"&slash"||strip(sdtm_dat_name),,'TERMSTR=CRLF LRECL=60000000');
        rc=fcopy('src','target');
        rc=filename('src',strip(rc_output_fullpath)||"&slash"||strip(output_zip_name),,'recfm=n LRECL=60000000');
        rc=filename('target',strip(tmp_zip_fullpath)||"&slash"||strip(sdtm_zip_name),,'recfm=n LRECL=60000000');
        rc=fcopy('src','target');
        
    return;

    WRTZIPCMD:
        stmt='filename outzip "'||strip(sdtm_output_fullpath)||"&slash"||strip(sdtm_final_zip_name)||'";';
        put stmt;
        stmt='%let zippath='||strip(tmp_zip_fullpath)||';';
        put stmt;
        stmt='%rc_zip(zippath=%superq(zippath),outfref=outzip,rcvar=rc_rc);';
        put stmt;
        stmt='filename outzip clear;';
        put stmt;
    return;

    TERM:    
        rc=filename('src');
        rc=filename('target');
    
    return;
         
    run;
           
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end; 

    %inc zippgm; 
            
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end; 

    data &efile_indsn;
             modify &efile_indsn(%fcf_get_ds_opts(ds_name=&efile_indsn))
                    new_sdtm_efile (keep=efile_key direct_transfer_efile_name);
             by efile_key;
    run;  
            
    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end; 
               


%MACRO_END:
  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    
  


%mend;

