/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_fincen_sum_x.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Program to convert code values from the old format to new format.
|          It also extract transmitter, filer, and branch information based
|          on current datetime and ENTITY_KEY in FSK_RC_REPORT_ENTITY
|            
|          This program is called by rc_efile_fincen_data_x. 
|
| Output tables: 
|         transmitter - current transmitter info
|         filer - current filing institution info
|         report_branch - branch node with only entity_number, one for each
|                         branch in a report
|         branch_list - list of branches that will be included in e-file with detailed branch information
|          
+------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key
|          form_type_code = form type code (defined in FSK_RC_FORM_CONFIG)
|          report_list = report key list of reports to be processed 
|                        The keys are separated by ' ' 
|          in_dsn = input table with report node names and values
|          seq_dsn = input table with list of reports with assigned e-file keys
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_efile_fincen_sum_x(
                            form_config_key=,
                            form_type_code=,
                            in_dsn=,
                            efile_key=,
                            seq_dsn=
                           );
 
    %let trans_rc=0;
    %let _fcfmsg_=;

    %if &efile_key= %then %let efile_key=0;

    /* load code conversion data into sas formats */
    %rc_efile_get_conversion( form_type_code=&form_type_code);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;

      proc sql;
       
        create table transmitter as
        select distinct *
        from rc_cfg.fincen_transmitter(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_transmitter))
        where  form_config_key=&form_config_key
               and datetime() between effective_from_date and effective_to_date
        ;

        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: Failed to extract transmitter data;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        create table filer as
        select distinct *
        from rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution))
        where  form_config_key=&form_config_key
               and datetime() between effective_from_date and effective_to_date
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: Failed to extract filer data;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

        /*---------------------------------------------------------
         * Create branch nodes for all branches of reports in in_dsn
         * It is needed because in_dsn may not have any branch nodes 
         *---------------------------------------------------------*/
        
        create table report_branch as
        select distinct &efile_key as efile_key, 
                        report_key, 
                        entity_level_code,
                        entity_level_code as source_entity_level_code length=32, 
                        entity_key,
                        'branch' as form_efile_record_id length=32,
                        '' as node_path length=128,
                        'entity_number' as node_name length=32,
                        entity_number as node_value length=2000,
                        '' as source_field_type length=32 
        from   seg_kc.fsk_rc_report_entity
        where  entity_level_code='BRANCH' and
		       logical_delete_ind='N' and
               report_key in (select distinct report_key 
                             from  &in_dsn(%fcf_get_ds_opts(ds_name=&in_dsn))
                             )
        order by report_key, source_entity_level_code, 
                 entity_key, form_efile_record_id, node_path     
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: Failed to extract branch associated with reports;
            %fcf_put_error;
            %goto MACRO_END;
        %end;

         /* if efile_key is not assigned, get efile key from sequence table*/
        %if &efile_key = 0 %then
        %do;
             update report_branch as branch    
             set efile_key=(select efile_key
                  from &seq_dsn(%fcf_get_ds_opts(ds_name=&seq_dsn)) as seq 
                  where branch.report_key = seq.report_key 
                  );
             %fcf_rcset(&sqlrc);
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                 %let _fcfmsg_=ERROR: RC: Failed to update efile_key in report_branch;
                 %fcf_put_error;
                 %goto MACRO_END;
             %end;
        %end;
  

        /*---------------------------------------------------------
         * Create branch list with branch detail information
         * Only the branches associated reports are included. 
         *---------------------------------------------------------*/
        
        create table branch_list as
        select distinct branch.*, branch_key as entity_key       
        from   rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch)) as branch
        where branch_key in (select distinct entity_key 
                             from  report_branch)
        ;

        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: Failed to create branch_list;
            %fcf_put_error;
            %goto MACRO_END;
        %end;
        create unique index entity_key
        on branch_list(entity_key)
        ;
        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: Failed to create index for branch_list;
            %fcf_put_error;
            %goto MACRO_END;
        %end;
 
        

 
%MACRO_END:
  
    quit;
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    


%mend;
