/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_resp_fince_ctrx_read
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    read the CTRX acknowledgment receipt xml with xml map. 
| NOTE:          Output tables are:                                              
|               _1A: e-file level response                                       
|                _3A: rr (aka activity) level response                           
|               _ctrx_error_codes: error level response                         
|               The structures of the tables were originally designed based on  
|               the old .asc response files structures.                          
|               They will be used as input to ecmrr_resp_fincen_update           
+------------------------------------------------------------------------------+
| USAGE:    INFREF=fileref pointing to a CTRX response file.                  
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/



%macro rc_resp_fincen_ctrx_read ( infref= 
                                     );
      
    %let trans_rc=;
    %let _fcfmsg_=;
    /* Remove output from previous run */
    proc datasets lib=work nowarn nolist;
         delete 
                _1A
                _3A 
                _ctrx_error_codes
                efilingactivityxml
                efilingbatchxml
                efilingactivityerrorxml
                efilingsubmissionxml
    ;
    quit;

 
    /* load the input file to sas library with xml maper. */
    libname &infref xml92 xmlmap="&fcf_xml_map_path&slash.rc_fincen_ackd.map";

    /* Make the xml mapped data into sas tables before releasing the library*/
    proc copy in=&infref out=work;
    run;     
              
    %fcf_rcset(&syserr);
                        

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Cannot load xml data into sas table.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;
     
    %if %upcase(&rc_debug) = N %then
    %do;
        libname &infref clear;
    %end;
   
     proc sql;
          /* Read the efile level data to create _1A */
          /* In the xml output, coverage period is not relevant any more */
          create table _1A as
          select EfilingSubmissionXML_StatusCode as SUBMISSION_STATUS_CODE,
                 '' as RECORD_TYPE length=2,
                 '' as COVERAGE_BEGIN_DT length=8,
                 '' as COVERAGE_END_DT length=8 
          from EfilingSubmissionXml            
          ;
          
     %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Cannot create _1A.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;

     /* Read the activtiy level data to create _3A              */
     /* In the xml output, record_type is not relevant any more */
     /* SeqNum contains REPORT_KEY. Use it as _3A USER_FIELD    */        

     create table _3A as
     select EFilingActivityXML_ORDINAL as TRANS_SEQ_NUMBER,
            '' as RECORD_TYPE length=2,
            put(EFilingActivityXML_SeqNum,z10.) as USER_FIELD,
            BSAID as BSA_ID length=14  
     from EfilingActivityXml              
     ;
          
     %fcf_rcset(&sqlrc);

     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=ERROR: RC: Cannot create _3A.;
         %fcf_put_error;
         %goto MACRO_END;
     %end;

     /* Read error level data to create _CTRX_error_codes                         */
     /* In the xml output, record_type and line_number are not relevant any more      */
     /* ErrorContext contains xml path of the data value that trigger the error   */ 

     create table _ctrx_error_codes as
     select err.ErrorTypeCode as ERROR_CODE,
            0 as LINE_NUMBER, 
            '' as RECORD_TYPE length=2,
            err.Efilingactivityerrorxml_SeqNum as ERROR_SEQ_NUMBER,
            err.EFilingActivityXML_ORDINAL as TRANS_SEQ_NUMBER,
            act.user_field,
            act.bsa_id,
            err.ErrorContextText as ERROR_CONTEXT               
     from EfilingActivityErrorXml as err
     left join _3A as act
     on  err.EFilingActivityXML_ORDINAL = act.trans_seq_number 
     where error_code ne ''
     ;
          
    %fcf_rcset(&sqlrc);
  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: RC: Unable to create _ctrx_error_codes;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
 
%MACRO_END:    
  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
   

%mend;
