/* MAINDOC
+-------------------------------------------------------------------------------------------------------+
| NAME:    rc_rest_get_efile_collection.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Call efileCollection REST service to create an .xml file with information 
|          needed for validating or e-filing of one or many reports. 
+-------------------------------------------------------------------------------------------------------+
| USAGE:   report_list    = list of report keys for which efile collection xml will be generated
|                           the report keys should be comma separated. |                            
|          outfref        = sas file reference for the output .xml file 
|                           if this parameter is not provided, a temporary file will be created
|          outdsn         = name of output sas table containing the list of reports with the data fields
|                           defined in rc_efile_collection_<form_type_code>.map
|                           if this parameter is not provided, no dataset will be generated
|          form_type_code = form type code (defined in FSK_RC_FORM_CONFIG) 
+--------------------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
%macro rc_rest_get_efile_collection(report_list=,
                                   outfref=,
                                   outdsn=efile_collection_list,
                                   form_type_code=SARX
                                   );
 
    %let trans_rc=0;
    %let _fcfmsg_=;
	
	 %if %nrbquote(&report_list) = %nrbquote() %then %do;
        %let _fcfmsg_=ERROR: Parameter report_list is required.;
        %fcf_put_error;
        %fcf_rcset(5);
        %goto MACRO_END;
    %end;

    %if &outfref= %then 
    %do;
        filename srv_resp temp;
        %let outfref=srv_resp;
    %end;
      	
    %if %nrbquote(&fcf_service_RegulatoryReports)=%nrbquote() %then
    %do;         
        %let fcf_service_RegulatoryReports=;
        %fcf_get_rest_url(domainType=RegulatoryReports);

        %if %nrbquote(&fcf_service_RegulatoryReports)=%nrbquote() %then
        %do;
            %let _fcfmsg_=Error: RC: cannot retrieve report service url.;
            %fcf_put_error;     
            %goto MACRO_END;
        %end;
   %end;    
           
    
   filename tmp_resp temp;

           /* call regulatorReport REST service */
            
   proc http url= "&fcf_service_RegulatoryReports./efileCollection.xml?reportKeys=&report_list.%str(&)businessUnit=&segKCDBUser"
             out=tmp_resp
             method="get"
             ct="application/x-www-form-urlencoded" 
             HTTP_TOKENAUTH                       
   %if %str(&rc_webusername) ne %str() and
       %str(&rc_webpassword) ne %str() %then
   %do;                             
             webusername="&rc_webusername" webpassword="&rc_webpassword"  
   %end;    
   ;
   run;
             
   %fcf_rcset(&syserr);
           
   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %let _fcfmsg_=Error: RC: cannot call report REST service.;
       %fcf_put_error;     
       %goto MACRO_END;
   %end;                      
                        
   /* REST data can contain &lt; and &gt; characters, need to convert to actual < and > */
   data _null_;
        infile tmp_resp recfm=v lrecl=32767 truncover;
        file &outfref. recfm=v lrecl=32767;
        format new_line $32767. full_line $32767.;
        input @1 full_line $32767.;
		/* convert ONLY lines that are return via JSON and still have html encoding to convert. */
		if (index(full_line,'<fincenFiler>') > 0) or 
		   (index(full_line,'<fincenTransmitter>') > 0) or 
		   (index(full_line,'UiJSON>') > 0) then do;
           new_line=tranwrd(full_line,'&lt;','<');
           new_line=tranwrd(new_line,'&gt;','>');
		   end;
		else 
		   new_line = full_line;
        put new_line;
   run;

   %fcf_rcset(&syserr);                     

   %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
   %let _fcfmsg_=ERROR: RC: Problem translating REST output lt/gt tags to <> characters.;
        %fcf_put_error;
        %goto MACRO_END;
   %end;

   %if %str(&outdsn) ne %str() %then
   %do; 
        /*  rc_report_attachment.map extract all reportKeys, attachmentName, and attachmendId */
        /*  Attachment information is needed for SARX.                                        */
        libname tmp xmlv2 "%sysfunc(pathname(&outfref))" 
                xmltype=xmlmap xmlmap="&fcf_xml_map_path/rc_efile_collection_%lowcase(&form_type_code).map";
   
        data &outdsn(%fcf_get_ds_opts(ds_name=&outdsn, output_ds=Y));
             set tmp.report;
        run;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %let _fcfmsg_=ERROR: RC: failed to extract data from efileCollection;
             %fcf_put_error;
             %goto MACRO_END;
        %end;

        %if  %rc_nobs(&outdsn)=0 %then %do; 
             %let _fcfmsg_=ERROR: RC: no reports in efileCollection;
             %fcf_put_error;
             %fcf_rcset(5);
             %goto MACRO_END;
        %end;
    %end;
             


%MACRO_END:
      

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
        %fcf_execute_macro(fcf_error);
    %else 
        %fcf_execute_macro(fcf_success);
              
%mend;
 
