/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_validate_driver.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    validate report data and return an error report. 
|          
+------------------------------------------------------------------------------+
| USAGE: 
|          report_list     = report key list of reports to be validated 
|                            Keys are separated by ' ' 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro rc_validate_driver(report_list=
                         ,form_config_key=
						 ,form_type_code=
						 ,extractDataYN=Y);

%let trans_rc=0;
%let _fcfmsg_=;
/* Provide ability to skip the XML validation step, if desired. */
%let validateXML_YN=Y;	

%let form_type_code=%trim(&form_type_code);


%if &extractDataYN ne N %then
%do;
   
	 /* Initialize to blank before determining if will provided or not. */
     %let rest_report_file=;
	 
     /* SARX will assemble data in different manner than past reports. It will build an XML file to validate data */
	 /* and same file will be translated for e-file format and then validated against XSD schema.                 */
     %if &form_type_code.=SARX or &form_type_code.=DOEPX %then %do;
	     /* Create a macro variable &rest_report_file this can be passed to validation          */
		 /* XML section to reuse created file versus duplicating effort in that validation.     */
	     %let rest_report_file=%sysfunc(getoption(work))&slash.raw_%lowcase(&form_type_code)_efileCollection.xml;
		 filename restxml "&rest_report_file.";
		 /* Can set macro variable fcf_service_RegulatoryReports for hardcoded rest URL.  */
		 /* %let fcf_service_RegulatoryReports=http://<host>:<port>/SASComplianceSolutionsMid/rest/regulatoryReports; */
		 
		 %rc_rest_get_efile_collection(report_list=%superq(report_list),outfref=restxml);
		 
		 %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
		    %end;

	    %end;
	 %else %do;
        %rc_validate_fincen_data(report_list=&report_list
                               ,form_config_key=&form_config_key
                               ,form_type_code=&form_type_code                              
                             );   
	    %end;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END; 
    %end;
%end;
 
 data &form_type_code._error; 
      length error_record_type $ 32
             error_code        $ 4
             target_field_name $ 32
             report_key      8 
             entity_key      8
             entity_level_code $  32
			 error_field $ 100
         ;
         stop;
    run;
 
    proc sql noprint;
         select distinct trim(error_record_type) into
                :record_id_1 - 
         from rc_cfg.&form_type_code._fincen_error_code(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=&form_type_code._fincen_error_code))
         where error_record_type not in (' ', '1A', '2A' 'transmitter' 'filer')
         ;
         %let record_id_cnt=&sqlobs;
        
    quit;

     /* SARX will Validate report data using XSLT and XML representation of report file. */
     %if &form_type_code.=SARX or &form_type_code.=DOEPX %then %do;
		 %let vout_file=%sysfunc(getoption(work))&slash.%lowcase(&form_type_code)Validate.xml;
		 /* Call XSLT to validate data on Report XML file and massage output to create _T_ERROR  */
		 /* and append to <form_type_code>_error.                                                */
	     filename valxslt "&rc_template_path&slash.rc_%lowcase(&form_type_code)_validate.xsl";
		 filename rptxml  "&rest_report_file.";	 
         filename voutxml  "&vout_file.";
         proc xsl in=rptxml xsl=valxslt out=voutxml; 
         run;
 	     %fcf_rcset(&syserr);
         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: Problem applying PROC XSL to validate RAW XML report file;
             %fcf_put_error;
             %goto MACRO_END;
             %end;
	
	     /* Add one more test to look for the Temp xml file, if does not exist, make a note and stop validation */
	     %if %sysfunc(fexist(VOUTXML)) eq 0 %then %do;
	        %let _fcfmsg_=ERROR: Problem creating %lowcase(&form_type_code)Validate.xml with PROC XSL;
            %fcf_put_error;
            %goto MACRO_END;
	        %end;
			
		 /* Should have some output in outxml file to process to see if any errors. */
		 libname readerrs xml "&vout_file.";
		 %if %sysfunc(exist(readerrs.reportvalidation)) %then %do;
	         proc sql noprint;
		     select count(*) into :validate_errors
		         from sashelp.vtable where libname='READERRS' 
                          and memname='REPORTVALIDATION';
		     quit;
		     %end;
		 /* problem trying to establish XML libname to created validation file. */
		 %else %do;
		 	 %let _fcfmsg_=ERROR: Problem with accessing report validation with XML engine.;
             %fcf_put_error;
             %goto MACRO_END;
		     %end;
			 
		 /* Validation errors found, so extract to join with error codes. */	 
		 %if &validate_errors. > 0 %then %do;
		     data &form_type_code._error_xml;
			    if 0 then set &form_type_code._error;
			    set readerrs.reportvalidation (rename=(ERRORCODE=ERROR_CODE ENTITYNAME=ERROR_FIELD TARGETFIELDNAME=TARGET_FIELD_NAME
				                                       ITEMNUMBER=ITEM_NUMBER ENTITYKEY=ENTITY_KEY
				                                       ERRORRECORDTYPE=ENTITY_LEVEL_CODE));
			    format TARGET_FIELD_SHORT_DESC $100.;
                TARGET_FIELD_SHORT_DESC=cat(TARGET_FIELD_NAME,'(Item: ',ITEM_NUMBER,')');
				report_key=&report_list.;
				
			 run;

		     %end;
		 /* No errors found, so create the empty dataset. */
		 %else %do;
		     %put TRACE: All Good on validation.;
		     /* Create an empty dataset for the no error situation. */
		     data &form_type_code._error_xml;
		        set &form_type_code._error;
		        format TARGET_FIELD_SHORT_DESC $100.;
		     run;
			 %end;
		 libname readerrs clear;
	     %end; /* SARX create report file via REST and Validate data. */
		 
	 /* non-SARX reports will validate report data via SAS macros and datastep.          */
	 %else %do;
	     /* Include macro contain if/then logic of conditions to test and FinCEN error codes. */
         %inc rc_tmplt(rc_fincen_%lowcase(&form_type_code)_validation_code_macro.sas);
		 
         %do irecord=1 %to &record_id_cnt;     
    
            %if %sysfunc(exist(&form_type_code._&&&record_id_&irecord)) %then
               %do;
        
               data _t_error;
               if 0 then set &form_type_code._error;
               set &form_type_code._&&&record_id_&irecord;
               keep error_record_type error_code report_key entity_level_code entity_key target_field_name error_field ;
               retain error_record_type "&&&record_id_&irecord" error_field;
			   if trim(lowcase("&&&record_id_&irecord"))="party" then do;
			      if TaxID ne '' then 
				      error_field=trim("Tax ID: " || TaxID);
			      end;
			   /* Add these 4 lines for SAR 4A Subject record identifier. Will use TAX_ID.    */
			   /* Can change this to be Last Name for SAR.  Variable is LAST_NM in 4A table.  */
			   if trim(upcase("&&&record_id_&irecord"))="4A" then do;
                  if TAX_ID ne '' then
                      error_field=trim("Tax ID: " || TAX_ID);
                  end;
			   if trim(lowcase("&&&record_id_&irecord"))="branch" then do;
			      if branch_Name ne '' then 
				      error_field=trim("Name: " || branch_Name);
			      end;
			   if trim(lowcase("&&&record_id_&irecord"))="foreign_cash_in" then do;
			      if foreignCashInCountryCode ne '' then 
			         error_field=trim("Country Code: " || foreignCashInCountryCode);
			      end;
			   if trim(lowcase("&&&record_id_&irecord"))="foreign_cash_out" then do;
			      if foreignCashOutCountryCode ne '' then 
			         error_field=trim("Country Code: " || foreignCashOutCountryCode);
			      end;
			   /* Execute the block of code from the included file. */
               %vldt_&form_type_code._&&&record_id_&irecord;		   
               run;
             
               %fcf_rcset(&syserr);                     

               %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                   %let _fcfmsg_=ERROR: RC: Unable to validate WORK.&form_type_code._&&&record_id_&irecord;
                   %fcf_put_error;
                   %goto MACRO_END;
                   %end;       
 
               proc append base=&form_type_code._error data=_t_error (where=(error_code > '')) force;
               run;

               
               %fcf_rcset(&syserr);

               %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=ERROR: RC: Unable to append error table of WORK.&form_type_code._&&&record_id_&irecord to master error table;
                  %fcf_put_error;
                  %goto MACRO_END;
                  %end;        

               /* remove the temp table for the next cycle */
               proc sql noprint;
                  drop table _t_error;
              quit;
              %end;
          %end;
        /* Cross validation execution from the included report type file. */
        %vldt_&form_type_code;
		%end; /* block for the non-SARX reports */


        /***********************************************************************/
        /* Obtain error description and types.                                 */
        /***********************************************************************/
		
		/* New XML forms will pull some information from the XSLT validation   */
		/* instead of some of the lookup tables used in past forms.            */
        %if &form_type_code.=SARX or &form_type_code.=DOEPX %then %do;
           %let error_cnt=0;
           proc sql;
           create table &form_type_code._error_more as 
           select   a.report_key  
                  , a.entity_level_code
                  , a.entity_key 
                  , a.error_field
                  , a.target_field_short_desc
                  , b.*
                  
           from &form_type_code._error_xml as a
           left join  rc_cfg.&form_type_code._fincen_error_code(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=&form_type_code._fincen_error_code)) as b
           on a.error_code=b.error_code
              and a.entity_level_code=b.error_record_type
           
           order by report_key, error_record_type, entity_key, error_code
           ;
           %let error_cnt=&sqlobs;
           quit;  
		   %end;
		%else %do;
           %let error_cnt=0;
           proc sql;
           create table &form_type_code._error_more as 
           select   a.report_key  
                  , a.entity_level_code
                  , a.entity_key 
                  , a.error_field
                  , b.*
                  , c.target_field_short_desc
           from &form_type_code._error as a
           left join  rc_cfg.&form_type_code._fincen_error_code(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=&form_type_code._fincen_error_code)) as b
           on a.error_code=b.error_code
              and a.error_record_type=b.error_record_type
           left join rc_cfg.form_efile_field_config(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=form_efile_field_config)) as c
           on a.error_record_type=c.form_efile_record_id           
              and upcase(a.target_field_name)=upcase(c.target_field_name)
              and c.form_config_key=&form_config_key
           order by report_key, error_record_type, entity_key, error_code
           ;
           %let error_cnt=&sqlobs;
           quit;  
        %end;

         %fcf_rcset(&sqlrc);

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to create table &form_type_code._error_more;
             %fcf_put_error;
             %goto MACRO_END;
         %end;

         %if &error_cnt > 0 %then
            %do;
			/* Produce error reports */        
            proc report data=&form_type_code._error_more nowindows center ;
               column report_key entity_key error_record_type error_field error_code error_type_code  error_desc target_field_short_desc;
               define report_key /"Report Key";
               define entity_key /"Entity Key";
               define error_record_type /width=20 flow "Record Identity";
               define error_field / "Record Identifier";			 
               define error_code /width=5 "Error Code" ;
               define error_type_code /width=10 format=$fincen_error_type. "Error Type"  ;
               define error_desc /width=30 flow "Error Description" ;  
               define target_field_short_desc /width=30 flow "Error Field" ;  
               title "&form_type_code Data Validation Error Report";      
            run;
			%end;
          %else
          %do;
            options ls=72;
			ods html close; 
            data _null_;
                 file _webout;
                 put "No Data Validation errors found for report: %trim(&report_list).";
            run;
			ods html file=_webout;
          %end;

         %fcf_rcset(&syserr);

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
             %let _fcfmsg_=ERROR: RC: Unable to create error report for &form_type_code._error_more;
             %fcf_put_error;
             %goto MACRO_END;
         %end; 

        /* Call the generation of an XML file in temp location and validate the XML file against XSD for e-file errors. */
		/* XML file will be generated and validated in the driver macro.                                                */
		/* Also, Only call the XML validation for form_type_code that is an XML submitted type.                         */
		%let reportTypeXML_YN=N;
		data _null_;
		   format key_value $8.;
		   /* make sure to use the form_type_code in case sites use different form_config_key values. */
		   key_value=trim("&form_type_code.");
		   if key_value in ("CTRX","SARX","DOEPX") then 
		      call symput('reportTypeXML_YN','Y');
		run;
		%if &validateXML_YN eq Y and &reportTypeXML_YN eq Y %then
            %do;
            %rc_validate_xml_driver(efile_key=-1
		                          , report_key=&report_list.
								  , report_file_xml=&rest_report_file.
							      , form_type_code=&form_type_code.
							      , form_config_key=&form_config_key.
							      , efile_path=WORK
							      , error_file=WORK.XML_ERRORS
							      );
							  
		    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %goto MACRO_END;
                %end;
						
			/* Display the results of the XML Validation which should be in the error_file= dataset. */	
			%let validation_errors=0;
			proc sql noprint;
			    select nlobs into :validation_errors 
				    from SASHELP.VTABLE
					where libname="WORK" and memname='XML_ERRORS';
				quit;
			%if &validation_errors. > 0 %then
                %do;
                /* Produce XML Validation Error Report */  
                proc report data=work.xml_errors nowindows center ;
                    column error_line_number error_column_number error_message;
					define error_line_number / "Line";
					define error_column_number / "Column";
                    define error_message /width=80 flow "Error Description";
                    title "&form_type_code XML Validation Error Report";      
                run;
				%end;
            %else %do;
			    /* Let them know the XML validation was successful and no errors. */
                options ls=72;
				ods html close;
                data _null_;
                   file _webout;
                put "<p>No XML Validation errors found for report: %trim(&report_list).";
                run;
				ods html file=_webout;
               %end;
			%end; 

%MACRO_END:    

  
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
   

%mend;
 
