/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_validate_fincen_data.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    Transform the report data for validation (detail record level)
+------------------------------------------------------------------------------+
| USAGE:   form_config_key = form key
|          form_type_code  = form type code (defined in FSK_RC_FORM_CONFIG)
|          report_list     = report key list of reports to be processed 
|                            Keys are separated by ' ' 
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

  

%macro rc_validate_fincen_data( report_list=,
                                  form_config_key=, 
                                  form_type_code=                              
                             );                                        

 
   
    %let trans_rc=0;   
    %let _fcfmsg_=;

	/* Conditional report type will be done by form_type_code instead of form_config_key */
    %if &form_type_code = SAR or &form_type_code=CTR or &form_type_code=DOEP  %then
    %do;
       /* for the old FinCEN forms, create data as in e-file */
       %rc_efile_fincen_data(report_list=&report_list,
                             form_config_key=&form_config_key,
                             form_type_code=&form_type_code
                            );
       %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
       %end;

    %end;
    %else
    %do;
        /* For the new FinCEN forms with e-files in XML format, */
        /* just transpose the data fields for validation        */
        /* The final e-files will be validated against the      */
        /* xml schema.                                          */
        %let record_id_cnt=0;
        proc sql noprint;
             select distinct trim(form_efile_record_id) into
                    :record_id_1 - 
             from rc_cfg.form_efile_record_config(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=form_efile_record_config))
             where form_config_key=&form_config_key
             ;
             %let record_id_cnt=&sqlobs;
            
        quit;
        
         /* remove existing tables */
         proc datasets lib=work nolist nowarn;
             
         %do irecord=1 %to &record_id_cnt; 
             %if %sysfunc(exist(&form_type_code._&&&record_id_&irecord)) %then
            %do;
                delete &form_type_code._&&&record_id_&irecord;
            %end; 
         %end;
         run;
         quit;
        /*---------------------------------------------
         * Transpose report node data into records
         *---------------------------------------------*/

       %rc_efile_transpose_data(report_list=&report_list,
                                form_config_key=&form_config_key, 
                                form_type_code=&form_type_code);

      
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

        /*---------------------------------------------
         * Assign Prior Agency Reference ID 
         *---------------------------------------------*/
        proc sql;
             
              update &form_type_code._report as rec_report
              set priorAgencyRefId=(select prior_agency_ref_id    
                                       from seg_kc.fsk_rc_report rpt
                                       where rec_report.report_key=rpt.report_key)
              where filingTypeCode ne 'A'
              ;
        quit;

        %fcf_rcset(&sqlrc);

         %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
              %let _fcfmsg_=ERROR: RC: Unable to update PRIOR_AGENCY_REF_ID in WORK.&form_type_code._REPORT;
               %fcf_put_error;
               %goto MACRO_END;
         %end;

        /* create a empty party table if no party exists in the report */
       %if %sysfunc(exist(&form_type_code._party))=0 and %sysfunc(exist(rc_db.&form_type_code._party)) %then 
            %do;
             data &form_type_code._party;
                if 0 then set rc_db.&form_type_code._party(%fcf_get_ds_opts(lib_name=rc_db, ds_name=&form_type_code._party));
                stop; 
             run;
           %end;
        
        /* For Branch data validation, will need more details than the transpose data provided. */
		%if %sysfunc(exist(&form_type_code._branch))=1 %then
		    %do;
			proc sql noprint;
			   create table &form_type_code._branch_details as
               select distinct branch.*, branch_key as entity_key       
                  from rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch)) as branch
                     where branch_key in 
					     (select distinct entity_key from &form_type_code._branch)
				  order by branch_key
            ;
			create unique index entity_key
                on &form_type_code._branch_details(entity_key);
			create unique index entity_key
                on &form_type_code._branch(entity_key);
           quit;
		   
		   %fcf_rcset(&sqlrc);
           %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: RC: Unable to create Branch Details dataset: WORK.&form_type_code._branch_details;
                %fcf_put_error;
                %goto MACRO_END;
           %end;
		   
		   /* update the missing values with details from direct query to branch information */
		   data &form_type_code._branch;
		      modify &form_type_code._branch &form_type_code._branch_details;
			  by entity_key;
			  if _iorc_=0 then replace;
		   run;
		   
		   %fcf_rcset(&syserr);
           %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                %let _fcfmsg_=ERROR: Problem updating &form_type_code._branch with information from WORK.&form_type_code._branch_details;
                %fcf_put_error;
                %goto MACRO_END;
           %end;
		   
		   /* After done with updating of information, delete the transaction table */
		   %if %sysfunc(exist(&form_type_code._branch_details))=1 %then 
		       %do;
			   proc sql noprint;
                  drop table &form_type_code._branch_details;
               quit;    
			   %end;
		   %end;
		 
		   
    %end;

%MACRO_END:
  

    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
       %fcf_execute_macro(fcf_error);
    %else 
       %fcf_execute_macro(fcf_success);
    


%mend;
 
