/*----------------------------------------------------------------------------------------
 *    NAME: rc_validate_xml.sas
 *   TITLE: XML Report Validate against FinCEN provided XSD schema files.
 * PRODUCT: SAS Currency Transaction Reporting
 *  SYSTEM: UNIX/Windows
 *---------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *---------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------
 *    This is a sample macro to validate a generated XML report against the FinCEN
 *    provided XSD Schema file to minimize the number of e-file rejections due to
 *    invalid XML files.
 *
 *    Requirement:
 *        A version of Java 7 or 8 JRE is required to execute the supplied JAR file and to
 *        resolve the path to FinCEN provided repositories. SAS supplied Java 1.7 will be the default.
 *
 *    Inputs:
 *        xml_file          = full path location of XML file to validate
 *        xsd_file          = full path location of XSD to validate against
 *        error_file        = work dataset with validation errors
 *        2 parameters with defaults defined in macro parameter definitions
 *        --------------------------------------------------------
 *        java_loc          = fully qualified path for JAVA 7/8 executable
 *                            On Windows, the filename for java executable will most likely be java.exe 
 *        jar_loc           = fully qualified path for the XML validation jar file
 *
 *    Output:
 *        SYSRC from X command will contain 0 if validation is clean; 1 if error found
 *           and error details will be found in the validation_details dataset. 
 *        And the XML source file will be saved in xmlsource dataset that can be displayed.
 *---------------------------------------------------------------------------------------*/

%macro rc_validate_xml(xml_file=
                        , xsd_file=&rc_template_path.&slash.schema&slash.FINCEN&slash.ctrx_schema.xsd
                        , java_loc=
                        , jar_loc=&rc_template_path.&slash.xmlvalidator.jar
						, error_file=work.errors
                        );

   
  /*---------------------------------------------------------------------------------------
   *   For now, will set some site specific defaults to find Java and location of Jar file
   *--------------------------------------------------------------------------------------*/
   %let successYN=Y;
   %let validationRC=0;
   %let valoptionsRC=0;
   %let validation_output=val_out.txt;
   %let win_cmd=val_cmd.bat;
   
   %let trans_rc=0;
   %let _fcfmsg_=;

   /* if java_loc is NOT passed (blank), build the path from SAS default environment variable */
   data _null_;
       format homeval setpath $256.;
       homeval=trim(sysget('JAVA_HOME'));
       passed_val=trim(symget('JAVA_LOC'));
       if passed_val = '' then do;
	      /* If this is Windows, the java executable will have .exe extension */
	      if index(trim("&SYSSCP."),"WIN") then 
		     setpath = cat(trim(homeval),"&slash.","bin","&slash.","java.exe");
		  /* UNIX, the binary will just be the filename without an extension */
		  else 
             setpath = cat(trim(homeval),"&slash.","bin","&slash.","java");
		  end;
       else
          setpath = trim(passed_val);
       call symput('JAVA_LOC',trim(setpath));
   run;

   /* Check for the XML and XSD file existence before proceeding.  */
   /* If not found, then going ahead and stop before attempting.   */
   %if %sysfunc(fileexist("&xml_file.")) eq 0 %then
      %let valoptionsRC=1;
   %if %sysfunc(fileexist("&xsd_file.")) eq 0 %then
      %let valoptionsRC=2;
   %if %sysfunc(fileexist("&java_loc.")) eq 0 %then
      %let valoptionsRC=3;
   %if %sysfunc(fileexist("&jar_loc.")) eq 0 %then
      %let valoptionsRC=4;

   /* Worst case scenario on not finding options, failed to pass back up the error flag */
   %if &valoptionsRC > 0 %then
      %let successYN=N;

   /* Create the validation output filename value in WORK location. */
   data _null_;
      format val_file workdir $256.;
      workdir=trim(getoption('WORK'));
      val_file=trim(workdir) || "&slash." || trim("&validation_output.");
      win_file=trim(workdir) || "&slash." || trim("&win_cmd.");
      call symput('VALFILE',trim(val_file));
      call symput('BATFILE',trim(win_file));
   run;


   %if &valoptionsRC eq 0 %then %do;
       /* Using Filename Pipe.  X command will not conditionally run.  %SYSEXE will only run Java that is default java */
       %if &sysscp eq WIN %then %do;
           /* Build the command in datastep to double quote file pathnames in case this is Windows so the quoting in */
           /* pipe command will be correct. Double each option that could be a pathname.                             */
           data _null_;
              format full_command $1024.;
              file "&batfile.";
              full_command=cat("",'"',"&java_loc.",'"'
                                 ,' -jar '
                                 ,'"',"&jar_loc.",'"'
                                 ,' "',"&xml_file.",'"'
                                 ,' "',"&xsd_file.",'"'
                                 ,' >','"',"&valfile.",'"',' 2>&1'
                                 ,"");
              call symput('pipecmd',trim(full_command));
              put full_command;
              run;
           /* single quote the command and double quote the path */
           data _null_;
               format full_cmd_path $1024.;
			   /* extra single quote is not needed if using the filename function for pipe */
               /*full_cmd_path=trim(cat("'",'"',"&batfile.",'"',"'"));*/
			   full_cmd_path=trim(cat('"',"&batfile.",'"'));
               call symput('BATCH_CMD',full_cmd_path);
               run;
		   /* Set filename pipe value and test to ensure it is capable to run (XCMD has been enabled). */
           %let filerf=runval;
           %let rc=%sysfunc(filename(filerf, %str(&batch_cmd.), pipe));
           %if &rc ne 0 %then %do;
		       %let _fcfmsg_=ERROR: Unable to execute system pipe command for Stored Process. Review the XCMD Launch property;
			   %let trans_rc=58;
			   options ls=72;
			   ods html close;
               data _null_;
                   file _webout;
                   put "&_fcfmsg_..";
               run;
			   ods html file=_webout;
			   %GOTO MACRO_END;
               %end;	
           /*filename runval pipe &batch_cmd.;*/
           %end;
        %else %do;
           /* Unix will not need the double quoting as in windows. */
           data _null_;
              format full_command $1024.;
              full_command=cat("&java_loc."
                                 ,' -jar '
                                 ,"&jar_loc."
                                 ,' ',"&xml_file."
                                 ,' ',"&xsd_file."
                                 ,' >',"&valfile.",' 2>&1'
                                 );
              call symput('pipecmd',trim(full_command));
              put full_command;
              run;
		   /* Set filename pipe value and test to ensure it is capable to run (XCMD has been enabled). */
           %let filerf=runval;
           %let rc=%sysfunc(filename(filerf, %str(&pipecmd.), pipe));
           %if &rc ne 0 %then %do;
		       %let _fcfmsg_=ERROR: Unable to execute system pipe command for Stored Process. Review the XCMD Launch property;
			   %let trans_rc=58;
			   options ls=72;
			   ods html close;
               data _null_;
                   file _webout;
                   put "&_fcfmsg_..";
               run;
			   ods html file=_webout;
			   %GOTO MACRO_END;
               %end;		   
           /*filename runval pipe "&pipecmd.";*/
           %end;
       data _null_;
           infile runval lrecl=1024 recfm=v missover;
           input @1 line $1024.;
           /*put line=;*/
       run;
       /* Check the output file for "xxxx is valid." for successful. */
       %if %sysfunc(fileexist("&valfile.")) eq 0 %then %do;
           %let validationRC=2;
           %GOTO MACRO_END;
           %end;
       /* Read the output file and look for is valid to set RC to 0. */
       %else %do;
           filename look "&valfile.";
           data validation_details;
              format error_line $1024.;
              infile look pad missover lrecl=1024 recfm=v end=done;
              input @1 error_line $1024.;
              if _n_=1 and index(error_line,'is valid.') > 0 then do;
                 call symput('validationRC',0);
                 stop;
                 end;
              else do;
                 call symput('validationRC',1);
                 if index(error_line,'at com.sas.xmlvalidator.') > 0 then delete;
                 output;
                 end;
           run;
		   /* if parsing errors are found, create a dataset of lines in error to flag */
		   data &error_file. (keep=error_line error_message error_line_number error_column_number);
		      format error_line_number 8. startlinepos endlinepos 8. startcolpos endcolpos 8. 
		             error_column_number 8. error_message $200.;
			    set validation_details;
			    if index(error_line,'org.xml.sax.SAXParseException') > 0 then do;
			        startlinepos=index(error_line,'lineNumber:') + 12; 
			        startcolpos=index(error_line,'columnNumber:') + 14; 
				    endlinepos=find(error_line,';',startlinepos);
				    endcolpos=find(error_line,';',startcolpos);
				    error_message=trim(substr(error_line,endcolpos+1));
				    error_line_number=input(substr(error_line,startlinepos,endlinepos-startlinepos),8.);
				    error_column_number=input(substr(error_line,startcolpos,endcolpos-startcolpos),8.);
				    output;
			        end;
		   run;
           filename look clear;
           %end;
       %end;

   %if ((&validationRC eq 0) && (&valoptionsRC eq 0)) %then %do;
       %put Note: Validation of &xml_file. was successful.;
       %end;
   %else %if ((&validationRC eq 1) && (&valoptionsRC eq 0)) %then %do;
       %put NOTE: Validation of &xml_file. failed. Error details in &error_file. dataset.;
	   %end;

%MACRO_END:

    %if &valoptionsRC=1 %then %do;
	    %let trans_rc=51;
        %let _fcfmsg_=ERROR: Unable to locate and open XML file passed with xml_file option.;
        %end;
    %if &valoptionsRC=2 %then %do;
	    %let trans_rc=52;
        %let _fcfmsg_=ERROR: Unable to locate and open XSD file passed with xsd_file option.;
        %end;
    %if &valoptionsRC=3 %then %do;
	    %let trans_rc=53;
        %let _fcfmsg_=ERROR: Unable to locate Java executable passed with java_loc option.;
		   options ls=72;
		   ods html close;
           data _null_;
                file _webout;
                put "&_fcfmsg_.";
           run;
     	   ods html file=_webout;
        %end;
    %if &valoptionsRC=4 %then %do;
	    %let trans_rc=54;
        %let _fcfmsg_=ERROR: Unable to locate xmlvalidator.jar file passed with jar_loc option.;
        %end;
    %if &validationRC=2 %then %do;
	    %let trans_rc=25;
        %let _fcfmsg_=ERROR: Problem executing the validation jar file. No output found.;
        %end;

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
	   %fcf_put_error;
       %fcf_execute_macro(fcf_error);
	   %end;
    %else 
       %fcf_execute_macro(fcf_success);		

%mend; /* rc_validate_xml */
