<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" indent="yes"/>
    <!-- Sample XLST file to validate SARX Report UI data and display FinCEN provided error code              -->
    <!-- and item number. The error codes will be joined with another lookup table for the                    -->
    <!-- error level and full error message.                                                                  -->
    <!-- Output format:
       <ReportValidations>
            <ReportValidation>
                 <entityName>
                 <entityKey>
                 <errorCode>
                 <itemNumber>
				 <targetFieldName>
				 <errorRecordType>
            </ReportValidation>
       </ReportValidations>
    -->

    <!-- Define some Global variables to use in validations -->
    <!-- FinCEN Valid country codes -->
    <xsl:variable name='fincen_us_countries' select="'US-CA-MX-AS-FM-GU-MH-MP-PR-PW-VI-UM'"/>
	<!-- FinCEN Valid US Territories for ZipCode validation (length 5 or 9 and no characters) -->
    <xsl:variable name='fincen_us_zipcodes' select="'US-AS-FM-GU-MH-MP-PR-PW-VI-UM'"/>
	<!-- Validate Country/State Lookups to help with C10 fatal errors -->
	<xsl:variable name='ca_state_codes' select="'AB-BC-MB-NB-NL-NS-NT-NU-ON-PE-QC-SK-YT'"/>
    <!-- Mexico State Codes have to follow ISO 3166-2: MX standard, so 3 chars -->
    <!-- Keeping DIF and CMX that are the old and new names for Mexico City -->
	<xsl:variable name='mx_state_codes' 
	 select="'CMX-AGU-BCN-BCS-CAM-COA-COL-CHP-CHH-DIF-DUR-GUA-GRO-HID-JAL-MEX-MIC-MOR-NAY-NLE-OAX-PUE-QUE-ROO-SLP-SIN-SON-TAB-TAM-TLA-VER-YUC-ZAC'"/>
	<!-- These include the military bases, such as AE, AA and AP -->
	<xsl:variable name='us_state_codes' 
	 select="'AL-AK-AE-AA-AP-AZ-AR-CA-CO-CT-DE-DC-FL-GA-HI-ID-IL-IN-IA-KS-KY-LA-ME-MD-MA-MI-MN-MS-MO-MT-NE-NV-NH-NJ-NM-NY-NC-ND-OH-OK-OR-PA-RI-SC-SD-TN-TX-UT-VT-VA-WA-WV-WI-WY'"/>
    <xsl:variable name='as_state_codes' select="'AS'"/>
    <xsl:variable name='fm_state_codes' select="'FM'"/>
    <xsl:variable name='gu_state_codes' select="'GU'"/>
    <xsl:variable name='mh_state_codes' select="'MH'"/>
    <xsl:variable name='mp_state_codes' select="'MP'"/>
    <xsl:variable name='pr_state_codes' select="'PR'"/>
    <xsl:variable name='pw_state_codes' select="'PW'"/>
    <xsl:variable name='vi_state_codes' select="'VI'"/>
    <xsl:variable name='um_state_codes' select="'UM'"/>
    <!-- alpha characters -->
    <xsl:variable name="alpha" select="'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>



    <!-- Valid phone types, if not one, Fatal Error -->
    <xsl:variable name='fincen_valid_phoneType' select="'A-B-C-D'"/>
    <!-- Global Joint report flag since used deep down in party tables -->
    <xsl:variable name='globalJointReport' select="/EfileCollection/reportCollections/reportCollection/report/reportUiJSON/jointReportFlag"/>

    <xsl:template match="/EfileCollection">
        <xsl:text>&#xd;</xsl:text>
        <ReportValidations>

            <!-- Fatal - No FINCEN_INSTITUTION found -->
            <xsl:if test="(not(fincenFiler))">
                <xsl:variable name='item_number' select="'Filing'"/>
                <xsl:variable name='error_code' select="'A4'"/>
                <xsl:variable name='target_field_name' select="'Institution'"/>
                <xsl:variable name='error_record_type' select="'batch'"/>
                <ReportValidation>
                    <entityName>fincenFiler</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <!-- Fatal - No FINCEN_TRANSMITTER found -->
            <xsl:if test="(not(fincenTransmitter))">
                <xsl:variable name='item_number' select="'Transmitter'"/>
                <xsl:variable name='error_code' select="'G0'"/>
                <xsl:variable name='target_field_name' select="'Institution'"/>
                <xsl:variable name='error_record_type' select="'transmitter'"/>
                <ReportValidation>
                    <entityName>fincenTransmitter</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <!-- Count for at least one Subject on the report -->
            <xsl:variable name='subject_count' select="count(reportCollections/reportCollection/report/subjects)"/>
            <xsl:if test="$subject_count &lt; 1">
                <xsl:variable name='item_number' select="'Subject'"/>
                <xsl:variable name='error_code' select="'C0'"/>
                <xsl:variable name='target_field_name' select="'Report'"/>
                <xsl:variable name='error_record_type' select="'party'"/>
                <ReportValidation>
                    <entityName>Subject</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>
            <!-- Maximum subjects -->
            <xsl:if test="$subject_count &gt; 999">
                <xsl:variable name='item_number' select="'Subject'"/>
                <xsl:variable name='error_code' select="'C1'"/>
                <xsl:variable name='target_field_name' select="'Report'"/>
                <xsl:variable name='error_record_type' select="'party'"/>
                <ReportValidation>
                    <entityName>Subject</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <!-- Count for at least one institution where activity occurred. -->
            <xsl:variable name='inst_count' select="count(reportCollections/reportCollection/report/institutions)"/>
            <xsl:if test="$inst_count &lt; 1">
                <xsl:variable name='item_number' select="'Activity Occurred'"/>
                <xsl:variable name='error_code' select="'D0'"/>
                <xsl:variable name='target_field_name' select="'Institutions'"/>
                <xsl:variable name='error_record_type' select="'branch'"/>
                <ReportValidation>
                    <entityName>Institutions Where Activity</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>
            <!-- Maximum institutions -->
            <xsl:if test="$inst_count &gt; 99">
                <xsl:variable name='item_number' select="'Activity Occurred'"/>
                <xsl:variable name='error_code' select="'E1'"/>
                <xsl:variable name='target_field_name' select="'Institutions'"/>
                <xsl:variable name='error_record_type' select="'filer'"/>
                <ReportValidation>
                    <entityName>Institutions Where Activity</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <!-- check for maximum branch count of 99 -->
            <xsl:variable name='branch_count' select="count(reportCollections/reportCollection/report/institutions/relationships/entityLevelCode[text()='BRANCH'])"/>
            <xsl:if test="$branch_count &gt; 99">
                <xsl:variable name='item_number' select="'Branches'"/>
                <xsl:variable name='error_code' select="'D1'"/>
                <xsl:variable name='target_field_name' select="'Institutions'"/>
                <xsl:variable name='error_record_type' select="'branch'"/>
                <ReportValidation>
                    <entityName>Institutions Where Activity</entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <!-- Sum up the total lossAmount for All the institutions.  Compare this total against the report involved Amount. -->
            <xsl:variable name="institutionLossAmount" select="sum(reportCollections/reportCollection/report/institutions/entityUiJSON/lossAmount)"/>
            <xsl:variable name="reportAmount" select="reportCollections/reportCollection/report/reportUiJSON/involvedAmount"/>
            <!-- Item 29 and Total of all 67 comparison. -->
            <xsl:if test="$institutionLossAmount &gt; $reportAmount">
                <xsl:variable name='item_number' select="'67'"/>
                <xsl:variable name='error_code' select="'D34'"/>
                <xsl:variable name='target_field_name' select="'Institution Loss Amount'"/>
                <xsl:variable name='error_record_type' select="'branch'"/>
                <ReportValidation>
                    <entityName>Report: <xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityName>
                    <errorCode><xsl:value-of select="$error_code"/></errorCode>
                    <entityKey><xsl:value-of select="reportCollections/reportCollection/report/reportKey"/></entityKey>
                    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                    <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                    <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
                </ReportValidation>
            </xsl:if>

            <xsl:apply-templates select="fincenFiler"/>

            <xsl:apply-templates select="fincenTransmitter"/>

            <xsl:apply-templates select="reportCollections/reportCollection/report/institutions/relationships/reportEntity"/>

            <xsl:apply-templates select="reportCollections/reportCollection/report"/>

            <xsl:apply-templates select="reportCollections/reportCollection/report/subjects"/>

            <xsl:apply-templates select="reportCollections/reportCollection/report/reportUiJSON"/>



        </ReportValidations>
    </xsl:template>

    <!-- Validate Financial Transmitter -->
    <xsl:template match="fincenTransmitter">

        <xsl:if test="(transmitterName='') or not (transmitterName)">
            <xsl:variable name='item_number' select="'Name'"/>
            <xsl:variable name='error_code' select="'G2'"/>
			<xsl:variable name='target_field_name' select="'Transmitter'"/>
			<xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(transmitterAddress='') or not (transmitterAddress)">
            <xsl:variable name='item_number' select="'Address'"/>
            <xsl:variable name='error_code' select="'G3'"/>
			<xsl:variable name='target_field_name' select="'Transmitter'"/>
			<xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(transmitterCityName='') or not (transmitterCityName)">
            <xsl:variable name='item_number' select="'City'"/>
            <xsl:variable name='error_code' select="'G4'"/>
			<xsl:variable name='target_field_name' select="'Transmitter'"/>
			<xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:if test="(transmitterTaxId='')">
            <xsl:variable name='item_number' select="'TaxID'"/>
            <xsl:variable name='error_code' select="'G14'"/>
			<xsl:variable name='target_field_name' select="'Transmitter'"/>
			<xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:if test="(transmitterZipCode='')">
            <xsl:variable name='item_number' select="'ZipCode'"/>
            <xsl:variable name='error_code' select="'G7'"/>
            <xsl:variable name='target_field_name' select="'Transmitter'"/>
            <xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:if test="(transmitterCountryCode='')">
            <xsl:variable name='item_number' select="'Country'"/>
            <xsl:variable name='error_code' select="'G9'"/>
            <xsl:variable name='target_field_name' select="'Transmitter'"/>
            <xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:if test="(transmitterStateCode='')">
            <xsl:variable name='item_number' select="'State'"/>
            <xsl:variable name='error_code' select="'G5'"/>
            <xsl:variable name='target_field_name' select="'Transmitter'"/>
            <xsl:variable name='error_record_type' select="'transmitter'"/>
            <ReportValidation>
                <entityName>Transmitter Name: <xsl:value-of select="transmitterName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="transmitterKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Validate Financial Institution (Filing and Where Activity Occurred -->
    <xsl:template match="fincenFiler">

        <!-- Item 76 Filer Name -->
        <xsl:if test="not (institutionLegalName)">
            <xsl:variable name='item_number' select="'76'"/>
            <xsl:variable name='error_code' select="'B1'"/>
			<xsl:variable name='target_field_name' select="'Institution Legal Name'"/>
			<xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="institutionLegalName=''">
            <xsl:variable name='item_number' select="'76'"/>
            <xsl:variable name='error_code' select="'B2'"/>
			<xsl:variable name='target_field_name' select="'Institution Legal Name'"/>
			<xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 77 TIN Number -->
        <xsl:if test="(institutionTaxId='') or not (institutionTaxId)">
            <xsl:variable name='item_number' select="'77'"/>
            <xsl:variable name='error_code' select="'E7'"/>
			<xsl:variable name='target_field_name' select="'Institution Tax ID'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="number(institutionTaxId ) != institutionTaxId">
            <xsl:variable name='item_number' select="'77'"/>
            <xsl:variable name='error_code' select="'E9'"/>
			<xsl:variable name='target_field_name' select="'Institution Tax ID'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 78 TIN Type -->
        <xsl:if test="(institutionTaxIdType='') or not (institutionTaxIdType)">
            <xsl:variable name='item_number' select="'78'"/>
            <xsl:variable name='error_code' select="'E46'"/>
			<xsl:variable name='target_field_name' select="'Institution Tax ID Type'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 79 Institution Type Code -->
        <xsl:if test="(institutionTypeCode='') or not (institutionTypeCode)">
            <xsl:variable name='item_number' select="'79'"/>
            <xsl:variable name='error_code' select="'E37'"/>
			<xsl:variable name='target_field_name' select="'Institution Type Code'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 75 Institution Regulatory Code -->
        <xsl:if test="(institutionPrimFedRegCode='') or not (institutionPrimFedRegCode)">
            <xsl:variable name='item_number' select="'75'"/>
            <xsl:variable name='error_code' select="'E3'"/>
			<xsl:variable name='target_field_name' select="'Institution Primary Regulatory Code'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- 79/75 Cross Reference Validations -->
        <xsl:if test="((institutionTypeCode='A') or (institutionTypeCode='C') or (institutionTypeCode='D')) and not (institutionPrimFedRegCode='D')">
           <xsl:variable name='item_number' select="'75'"/>
           <xsl:variable name='error_code' select="'E4'"/>
			<xsl:variable name='target_field_name' select="'Institution Type Code'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
           <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
    		    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
           </ReportValidation>
        </xsl:if>
        <xsl:if test="(institutionTypeCode='E') and not (institutionPrimFedRegCode='A' or institutionPrimFedRegCode='G')">
           <xsl:variable name='item_number' select="'75'"/>
           <xsl:variable name='error_code' select="'E4'"/>
		   <xsl:variable name='target_field_name' select="'Institution Type Code'"/>
		   <xsl:variable name='error_record_type' select="'filer'"/>
           <ReportValidation>
               <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
               <errorCode><xsl:value-of select="$error_code"/></errorCode>
               <entityKey><xsl:value-of select="institutionKey"/></entityKey>
			   <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
			   <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
			   <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
           </ReportValidation>
        </xsl:if>
        <xsl:if test="(institutionTypeCode='B') and not (institutionPrimFedRegCode='B' or institutionPrimFedRegCode='C'
                       or institutionPrimFedRegCode='D' or institutionPrimFedRegCode='E' or institutionPrimFedRegCode='F')">
            <xsl:variable name='item_number' select="'75'"/>
            <xsl:variable name='error_code' select="'E4'"/>
			<xsl:variable name='target_field_name' select="'Institution Type Code'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>


        <!-- Gaming Type; relates to the Institution where activity occurred. -->
        <xsl:if test="((institutionTypeCode='A') and not(institutionGamingTypeCode='A' or
                                                         institutionGamingTypeCode='B' or
                                                         institutionGamingTypeCode='C' or
                                                         institutionGamingTypeCode='Z'))">
            <xsl:variable name='item_number' select="'53'"/>
            <xsl:variable name='error_code' select="'D10'"/>
			<xsl:variable name='target_field_name' select="'Institution Gaming Type Code'"/>
			<xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(institutionGamingTypeCode='A' or institutionGamingTypeCode='B' or
                       institutionGamingTypeCode='C' or institutionGamingTypeCode='Z') and (not(institutionTypeCode='A'))">
            <xsl:variable name='item_number' select="'53'"/>
            <xsl:variable name='error_code' select="'D12'"/>
			<xsl:variable name='target_field_name' select="'Institution Gaming Type Code'"/>
			<xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- The Other choice for Gaming -->
        <xsl:if test="((institutionGamingTypeCode='Z') and (not (institutionGamingTypeOther) or (institutionGamingTypeOther='')))">
            <xsl:variable name='item_number' select="'54'"/>
            <xsl:variable name='error_code' select="'D15'"/>
			<xsl:variable name='target_field_name' select="'Institution Gaming Type Code'"/>
			<xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(institutionGamingTypeOther) and not (institutionGamingTypeCode='Z')">
            <xsl:variable name='item_number' select="'54'"/>
            <xsl:variable name='error_code' select="'D16'"/>
			<xsl:variable name='target_field_name' select="'Institution Gaming Type Other'"/>
			<xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 80 Type of Securities -->
        <!-- Count the number of true selected securities to help with error checks. -->
        <xsl:variable name="sf_count" select="count(*[contains(local-name(),'institutionSfTypeCode')][text()='true'])"/>
        <xsl:if test="(institutionTypeCode='E') and ($sf_count &lt; 1)">
            <xsl:variable name='item_number' select="'80'"/>
            <xsl:variable name='error_code' select="'E15'"/>
			<xsl:variable name='target_field_name' select="'Institution Securities/Futures'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="not(institutionTypeCode='E') and ($sf_count &gt; 0)">
            <xsl:variable name='item_number' select="'80'"/>
            <xsl:variable name='error_code' select="'E17'"/>
			<xsl:variable name='target_field_name' select="'Institution Securities/Futures'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- The Other for the Institution type -->
        <xsl:if test="((institutionTypeCode='Z') and (not (institutionTypeOther) or (institutionTypeOther='')))">
            <xsl:variable name='item_number' select="'79'"/>
            <xsl:variable name='error_code' select="'E13'"/>
			<xsl:variable name='target_field_name' select="'Institution Type Other'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(institutionTypeOther) and not (institutionTypeCode='Z')">
            <xsl:variable name='item_number' select="'79'"/>
            <xsl:variable name='error_code' select="'E14'"/>
			<xsl:variable name='target_field_name' select="'Institution Type Other'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 81 Institution ID -->
        <xsl:if test="((institutionId='') or not (institutionId)) and  (not(institutionIdTypeCode='') or not(institutionIdTypeCode))">
            <xsl:variable name='item_number' select="'81'"/>
            <xsl:variable name='error_code' select="'E21'"/>
			<xsl:variable name='target_field_name' select="'Institution ID'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="((institutionIdTypeCode='') or not (institutionIdTypeCode)) and  (not(institutionId='') or not(institutionId))">
            <xsl:variable name='item_number' select="'81'"/>
            <xsl:variable name='error_code' select="'E21'"/>
			<xsl:variable name='target_field_name' select="'Institution ID Type'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="number(institutionId ) != institutionId">
            <xsl:variable name='item_number' select="'81'"/>
            <xsl:variable name='error_code' select="'E22'"/>
			<xsl:variable name='target_field_name' select="'Institution ID'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 82 Address -->
        <xsl:if test="(institutionAddress='') or not (institutionAddress)">
            <xsl:variable name='item_number' select="'82'"/>
            <xsl:variable name='error_code' select="'E24'"/>
			<xsl:variable name='target_field_name' select="'Institution Address'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 83 City -->
        <xsl:if test="(institutionCityName='') or not (institutionCityName)">
            <xsl:variable name='item_number' select="'83'"/>
            <xsl:variable name='error_code' select="'E26'"/>
			<xsl:variable name='target_field_name' select="'Institution City'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 84 State -->
        <xsl:variable name='CountryCode' select="institutionCountryCode"/>
        <xsl:if test="contains($fincen_us_countries,$CountryCode) and ((institutionStateCode='') or not (institutionStateCode))">
            <xsl:variable name='item_number' select="'84'"/>
            <xsl:variable name='error_code' select="'E28'"/>
			<xsl:variable name='target_field_name' select="'Institution Country'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Foreign country, cannot have State value -->
        <xsl:if test="not(contains($fincen_us_countries,$CountryCode)) and (not(institutionStateCode='') and (institutionStateCode))">
            <xsl:variable name='item_number' select="'84'"/>
            <xsl:variable name='error_code' select="'E30'"/>
			<xsl:variable name='target_field_name' select="'Institution Country'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 85 Postal Code -->
        <xsl:if test="(institutionZipCode='') or not (institutionZipCode)">
            <xsl:variable name='item_number' select="'85'"/>
            <xsl:variable name='error_code' select="'E34'"/>
			<xsl:variable name='target_field_name' select="'Institution ZipCode'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- ZipCode containing punctuation -->
        <xsl:variable name='ZipCode' select="institutionZipCode"/>
        <xsl:if test="contains($ZipCode,'.') or contains($ZipCode,'-') or contains($ZipCode,'/') or contains($ZipCode,'\')
                   or contains($ZipCode,'=') or contains($ZipCode,',') or contains($ZipCode,' ') or contains($ZipCode,'|')">
            <xsl:variable name='item_number' select="'85'"/>
            <xsl:variable name='error_code' select="'E32'"/>
			<xsl:variable name='target_field_name' select="'Institution ZipCode'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 86 Country -->
        <xsl:if test="(institutionCountryCode='') or not (institutionCountryCode)">
            <xsl:variable name='item_number' select="'86'"/>
            <xsl:variable name='error_code' select="'E37'"/>
			<xsl:variable name='target_field_name' select="'Institution Country'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Institution: <xsl:value-of select="institutionLegalName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="institutionKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Validation for Branch data where Activity Occurred.  Focus on State and Country errors. -->
    <xsl:template match="reportEntity">

        <xsl:variable name='branchCountry' select="entityUiJSON/branchCountryCode"/>
        <xsl:variable name='branchState' select="entityUiJSON/branchStateCode"/>

        <!-- Foreign country, cannot have State value -->
        <xsl:if test="not(contains($fincen_us_countries,$branchCountry)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D45'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Foreign country, cannot have State value -->
        <xsl:if test="not(contains($fincen_us_countries,$branchCountry)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D44'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- US Territory and no state value -->
        <xsl:if test="( ((entityUiJSON/branchStateCode='') or not(entityUiJSON/branchStateCode)) and contains($fincen_us_countries,$branchCountry) )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D46'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 74 Branch Country invalid characters for country and length has to be 2 -->
        <xsl:if test="( (translate(entityUiJSON/branchCountryCode, $alpha, '')) or (not(string-length(entityUiJSON/branchCountryCode)=2)) )">
        <xsl:variable name='item_number' select="'74'"/>
            <xsl:variable name='error_code' select="'D49'"/>
            <xsl:variable name='target_field_name' select="'Branch Country'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 74 Branch Country -->
        <xsl:if test="(entityUiJSON/branchCountryCode='') or not (entityUiJSON/branchCountryCode)">
            <xsl:variable name='item_number' select="'74'"/>
            <xsl:variable name='error_code' select="'D50'"/>
            <xsl:variable name='target_field_name' select="'Branch Country'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- test for the valid us territory and state values -->
        <xsl:if test="( (not(contains($us_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='US') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mx_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='MX') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($ca_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='CA') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($as_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='AS') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($fm_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='FM') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($gu_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='GU') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mh_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='MH') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mp_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='MP') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pr_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='PR') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pw_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='PW') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($vi_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='VI') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($um_state_codes,$branchState)) and (not(entityUiJSON/branchStateCode='') and (entityUiJSON/branchStateCode))) and (entityUiJSON/branchCountryCode='UM') )">
            <xsl:variable name='item_number' select="'72'"/>
            <xsl:variable name='error_code' select="'D47'"/>
            <xsl:variable name='target_field_name' select="'Branch State'"/>
            <xsl:variable name='error_record_type' select="'branch'"/>
            <ReportValidation>
                <entityName>Branch: <xsl:value-of select="entityName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="entityNumber"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <xsl:template match="report">

        <!-- Narrative -->
        <xsl:if test="(reportNarrative='') or not (reportNarrative)">
            <xsl:variable name='item_number' select="'Narrative'"/>
            <xsl:variable name='error_code' select="'F2'"/>
			<xsl:variable name='target_field_name' select="'Narrative'"/>
			<xsl:variable name='error_record_type' select="'attachment'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Validation Against a possible multiple parties. -->
    <xsl:template match="subjects">

        <xsl:apply-templates select="entityUiJSON"/>
        <xsl:apply-templates select="relationships"/>
        <!--xsl:apply-templates select="relationships/bridgeUiJSON"/-->

    </xsl:template>

    <!-- Need a check to see if no institution to subjects have been established for a C41 message -->
    <xsl:template match="relationships">
        <xsl:if test="bridgeUiJSON and bridgeUiJSON=''">
           <xsl:variable name='item_number' select="'24'"/>
            <xsl:variable name='error_code' select="'C41'"/>
            <xsl:variable name='target_field_name' select="'Subject Relationship'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../entityName"/> (<xsl:value-of select="reportEntity/entityNumber"/>)</entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="parentEntityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:apply-templates select="bridgeUiJSON"/>

    </xsl:template>

    <!-- Drill down into the subjects Detail -->
    <xsl:template match="entityUiJSON">

        <!-- for some fields, they are option depending on the entityFlag='true' or the -->
        <!-- subjectDetailsUnknownFlag='true'.                                          -->


        <!-- Item 5 First Name -->
        <xsl:if test="( (((firstName='') or not(firstName)) and (not(firstNameUnknownFlag='true')))
                 and ((not(subjectDetailsUnknownFlag='true')) and not(entityFlag='true')) )">
            <xsl:variable name='item_number' select="'5'"/>
            <xsl:variable name='error_code' select="'C4'"/>
			<xsl:variable name='target_field_name' select="'Subject First Name'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 4 Last Name -->
        <xsl:if test="( (((lastName='') or not(lastName)) and (not(lastNameUnknownFlag='true')))
                 and (not(subjectDetailsUnknownFlag='true')) )">
            <xsl:variable name='item_number' select="'4'"/>
            <xsl:variable name='error_code' select="'C3'"/>
			<xsl:variable name='target_field_name' select="'Subject Last Name'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 8 Gender -->
        <xsl:if test="( ((gender='') or not(gender)) and
                      ((not(subjectDetailsUnknownFlag='true')) and not(entityFlag='true')) )">
            <xsl:variable name='item_number' select="'8'"/>
            <xsl:variable name='error_code' select="'C5'"/>
			<xsl:variable name='target_field_name' select="'Subject Gender'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="((gender='A') or (gender='B')) and
                      (entityFlag='true')">
            <xsl:variable name='item_number' select="'8'"/>
            <xsl:variable name='error_code' select="'C53'"/>
			<xsl:variable name='target_field_name' select="'Subject Gender'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 19 DOB -->
        <xsl:if test="( (((birthDate='') or not(birthDate)) and (not(birthDateUnknownFlag='true')))
                 and ((not(subjectDetailsUnknownFlag='true')) and not(entityFlag='true')) )">
            <xsl:variable name='item_number' select="'19'"/>
            <xsl:variable name='error_code' select="'C35'"/>
			<xsl:variable name='target_field_name' select="'Subject Date of Birth'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 19 DOB < 01/01/1900 -->
        <xsl:variable name='dob_date' select="10000 * substring(birthDate,1,4) + 100 * substring(birthDate,6,2) + substring(birthDate,9,2)"/>
        <xsl:variable name='jan1_date' select="10000 * '1900' + 100 * '01' + '01'"/>
        <xsl:if test="$dob_date &lt; $jan1_date">
            <xsl:variable name='item_number' select="'19'"/>
            <xsl:variable name='error_code' select="'C36'"/>
			<xsl:variable name='target_field_name' select="'Subject Date of Birth'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 16 TIN and 17 Type -->
        <xsl:if test="(( ((TaxID='') or not(TaxID) and (TaxIDType='') or not(TaxIDType)) and (not(TaxIDUnknownFlag='true')) )
                 and ( not(subjectDetailsUnknownFlag='true') ) )">
            <xsl:variable name='item_number' select="'16-17'"/>
            <xsl:variable name='error_code' select="'C19'"/>
			<xsl:variable name='target_field_name' select="'Subject TIN'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 16 TIN -->
        <xsl:if test="(( ((TaxID='') or not(TaxID)) and (not(TaxIDUnknownFlag='true')) )
                 and ( not(subjectDetailsUnknownFlag='true') ) )">
            <xsl:variable name='item_number' select="'16'"/>
            <xsl:variable name='error_code' select="'C20'"/>
			<xsl:variable name='target_field_name' select="'Subject TIN'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 17 TIN Type-->
        <xsl:if test="(( ((TaxIDType='') or not(TaxIDType)) and (not(TaxIDUnknownFlag='true')) )
                 and ( not(subjectDetailsUnknownFlag='true') ) )">
            <xsl:variable name='item_number' select="'17'"/>
            <xsl:variable name='error_code' select="'C54'"/>
			<xsl:variable name='target_field_name' select="'Subject TIN Type'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 11-15 Address Down One Level under subjectAddress -->
        <xsl:if test="(not(subjectAddress) and not(subjectDetailsUnknownFlag='true'))">
            <xsl:variable name='item_number' select="'11-15'"/>
            <xsl:variable name='error_code' select="'C7'"/>
			<xsl:variable name='target_field_name' select="'Subject Address'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:apply-templates select="subjectAddress"/>

        <!-- Item 18 Subject ID's Down One Level under subjectID -->
        <xsl:if test="( not(subjectID) and ((not(subjectDetailsUnknownFlag='true')) and (not(noSubjectIDFlag='true'))) )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C22'"/>
			<xsl:variable name='target_field_name' select="'Subject ID'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:apply-templates select="subjectID"/>
		
		<!-- Leading and Trailing Blanks for First Name and Last Name -->
		<xsl:variable name="trimlastName">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="lastName"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="trimfirstName">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="firstName"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='lentrimlastName' select="string-length(lastName) - string-length($trimlastName)"/>
        <xsl:variable name='lentrimfirstName' select="string-length(firstName) - string-length($trimfirstName)"/>
        <xsl:if test="( firstName and ($lentrimfirstName &gt; 0) )">
            <xsl:variable name='item_number' select="'5'"/>
            <xsl:variable name='error_code' select="'A19'"/>
			<xsl:variable name='target_field_name' select="'Subject First Name'"/>
			<xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		<xsl:if test="( lastName and ($lentrimlastName &gt; 0) )">
            <xsl:variable name='item_number' select="'4'"/>
            <xsl:variable name='error_code' select="'A19'"/>
			<xsl:variable name='target_field_name' select="'Subject Last Name'"/>
			<xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 9 Alternate Names -->
        <xsl:apply-templates select="subjectAlternateName"/>

        <!-- Item 20-21 Phone information Down One Level under subjectPhone -->
        <xsl:apply-templates select="subjectPhone"/>

        <!-- debug statement only -->
        <!--testname><xsl:value-of select="lastName"/></testname-->

        <!-- Item 22 Web URL/email Down One Level under subjectURL -->
        <xsl:apply-templates select="subjectUrl"/>
        <xsl:apply-templates select="subjectEmail"/>

        <!-- Item 24-26 Subject Relation Down One level under subjectInstitutionRelation -->
        <xsl:apply-templates select="subjectInstitutionRelation"/>

        <!-- Item 27 Subject Accounts Down One Level under subjectAccount -->
        <xsl:if test="(subjectAccount/accountID) and ((noAccountFlag='true'))">
            <xsl:variable name='item_number' select="'27'"/>
            <xsl:variable name='error_code' select="'C47'"/>
			<xsl:variable name='target_field_name' select="'Subject Accounts'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( not(subjectAccount/accountID) and ((noAccountFlag='false') or not(noAccountFlag)) )">
            <xsl:variable name='item_number' select="'27'"/>
            <xsl:variable name='error_code' select="'C48'"/>
			<xsl:variable name='target_field_name' select="'Subject Accounts'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:apply-templates select="subjectAccount"/>

    </xsl:template>

    <!-- Subject Alternate Names Item 9 -->
    <xsl:template match="subjectAlternateName">
        <xsl:variable name="trimalternateName">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="alternateName"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='lenalternateName' select="string-length(alternateName)- string-length($trimalternateName)"/>
        <xsl:if test="(alternateName and ($lenalternateName &gt; 0) )">
            <xsl:variable name='item_number' select="'9'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject Alternate Name'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
    </xsl:template>

    <!-- Subject Address Items 11-15 -->
    <xsl:template match="subjectAddress">

        <!-- Item 11 Address -->
        <xsl:if test="( (address='') or not(address) ) and ( not(../subjectDetailsUnknownFlag='true') and not(addressUnknownFlag='true'))">
            <xsl:variable name='item_number' select="'11'"/>
            <xsl:variable name='error_code' select="'C8'"/>
			<xsl:variable name='target_field_name' select="'Subject Address'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 12 City -->
        <xsl:if test="( (city='') or not(city) ) and ( not(../subjectDetailsUnknownFlag='true') and not(cityUnknownFlag='true'))">
            <xsl:variable name='item_number' select="'12'"/>
            <xsl:variable name='error_code' select="'C9'"/>
			<xsl:variable name='target_field_name' select="'Subject City'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 13 State -->
        <xsl:variable name='addressCountryCode' select="countryCode"/>
		<xsl:variable name='addressStateCode' select="stateCode"/>
        <xsl:if test="( ( (((stateCode='') or not(stateCode)) and (not(stateUnknownFlag='true'))) and contains($fincen_us_countries,$addressCountryCode) )
                  and (not(../subjectDetailsUnknownFlag='true')) )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C11'"/>
			<xsl:variable name='target_field_name' select="'Subject State'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Foreign country, cannot have State value -->
        <xsl:if test="( (not(contains($fincen_us_countries,$addressCountryCode))) and ( (not(stateCode='') and (stateCode)) ) )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C12'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Country and Country Unknown cannot both be populated. -->
        <xsl:if test="( (countryUnknownFlag='true') and ( (not(countryCode='') and (countryCode)) ) )">
            <xsl:variable name='item_number' select="'15'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject Country'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- State and State Unknown cannot both be populated. -->
        <xsl:if test="( (stateUnknownFlag='true') and ( (not(stateCode='') and (stateCode)) ) )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'A24'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- City and City Unknown cannot both be populated. -->
        <xsl:if test="( (cityUnknownFlag='true') and ( (not(city='') and (city)) ) )">
            <xsl:variable name='item_number' select="'12'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject City'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Zipcode and Zipcode Unknown cannot both be populated. -->
        <xsl:if test="( (postalCodeUnknownFlag='true') and ( (not(postalCode='') and (postalCode)) ) )">
            <xsl:variable name='item_number' select="'14'"/>
            <xsl:variable name='error_code' select="'A24'"/>
            <xsl:variable name='target_field_name' select="'Subject ZipCode'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Address and Address Unknown cannot both be populated. -->
        <xsl:if test="( (addressUnknownFlag='true') and ( (not(address='') and (address)) ) )">
            <xsl:variable name='item_number' select="'11'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject Address'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Invalid State Code for US, MX or Canada Countries.  This produces fatal C10 error codes for Subject. -->
        <xsl:if test="( (not(contains($ca_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='CA') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		<xsl:if test="( (not(contains($mx_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='MX') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		<xsl:if test="( (not(contains($us_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='US') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($as_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='AS') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($fm_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='FM') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($gu_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='GU') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mh_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='MH') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mp_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='MP') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pr_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='PR') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pw_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='PW') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($vi_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='VI') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($um_state_codes,$addressStateCode)) and (not(stateCode='') and (stateCode))) and (countryCode='UM') )">
            <xsl:variable name='item_number' select="'13'"/>
            <xsl:variable name='error_code' select="'C10'"/>
            <xsl:variable name='target_field_name' select="'Subject State'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 14 Zip -->
        <xsl:if test="( ((postalCode='') or not(postalCode)) and (not(../subjectDetailsUnknownFlag='true') and not(postalCodeUnknownFlag='true')) )">
            <xsl:variable name='item_number' select="'14'"/>
            <xsl:variable name='error_code' select="'C15'"/>
			<xsl:variable name='target_field_name' select="'Subject ZipCode'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- ZipCode containing punctuation -->
        <xsl:variable name='partyZipCode' select="postalCode"/>
        <xsl:if test="contains($partyZipCode,'.') or contains($partyZipCode,'-') or contains($partyZipCode,'/') or contains($partyZipCode,'\')
                   or contains($partyZipCode,'=') or contains($partyZipCode,',') or contains($partyZipCode,' ') or contains($partyZipCode,'|')">"
            <xsl:variable name='item_number' select="'14'"/>
            <xsl:variable name='error_code' select="'C14'"/>
			<xsl:variable name='target_field_name' select="'Subject ZipCode'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName(Zip): <xsl:value-of select="../lastName"/>(<xsl:value-of select="postalCode"/>)</entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- US and US Territory specific checks for C16 which is Length 5 or 9. -->
        <xsl:variable name='lenZipCode' select="string-length($partyZipCode)"/>
        <xsl:if test="( (contains($fincen_us_zipcodes,$addressCountryCode)) and (($lenZipCode != 5) and ($lenZipCode != 9)) and (not(postalCodeUnknownFlag='true'))
                         and (not(../subjectDetailsUnknownFlag='true')))">
            <xsl:variable name='item_number' select="'14'"/>
            <xsl:variable name='error_code' select="'C16'"/>
            <xsl:variable name='target_field_name' select="'Subject ZipCode'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName(Zip): <xsl:value-of select="../lastName"/>(<xsl:value-of select="postalCode"/>)</entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- US and US Territory specific checks for C14 (Fatal). Make sure its a number. -->
        <xsl:if test="( (contains($fincen_us_zipcodes,$addressCountryCode)) and (number(postalCode) != postalCode) and (not(postalCodeUnknownFlag='true'))
                         and (not(../subjectDetailsUnknownFlag='true')))">
            <xsl:variable name='item_number' select="'14'"/>
            <xsl:variable name='error_code' select="'C14'"/>
            <xsl:variable name='target_field_name' select="'Subject ZipCode'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName(Zip): <xsl:value-of select="../lastName"/>(<xsl:value-of select="postalCode"/>)</entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 15 Country -->
        <xsl:if test="(((countryCode='') or not(countryCode)) and (not(../subjectDetailsUnknownFlag='true'))
                 and (not(countryUnknownFlag='true')) )">
            <xsl:variable name='item_number' select="'15'"/>
            <xsl:variable name='error_code' select="'C18'"/>
			<xsl:variable name='target_field_name' select="'Subject Country'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Leading and Trailing Blanks for City and Address that can cause an A19 Schema error. -->
        <xsl:variable name="trimcity">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="city"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="trimaddress">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="address"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='lentrimcity' select="string-length(city)- string-length($trimcity)"/>
        <xsl:variable name='lentrimaddress' select="string-length(address)- string-length($trimaddress)"/>
        <xsl:if test="( address and ($lentrimaddress &gt; 0) )">
            <xsl:variable name='item_number' select="'11'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject Address'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( city and ($lentrimcity &gt; 0) )">
            <xsl:variable name='item_number' select="'12'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject City'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Subject ID's 18 -->
    <xsl:template match="subjectID">

        <!-- Item 18  -->
        <xsl:if test="((IDNumber and not(IDNumber='')) and (../noSubjectIDFlag='true'))">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C23'"/>
			<xsl:variable name='target_field_name' select="'Subject ID'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Drivers License specific -->
        <xsl:variable name='IDCountryCode' select="IDAuthorityCountryCode"/>
        <xsl:if test="( ((contains($fincen_us_countries,$IDCountryCode) and IDAuthorityCountryCode ) and ((IDAuthorityStateCode='') or not (IDAuthorityStateCode)))
                  and (IDType='A') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C25'"/>
			<xsl:variable name='target_field_name' select="'Subject ID'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <xsl:variable name='SubjectID' select="IDNumber"/>
        <xsl:if test="contains($SubjectID,'.') or contains($SubjectID,'-') or contains($SubjectID,'/') or contains($SubjectID,'\')
                   or contains($SubjectID,'=') or contains($SubjectID,',') or contains($SubjectID,' ') or contains($SubjectID,'|')">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C24'"/>
            <xsl:variable name='target_field_name' select="'IDNumber'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Passport and Alien Registration -->
        <xsl:if test="( ((IDAuthorityCountryCode='') or not (IDAuthorityCountryCode))
                  and   ((IDType='B') or (IDType='C')) )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C29'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Other type make sure they provide the Other value -->
        <xsl:if test="( ((IDTypeOther='') or not (IDTypeOther))
                  and   (IDType='Z') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C32'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Check the Other Type values for leading and trailing blanks that will trigger an A19. -->
        <!-- These are caught with xsd schema validation, but adding this extra validation.        -->
        <xsl:variable name="trimIDTypeOther">
            <xsl:call-template name="string-trim">
                <xsl:with-param name="string" select="IDTypeOther"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='lentrimIDTypeOther' select="string-length(IDTypeOther) - string-length($trimIDTypeOther)"/>
        <xsl:if test="( IDTypeOther and ($lentrimIDTypeOther &gt; 0) )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'A19'"/>
            <xsl:variable name='target_field_name' select="'Subject ID Type'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>LastName(IDType): <xsl:value-of select="../lastName"/>(<xsl:value-of select="IDTypeOther"/>)</entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- If the ID Type is missing, FinCEN is flagging a Fatal C54 Error that is Tin Type Error. -->
        <xsl:if test="(( ((IDType='') or not(IDType)) and (not(noSubjectIDFlag='true')) )
                 and ( not(../subjectDetailsUnknownFlag='true') ) )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C54'"/>
            <xsl:variable name='target_field_name' select="'Subject ID Type'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- C28 Will Involve if selected a US Territory and State is not valid for territory. -->
        <xsl:variable name='IDStateCode' select="IDAuthorityStateCode"/>
        <xsl:if test="( (not(contains($us_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='US') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mx_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='MX') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($ca_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='CA') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($as_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='AS') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($fm_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='FM') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($gu_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='GU') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mh_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='MH') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($mp_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='MP') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pr_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='PR') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($pw_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='PW') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($vi_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='VI') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (not(contains($um_state_codes,$IDStateCode)) and (not(IDAuthorityStateCode='') and (IDAuthorityStateCode))) and (IDAuthorityCountryCode='UM') )">
            <xsl:variable name='item_number' select="'18'"/>
            <xsl:variable name='error_code' select="'C28'"/>
            <xsl:variable name='target_field_name' select="'Subject ID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>


    </xsl:template>

    <!-- Subject Phone Item 20-21 -->
    <xsl:template match="subjectPhone">
        <xsl:variable name='subjectphoneType' select="phoneTypeCode"/>
        <xsl:if test="(( not(contains($fincen_valid_phoneType,$subjectphoneType)) or (not(phoneTypeCode)) ) and phoneNumber)">
            <xsl:variable name='item_number' select="'20'"/>
            <xsl:variable name='error_code' select="'C37'"/>
			<xsl:variable name='target_field_name' select="'Subject Phone'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- 22a Subject URL -->
    <xsl:template match="subjectUrl">

        <!-- URL address, only validation it must contain at least 1 period and no space. -->
        <xsl:variable name='partyURL' select="URLAddress"/>
        <xsl:variable name='periodLocation' select="string-length(substring-before($partyURL,'.'))"/>
        <xsl:if test="URLAddress and not(URLAddress='') and (($periodLocation &lt; 1) or (contains(URLAddress,' ')))">
            <xsl:variable name='item_number' select="'22a'"/>
            <xsl:variable name='error_code' select="'C38'"/>
			<xsl:variable name='target_field_name' select="'Subject URL'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- 22 Subject Email -->
    <xsl:template match="subjectEmail">

        <!-- Email address, must have an @ sign and a period after the @ sign -->
        <xsl:variable name='partyEmail' select="emailAddress"/>
        <xsl:variable name='atSign' select="string-length(substring-before($partyEmail,'@'))"/>
        <xsl:variable name='stringAfter' select="substring-after($partyEmail,'@')"/>
        <xsl:variable name='periodAfter' select="string-length(substring-before($stringAfter,'.'))"/>
        <xsl:if test="emailAddress and not(emailAddress='') and (($atSign &lt; 1) or ($periodAfter &lt; 1))">
            <xsl:variable name='item_number' select="'22'"/>
            <xsl:variable name='error_code' select="'C38'"/>
			<xsl:variable name='target_field_name' select="'Subject Email'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Subject Relations to Institutions 24-26 -->
    <xsl:template match="bridgeUiJSON">

        <!-- Item 26 Status Date not provided, but status code B, C or D -->
        <xsl:if test="( (statusCode='B' or statusCode='C' or statusCode='D') and ((statusDate='') or not(statusDate)))">
            <xsl:variable name='item_number' select="'26'"/>
            <xsl:variable name='error_code' select="'C45'"/>
			<xsl:variable name='target_field_name' select="'Subject Relationship Date'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../../entityName"/> (<xsl:value-of select="../reportEntity/entityNumber"/>)</entityName>
                <!--entityName>Subject: <xsl:value-of select="../../entityName"/></entityName-->
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../parentEntityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 26 Status Date provided, but not a status code B, C or D -->
        <xsl:if test="( not(statusCode='B' or statusCode='C' or statusCode='D') and (not(statusDate='') and(statusDate)))">
            <xsl:variable name='item_number' select="'26'"/>
            <xsl:variable name='error_code' select="'C55'"/>
			<xsl:variable name='target_field_name' select="'Subject Relationship Status'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../../entityName"/> (<xsl:value-of select="../reportEntity/entityNumber"/>)</entityName>
                <!--entityName>Subject: <xsl:value-of select="../../entityName"/></entityName-->
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../parentEntityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- 25 status is selected, but not a director, employee, officer or owner -->
        <xsl:if test="(not(statusCode='') and (statusCode)) and (not(subjectRelationTypeH='true' or subjectRelationTypeI='true'
                                                                  or subjectRelationTypeK='true' or subjectRelationTypeL='true'))">
            <xsl:variable name='item_number' select="'25'"/>
            <xsl:variable name='error_code' select="'C56'"/>
			<xsl:variable name='target_field_name' select="'Subject Relationship'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../../entityName"/> (<xsl:value-of select="../reportEntity/entityNumber"/>)</entityName>
                <!--entityName>Subject: <xsl:value-of select="../../entityName"/></entityName-->
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../parentEntityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- 25 status is NOT selected, but this is one of: director, employee, officer or owner -->
        <xsl:if test="((statusCode='') or not(statusCode)) and ((subjectRelationTypeH='true' or subjectRelationTypeI='true'
                                                                  or subjectRelationTypeK='true' or subjectRelationTypeL='true'))">
            <xsl:variable name='item_number' select="'25'"/>
            <xsl:variable name='error_code' select="'C57'"/>
			<xsl:variable name='target_field_name' select="'Subject Relationship Status'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../../entityName"/> (<xsl:value-of select="../reportEntity/entityNumber"/>)</entityName>
                <!--entityName>Subject: <xsl:value-of select="../../entityName"/></entityName-->
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../parentEntityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- 25 Relation Type is one of director, employee, officer or owner AND Joint Report -->
        <!-- globalJointReport is a global variable defined at top since such a higher level -->
        <xsl:if test="($globalJointReport='true')  and (subjectRelationTypeH='true' or subjectRelationTypeI='true'
                                                   or subjectRelationTypeK='true' or subjectRelationTypeL='true')">
            <xsl:variable name='item_number' select="'25'"/>
            <xsl:variable name='error_code' select="'C52'"/>
			<xsl:variable name='target_field_name' select="'Subject Relationship'"/>
			<xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>Subject(Inst): <xsl:value-of select="../../entityName"/> (<xsl:value-of select="../reportEntity/entityNumber"/>)</entityName>
                <!--entityName>Subject: <xsl:value-of select="../../entityName"/></entityName-->
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../parentEntityKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Item 27 Subject Accounts -->
    <xsl:template match="subjectAccount">

        <xsl:variable name='SubjectAcct' select="accountID"/>
        <xsl:if test="contains($SubjectAcct,'.') or contains($SubjectAcct,'-') or contains($SubjectAcct,'/') or contains($SubjectAcct,'\')
                   or contains($SubjectAcct,'=') or contains($SubjectAcct,',') or contains($SubjectAcct,' ') or contains($SubjectAcct,'|')">
            <xsl:variable name='item_number' select="'27d'"/>
            <xsl:variable name='error_code' select="'C24'"/>
            <xsl:variable name='target_field_name' select="'AccountID'"/>
            <xsl:variable name='error_record_type' select="'party'"/>
            <ReportValidation>
                <entityName>LastName: <xsl:value-of select="../lastName"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../entityKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Report level information -->
    <xsl:template match="reportUiJSON">

        <!-- Item 91 LE Contact Phone -->
        <xsl:if test="number(LEContactTelephone ) != LEContactTelephone">
            <xsl:variable name='item_number' select="'91'"/>
            <xsl:variable name='error_code' select="'001'"/>
			<xsl:variable name='target_field_name' select="'LE Contact Phone'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 91a LE Contact Phone Extension -->
        <xsl:if test="number(LEContactTelephoneExt) != LEContactTelephoneExt">
            <xsl:variable name='item_number' select="'91a'"/>
            <xsl:variable name='error_code' select="'001'"/>
			<xsl:variable name='target_field_name' select="'LE Contact Phone Ext'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

         <!-- Item 93 Filing Institution Contact Office -->
         <xsl:if test="(contactOffice='') or not(contactOffice)">
             <xsl:variable name='item_number' select="'93'"/>
             <xsl:variable name='error_code' select="'E39'"/>
			<xsl:variable name='target_field_name' select="'Contact Office'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>   
             <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
             </ReportValidation>
         </xsl:if>

        <!-- Item 94 Filing Institution Contact Phone -->
        <xsl:if test="number(contactPhoneNumber) != contactPhoneNumber">
            <xsl:variable name='item_number' select="'94'"/>
            <xsl:variable name='error_code' select="'E42'"/>
			<xsl:variable name='target_field_name' select="'Contact Phone Number'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="not(contactPhoneNumber)">
            <xsl:variable name='item_number' select="'94'"/>
            <xsl:variable name='error_code' select="'A4'"/>
            <xsl:variable name='target_field_name' select="'Contact Phone Number'"/>
            <xsl:variable name='error_record_type' select="'batch'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 94a Filing Institution Contact Phone Extension -->
        <xsl:if test="number(contactPhoneExtension) != contactPhoneExtension">
            <xsl:variable name='item_number' select="'94a'"/>
            <xsl:variable name='error_code' select="'001'"/>
			<xsl:variable name='target_field_name' select="'Contact Phone Extension'"/>
			<xsl:variable name='error_record_type' select="'filer'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Check the filing type and the DCN Number -->
        <xsl:if test="(((filingTypeCode='A') and not(continuingActivityReportFlag='true')) and (../priorAgencyRefId))">
            <xsl:variable name='item_number' select="'1'"/>
            <xsl:variable name='error_code' select="'B18'"/>
            <xsl:variable name='target_field_name' select="'DCN Number'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../reportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( ((filingTypeCode='B') or (continuingActivityReportFlag='true')) and ( not(../priorAgencyRefId)) )">
            <xsl:variable name='item_number' select="'1'"/>
            <xsl:variable name='error_code' select="'B16'"/>
            <xsl:variable name='target_field_name' select="'DCN Number'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../reportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		
		<!-- Check for Initial filing and Continuing Activity flag for a Warning Message. -->
        <xsl:if test="((filingTypeCode='A') and (continuingActivityReportFlag='true'))">
            <xsl:variable name='item_number' select="'1'"/>
            <xsl:variable name='error_code' select="'B15'"/>
            <xsl:variable name='target_field_name' select="'Report Type'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../reportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

	   <!-- Check for (none selected) filing type and false for ContinuingActivity -->
       <!--xsl:if test="(  (not(filingTypeCode)) and ((not(continuingActivityReportFlag='true'))
	                                                and (not($globalJointReport='true'))) )"-->
        <xsl:if test="( (not(filingTypeCode)) and (not(continuingActivityReportFlag='true')) )">
            <xsl:variable name='item_number' select="'1'"/>
            <xsl:variable name='error_code' select="'B14'"/>
            <xsl:variable name='target_field_name' select="'Report Type'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../reportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>		

        <!-- Item 29 Amount Involved -->
        <xsl:if test="number(involvedAmount) != involvedAmount">
            <xsl:variable name='item_number' select="'29'"/>
            <xsl:variable name='error_code' select="'B20'"/>
			<xsl:variable name='target_field_name' select="'Involved Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 29, look for period, comma, minus characters -->
        <xsl:if test="contains(involvedAmount,'.') or contains(involvedAmount,'-')">
        <xsl:variable name='item_number' select="'29'"/>
            <xsl:variable name='error_code' select="'B20'"/>
			<xsl:variable name='target_field_name' select="'Involved Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 29 SAR Amount missing for Type amount known -->
        <xsl:if test="(((involvedAmount='') or not(involvedAmount)) and (not(involvedAmountUnknownCode) or (involvedAmountUnknownCode='')) )">
            <xsl:variable name='item_number' select="'29'"/>
            <xsl:variable name='error_code' select="'B21'"/>
			<xsl:variable name='target_field_name' select="'Involved Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 29 SAR Amount provided for the non-amount known types -->
        <xsl:if test="( (involvedAmount) and ((involvedAmountUnknownCode='A') or (involvedAmountUnknownCode='B')))">
            <xsl:variable name='item_number' select="'29'"/>
            <xsl:variable name='error_code' select="'B22'"/>
			<xsl:variable name='target_field_name' select="'Involved Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 30 Date range for activity (from) -->
        <xsl:if test="(((SARBeginDate='') or (SARBeginDate='null')) or (not(SARBeginDate)))">
            <xsl:variable name='item_number' select="'30'"/>
            <xsl:variable name='error_code' select="'B23'"/>
			<xsl:variable name='target_field_name' select="'Begin Date'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
			    <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 30 Date From > Date To -->
        <xsl:if test="not (SARBeginDate='') and not (SARBeginEnd='')">
            <xsl:variable name='item_number' select="'30'"/>
            <xsl:variable name='error_code' select="'B25'"/>
			<xsl:variable name='target_field_name' select="'Begin Date'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <xsl:variable name='start_date' select="10000 * substring(SARBeginDate,1,4) + 100 * substring(SARBeginDate,6,2) + substring(SARBeginDate,9,2)"/>
            <xsl:variable name='end_date' select="10000 * substring(SAREndDate,1,4) + 100 * substring(SAREndDate,6,2) + substring(SAREndDate,9,2)"/>
            <xsl:if test="$start_date &gt; $end_date">
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
            </xsl:if>
        </xsl:if>

        <!-- Item 31 SAR Cummulative Amount provided and not continuing  -->
        <xsl:if test="((not(cumulativeAmount='') and (cumulativeAmount)) and (not(continuingActivityReportFlag='true') or not(continuingActivityReportFlag)))">
            <xsl:variable name='item_number' select="'31'"/>
            <xsl:variable name='error_code' select="'B29'"/>
			<xsl:variable name='target_field_name' select="'Cumulative Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 31 Cumulative Amount invalid number  -->
        <xsl:if test="number(cumulativeAmount) != cumulativeAmount">
            <xsl:variable name='item_number' select="'31'"/>
            <xsl:variable name='error_code' select="'B27'"/>
			<xsl:variable name='target_field_name' select="'Cumulative Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Item 31, look for period, comma, minus characters -->
        <xsl:if test="contains(cumulativeAmount,'.') or contains(cumulativeAmount,'-')">
            <xsl:variable name='item_number' select="'31'"/>
            <xsl:variable name='error_code' select="'B27'"/>
			<xsl:variable name='target_field_name' select="'Cumulative Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 31 SAR Continuing provided and not the Cumulative Amount  -->
        <xsl:if test="((continuingActivityReportFlag='true') and (continuingActivityReportFlag)) and (not(cumulativeAmount) or (cumulativeAmount=''))">
            <xsl:variable name='item_number' select="'31'"/>
            <xsl:variable name='error_code' select="'B28'"/>
			<xsl:variable name='target_field_name' select="'Cumulative Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- 31 Cumulative Amount is less than 29 SAR Involved Amount -->
        <xsl:if test="(cumulativeAmount &lt; involvedAmount)">
            <xsl:variable name='item_number' select="'31'"/>
            <xsl:variable name='error_code' select="'B30'"/>
			<xsl:variable name='target_field_name' select="'Cumulative Amount'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Items 32-42 Other Text -->
        <!-- Item 32 Structuring -->
        <xsl:if test="((SARStructuredTypeZ='true') and (not (SARStructuredOther) or (SARStructuredOther='')))">
            <xsl:variable name='item_number' select="'32'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Structuring'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARStructuredOther) and not (SARStructuredTypeZ='true')">
            <xsl:variable name='item_number' select="'32'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Structuring Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/>   
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 33 Terrorist Financing -->
        <xsl:if test="((SARTerroristTypeZ='true') and (not (SARTerroristTypeOther) or (SARTerroristTypeOther='')))">
            <xsl:variable name='item_number' select="'33'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Terrorist Financing'"/>
			<xsl:variable name='error_record_type' select="'report'"/>  
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARTerroristTypeOther) and not (SARTerroristTypeZ='true')">
            <xsl:variable name='item_number' select="'33'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Terrorist Financing Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/>  
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 34 Fraud -->
        <xsl:if test="((SARFraudTypeZ='true') and (not (SARFraudTypeOther) or (SARFraudTypeOther='')))">
            <xsl:variable name='item_number' select="'34'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Fraud'"/>
			<xsl:variable name='error_record_type' select="'report'"/>  
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARFraudTypeOther) and not (SARFraudTypeZ='true')">
            <xsl:variable name='item_number' select="'34'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Fraud Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 35 Gaming Activities -->
        <xsl:if test="((SARCasinosTypeZ='true') and (not (SARCasinosTypeOther) or (SARCasinosTypeOther='')))">
            <xsl:variable name='item_number' select="'35'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Gaming Activities'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARCasinosTypeOther) and not (SARCasinosTypeZ='true')">
            <xsl:variable name='item_number' select="'35'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Gaming Activities Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 36 Money Laundering -->
        <xsl:if test="((SARMoneyLaundryTypeZ='true') and (not (SARMoneyLaundryTypeOther) or (SARMoneyLaundryTypeOther='')))">
            <xsl:variable name='item_number' select="'36'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Money Laundering'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARMoneyLaundryTypeOther) and not (SARMoneyLaundryTypeZ='true')">
            <xsl:variable name='item_number' select="'36'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Money Laundering Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 37 Identification/Documentation Type -->
        <xsl:if test="((SARIDTypeZ='true') and (not (SARIDTypeOther) or (SARIDTypeOther='')))">
            <xsl:variable name='item_number' select="'37'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Identification/Documentation'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARIDTypeOther) and not (SARIDTypeZ='true')">
            <xsl:variable name='item_number' select="'37'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Identification/Documentation Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 38 Other Activity -->
        <xsl:if test="((SAROtherTypeZ='true') and (not (SAROtherTypeOther) or (SAROtherTypeOther='')))">
            <xsl:variable name='item_number' select="'38'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Other Suspicious Activities'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SAROtherTypeOther) and not (SAROtherTypeZ='true')">
            <xsl:variable name='item_number' select="'38'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Other Suspicious Activities Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 39 Insurance -->
        <xsl:if test="((SARInsuranceTypeZ='true') and (not (SARInsuranceTypeOther) or (SARInsuranceTypeOther='')))">
            <xsl:variable name='item_number' select="'39'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Insurance'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARInsuranceTypeOther) and not (SARInsuranceTypeZ='true')">
            <xsl:variable name='item_number' select="'39'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Insurance Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 40 Securities/Futures -->
        <xsl:if test="((SARSFTypeZ='true') and (not (SARSFTypeOther) or (SARSFTypeOther='')))">
            <xsl:variable name='item_number' select="'40'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Securities/Futures'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARSFTypeOther) and not (SARSFTypeZ='true')">
            <xsl:variable name='item_number' select="'40'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Securities/Futures Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 41 Mortgage Fraud -->
        <xsl:if test="((SARMortgageFraudTypeZ='true') and (not (SARMortgageFraudTypeOther) or (SARMortgageFraudTypeOther='')))">
            <xsl:variable name='item_number' select="'41'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Mortgage Fraud'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARMortgageFraudTypeOther) and not (SARMortgageFraudTypeZ='true')">
            <xsl:variable name='item_number' select="'41'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Mortgage Fraud Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 42 Cyber Event -->
        <xsl:if test="((SARCyberTypeZ='true') and (not (SARCyberTypeOther) or (SARCyberTypeOther='')))">
            <xsl:variable name='item_number' select="'42'"/>
            <xsl:variable name='error_code' select="'B40'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARCyberTypeOther) and not (SARCyberTypeZ='true')">
            <xsl:variable name='item_number' select="'42'"/>
            <xsl:variable name='error_code' select="'B41'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 45 Products (other) -->
        <xsl:if test="((SARProductTypeZ='true') and (not (SARProductTypeOther) or (SARProductTypeOther='')))">
            <xsl:variable name='item_number' select="'45'"/>
            <xsl:variable name='error_code' select="'B62'"/>
			<xsl:variable name='target_field_name' select="'Product Types Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARProductTypeOther) and not (SARProductTypeZ='true')">
            <xsl:variable name='item_number' select="'45'"/>
            <xsl:variable name='error_code' select="'B63'"/>
			<xsl:variable name='target_field_name' select="'Product Types Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 46 Instruments (other) -->
        <xsl:if test="((SARInstrumentTypeZ='true') and (not (SARInstrumentTypeOther) or (SARInstrumentTypeOther='')))">
            <xsl:variable name='item_number' select="'46'"/>
            <xsl:variable name='error_code' select="'B62'"/>
			<xsl:variable name='target_field_name' select="'Instruments/Payment Mechanisms Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(SARInstrumentTypeOther) and not (SARInstrumentTypeZ='true')">
            <xsl:variable name='item_number' select="'46'"/>
            <xsl:variable name='error_code' select="'B63'"/>
			<xsl:variable name='target_field_name' select="'Instruments/Payment Mechanisms Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Must Select at least one Suspicious Activity item from 32-42 -->
        <xsl:variable name="count32" select="count(*[contains(local-name(),'SARStructuredType')][text()='true'])"/>
        <xsl:variable name="count33" select="count(*[contains(local-name(),'SARTerroristType')][text()='true'])"/>
        <xsl:variable name="count34" select="count(*[contains(local-name(),'SARFraudType')][text()='true'])"/>
        <xsl:variable name="count35" select="count(*[contains(local-name(),'SARCasinosType')][text()='true'])"/>
        <xsl:variable name="count36" select="count(*[contains(local-name(),'SARMoneyLaundryType')][text()='true'])"/>
        <xsl:variable name="count37" select="count(*[contains(local-name(),'SARIDType')][text()='true'])"/>
        <xsl:variable name="count38" select="count(*[contains(local-name(),'SAROtherType')][text()='true'])"/>
        <xsl:variable name="count39" select="count(*[contains(local-name(),'SARInsuranceType')][text()='true'])"/>
        <xsl:variable name="count40" select="count(*[contains(local-name(),'SARSFType')][text()='true'])"/>
        <xsl:variable name="count41" select="count(*[contains(local-name(),'SARMortgageFraudType')][text()='true'])"/>
        <xsl:variable name="count42" select="count(*[contains(local-name(),'SARCyberType')][text()='true'])"/>
        <xsl:variable name="count_all_choices" select="$count32 + $count33 + $count34 + $count35 + $count36 + $count37 + $count38 +
                                                           $count39 + $count40 + $count41 + $count42"/>
        <xsl:if test="($count_all_choices &lt; 1)">
            <xsl:variable name='item_number' select="'32-42'"/>
            <xsl:variable name='error_code' select="'B31'"/>
			<xsl:variable name='target_field_name' select="'Suspicious Activities'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Report: <xsl:value-of select="../rootReportKey"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 43 IP Addresses at Report Level -->
        <xsl:apply-templates select="IPAddress"/>
        <!-- Item 44 Cyber Events at Report Level -->
        <xsl:apply-templates select="cyberEvent"/>

    </xsl:template>

    <!-- Item 43 IP Addresses -->
    <xsl:template match="IPAddress">

        <!-- IP Address Format Validation. IPAddress node, warning message B42 -->
        <xsl:variable name='IPValue' select="IPAddress"/>
        <xsl:variable name='countColons' select="string-length($IPValue) - (string-length(translate($IPValue, ':', '')))"/>
        <xsl:variable name='countPeriods' select="string-length($IPValue) - (string-length(translate($IPValue, '.', '')))"/>
        <xsl:variable name='IPLength' select="string-length($IPValue)"/>

        <xsl:if test="( (($countPeriods &gt; 0) and (IPAddress and not(IPAddress='')))
                    and ( (($countPeriods &lt; 3) or ($countPeriods &gt; 3)) or ($IPLength &gt; 15 or $IPLength &lt; 7)) )">
            <xsl:variable name='item_number' select="'43'"/>
            <xsl:variable name='error_code' select="'B42'"/>
			<xsl:variable name='target_field_name' select="'IP Address'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>IP Address: <xsl:value-of select="IPAddress"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( (($countColons &gt; 0) and (IPAddress and not(IPAddress='')))
                    and ( (($countColons &lt; 7) or ($countColons &gt; 7)) or ($IPLength &gt; 39 or $IPLength &lt; 39)) )">
            <xsl:variable name='item_number' select="'43'"/>
            <xsl:variable name='error_code' select="'B42'"/>
			<xsl:variable name='target_field_name' select="'IP Address'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>IP Address: <xsl:value-of select="IPAddress"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( IPAddress and (($IPLength &lt; 7) or ($countColons=0 and $countPeriods=0)) )">
            <xsl:variable name='item_number' select="'43'"/>
            <xsl:variable name='error_code' select="'B42'"/>
            <xsl:variable name='target_field_name' select="'IP Address'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>IP Address: <xsl:value-of select="IPAddress"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="(IPAddressUTCDate or IPAddressUTCTime) and not(IPAddress)">
            <xsl:variable name='item_number' select="'43'"/>
            <xsl:variable name='error_code' select="'B42'"/>
            <xsl:variable name='target_field_name' select="'IP Address'"/>
            <xsl:variable name='error_record_type' select="'report'"/>
            <ReportValidation>
                <entityName>IP Address: <xsl:value-of select="IPAddress"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
                <itemNumber><xsl:value-of select="$item_number"/></itemNumber>
                <targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
                <errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Item 44 Cyber events -->
    <xsl:template match="cyberEvent">

        <!-- Date and Time ONLY valid if Type is for IP address -->
        <xsl:if test="((not(cyberEventUTCDate='') and (cyberEventUTCDate)) and (not(cyberEventTypeCode='1' or cyberEventTypeCode='8')))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B47'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Date'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Date and Time ONLY valid if Type is for IP address -->
        <xsl:if test="((not(cyberEventUTCTime='') and (cyberEventUTCTime)) and (not(cyberEventTypeCode='1' or cyberEventTypeCode='8')))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B49'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Timestamp'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Event Value exists but no Type Code -->
        <xsl:if test="((not(cyberEventValue='') and (cyberEventValue)) and ((cyberEventTypeCode='' or not(cyberEventTypeCode))))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B51'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Type'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Event Code exists, but no Event Value -->
        <xsl:if test="((not(cyberEventTypeCode='') and (cyberEventTypeCode)) and ((cyberEventValue='' or not(cyberEventValue))))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B45'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Code: <xsl:value-of select="cyberEventTypeCode"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		<!-- Cyber Event Other codes -->
		<xsl:if test="( (cyberEventTypeCode='999') and ((cyberEventValue='') or (not(cyberEventValue))) )">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B53'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Code: <xsl:value-of select="cyberEventTypeCode"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
		<xsl:if test="( (not(cyberEventTypeCode='999')) and (cyberEventTypeOther) )">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B54'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Other'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Code: <xsl:value-of select="cyberEventTypeCode"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

        <!-- Item 44 Cyber Event value validation based on Cyber Event Code -->
        <!-- cyberEventTypeCode and cyberEventValue.  Warning message B46        -->

        <!-- Type Code 2 and 9: URL address, only validation it must contain at least 1 period and no space. -->
        <xsl:variable name='eventValue' select="cyberEventValue"/>
        <xsl:variable name='periodLocation' select="string-length(substring-before($eventValue,'.'))"/>
        <xsl:if test="(cyberEventTypeCode='2' or cyberEventTypeCode='9') and (not(cyberEventValue='') and (($periodLocation &lt; 1) or (contains(cyberEventValue,' '))))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Type Code 5: Port, validation expects max length of 6 and all digits -->
        <xsl:variable name='portValue' select="cyberEventValue"/>
        <xsl:variable name='portLength' select="string-length($portValue)"/>
        <xsl:if test="(cyberEventTypeCode='5') and ( not(cyberEventValue='') and (($portLength &gt; 6) or (number(cyberEventValue) != cyberEventValue)) )">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Type Code 6: email address, validation expects one @ sign and period after @ sign -->
        <xsl:variable name='partyEmail' select="cyberEventValue"/>
        <xsl:variable name='atSign' select="string-length(substring-before($partyEmail,'@'))"/>
        <xsl:variable name='stringAfter' select="substring-after($partyEmail,'@')"/>
        <xsl:variable name='periodAfter' select="string-length(substring-before($stringAfter,'.'))"/>
        <xsl:if test="(cyberEventTypeCode='6') and (not(cyberEventValue='') and (($atSign &lt; 1) or ($periodAfter &lt; 1)))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Type Code 4 for MAC address; format XX:XX:XX:XX:XX:XX -->
        <xsl:variable name='MACValue' select="cyberEventValue"/>
        <xsl:variable name='countColons' select="string-length($MACValue) - (string-length(translate($MACValue, ':', '')))"/>
        <xsl:if test="(cyberEventTypeCode='4') and (not(cyberEventValue='') and (($countColons &lt; 5) or ($countColons &gt; 5)))">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <!-- Type Code 1 or 8 for IP address -->
        <xsl:variable name='IPValue' select="cyberEventValue"/>
        <xsl:variable name='Colons' select="string-length($IPValue) - (string-length(translate($IPValue, ':', '')))"/>
        <xsl:variable name='Periods' select="string-length($IPValue) - (string-length(translate($IPValue, '.', '')))"/>
        <xsl:variable name='IPLength' select="string-length($IPValue)"/>
        <xsl:if test="( ( (cyberEventTypeCode='1' or cyberEventTypeCode='8') and ($Periods &gt; 0) and (cyberEventValue and not(cyberEventValue='')))
                    and ( (($Periods &lt; 3) or ($Periods &gt; 3)) or ($IPLength &gt; 15 or $IPLength &lt; 7)) )">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>
        <xsl:if test="( ( (cyberEventTypeCode='1' or cyberEventTypeCode='8') and ($Colons &gt; 0) and (cyberEventValue and not(cyberEventValue='')))
                    and ( (($Colons &lt; 7) or ($Colons &gt; 7)) or ($IPLength &gt; 39 or $IPLength &lt; 39)) )">
            <xsl:variable name='item_number' select="'44'"/>
            <xsl:variable name='error_code' select="'B46'"/>
			<xsl:variable name='target_field_name' select="'Cyber Event Value'"/>
			<xsl:variable name='error_record_type' select="'report'"/> 
            <ReportValidation>
                <entityName>Cyber Event Value: <xsl:value-of select="cyberEventValue"/></entityName>
                <errorCode><xsl:value-of select="$error_code"/></errorCode>
                <entityKey><xsl:value-of select="../../reportKey"/></entityKey>
				<itemNumber><xsl:value-of select="$item_number"/></itemNumber>
				<targetFieldName><xsl:value-of select="$target_field_name"/></targetFieldName>
				<errorRecordType><xsl:value-of select="$error_record_type"/></errorRecordType>
            </ReportValidation>
        </xsl:if>

    </xsl:template>

    <!-- Custom templates to be used to determine leading and trailing white space for fields. -->
    <!-- XLST 1.0 is limited in functions for this capability.                                 -->
    <xsl:variable name="whitespace" select="'&#09;&#10;&#13; '" />
    <!-- Strips trailing whitespace characters from 'string' -->
    <xsl:template name="string-rtrim">
        <xsl:param name="string" />
        <xsl:param name="trim" select="$whitespace" />

        <xsl:variable name="length" select="string-length($string)" />

        <xsl:if test="$length &gt; 0">
            <xsl:choose>
                <xsl:when test="contains($trim, substring($string, $length, 1))">
                    <xsl:call-template name="string-rtrim">
                        <xsl:with-param name="string" select="substring($string, 1, $length - 1)" />
                        <xsl:with-param name="trim"   select="$trim" />
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$string" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <!-- Strips leading whitespace characters from 'string' -->
    <xsl:template name="string-ltrim">
        <xsl:param name="string" />
        <xsl:param name="trim" select="$whitespace" />

        <xsl:if test="string-length($string) &gt; 0">
            <xsl:choose>
                <xsl:when test="contains($trim, substring($string, 1, 1))">
                    <xsl:call-template name="string-ltrim">
                        <xsl:with-param name="string" select="substring($string, 2)" />
                        <xsl:with-param name="trim"   select="$trim" />
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$string" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <!-- Strips leading and trailing whitespace characters from 'string' -->
    <xsl:template name="string-trim">
        <xsl:param name="string" />
        <xsl:param name="trim" select="$whitespace" />
        <xsl:call-template name="string-rtrim">
            <xsl:with-param name="string">
                <xsl:call-template name="string-ltrim">
                    <xsl:with-param name="string" select="$string" />
                    <xsl:with-param name="trim"   select="$trim" />
                </xsl:call-template>
            </xsl:with-param>
            <xsl:with-param name="trim"   select="$trim" />
        </xsl:call-template>
    </xsl:template>

</xsl:stylesheet>